@@ -23,18 +23,19 @@
 **
 ****************************************************************************/
 
-#ifndef PLUGINSPEC_H
-#define PLUGINSPEC_H
+#pragma once
 
 #include "extensionsystem_global.h"
 
-#include <QString>
+#include <utils/porting.h>
+
 #include <QHash>
+#include <QStaticPlugin>
+#include <QString>
 #include <QVector>
 
 QT_BEGIN_NAMESPACE
-class QStringList;
-class QRegExp;
+class QRegularExpression;
 QT_END_NAMESPACE
 
 namespace ExtensionSystem {
@@ -60,14 +61,15 @@ struct EXTENSIONSYSTEM_EXPORT PluginDependency
 
     PluginDependency() : type(Required) {}
 
+    friend Utils::QHashValueType qHash(const PluginDependency &value);
+
     QString name;
     QString version;
     Type type;
     bool operator==(const PluginDependency &other) const;
+    QString toString() const;
 };
 
-uint qHash(const ExtensionSystem::PluginDependency &value);
-
 struct EXTENSIONSYSTEM_EXPORT PluginArgumentDescription
 {
     QString name;
@@ -92,7 +94,8 @@ public:
     QString description() const;
     QString url() const;
     QString category() const;
-    QRegExp platformSpecification() const;
+    QString revision() const;
+    QRegularExpression platformSpecification() const;
     bool isAvailableForHostPlatform() const;
     bool isRequired() const;
     bool isExperimental() const;
@@ -103,8 +106,9 @@ public:
     bool isForceEnabled() const;
     bool isForceDisabled() const;
     QVector<PluginDependency> dependencies() const;
+    QJsonObject metaData() const;
 
-    typedef QVector<PluginArgumentDescription> PluginArgumentDescriptions;
+    using PluginArgumentDescriptions = QVector<PluginArgumentDescription>;
     PluginArgumentDescriptions argumentDescriptions() const;
 
     // other information, valid after 'Read' state is reached
@@ -119,6 +123,7 @@ public:
 
     // dependency specs, valid after 'Resolved' state is reached
     QHash<PluginDependency, PluginSpec *> dependencySpecs() const;
+    bool requiresAny(const QSet<PluginSpec *> &plugins) const;
 
     // linked plugin instance, valid after 'Loaded' state is reached
     IPlugin *plugin() const;
@@ -128,6 +133,11 @@ public:
     bool hasError() const;
     QString errorString() const;
 
+    void setEnabledBySettings(bool value);
+
+    static PluginSpec *read(const QString &filePath);
+    static PluginSpec *read(const QStaticPlugin &plugin);
+
 private:
     PluginSpec();
 
@@ -139,6 +149,3 @@ private:
 };
 
 } // namespace ExtensionSystem
-
-#endif // PLUGINSPEC_H
-
