@@ -133,8 +133,8 @@ ModelManagerInterface::ModelManagerInterface(QObject *parent)
     qRegisterMetaType<QmlJS::PathAndLanguage>("QmlJS::PathAndLanguage");
     qRegisterMetaType<QmlJS::PathsAndLanguages>("QmlJS::PathsAndLanguages");
 
-    m_defaultProjectInfo.qtQmlPath = QFileInfo(
-                QLibraryInfo::location(QLibraryInfo::Qml2ImportsPath)).canonicalFilePath();
+    m_defaultProjectInfo.qtQmlPath =
+            FilePath::fromUserInput(QLibraryInfo::location(QLibraryInfo::Qml2ImportsPath));
     m_defaultProjectInfo.qtVersionString = QLibraryInfo::version().toString();
 
     updateImportPaths();
@@ -165,6 +165,7 @@ ModelManagerInterface::~ModelManagerInterface()
 static QHash<QString, Dialect> defaultLanguageMapping()
 {
     static QHash<QString, Dialect> res{
+        {QLatin1String("mjs"), Dialect::JavaScript},
         {QLatin1String("js"), Dialect::JavaScript},
         {QLatin1String("qml"), Dialect::Qml},
         {QLatin1String("qmltypes"), Dialect::QmlTypeInfo},
@@ -445,6 +446,20 @@ bool pInfoLessThanImports(const ModelManagerInterface::ProjectInfo &p1,
 
 }
 
+static QList<Utils::FilePath> generatedQrc(QStringList applicationDirectories)
+{
+    QList<Utils::FilePath> res;
+    for (const QString &pathStr : applicationDirectories) {
+        Utils::FilePath path = Utils::FilePath::fromString(pathStr);
+        Utils::FilePath generatedQrcDir = path.pathAppended(".rcc");
+        if (generatedQrcDir.isReadableDir()) {
+            for (const Utils::FilePath & qrcPath: generatedQrcDir.dirEntries(FileFilter(QStringList({QStringLiteral(u"*.qrc")}), QDir::Files)))
+                res.append(qrcPath.canonicalPath());
+        }
+    }
+    return res;
+}
+
 void ModelManagerInterface::iterateQrcFiles(
         ProjectExplorer::Project *project, QrcResourceSelector resources,
         const std::function<void(QrcParser::ConstPtr)> &callback)
@@ -460,18 +475,21 @@ void ModelManagerInterface::iterateQrcFiles(
             Utils::sort(pInfos, &pInfoLessThanAll);
     }
 
-    QSet<QString> pathsChecked;
+    QSet<Utils::FilePath> pathsChecked;
     for (const ModelManagerInterface::ProjectInfo &pInfo : qAsConst(pInfos)) {
         QStringList qrcFilePaths;
         if (resources == ActiveQrcResources)
             qrcFilePaths = pInfo.activeResourceFiles;
         else
             qrcFilePaths = pInfo.allResourceFiles;
-        for (const QString &qrcFilePath : qAsConst(qrcFilePaths)) {
+        for (const Utils::FilePath &p : generatedQrc(pInfo.applicationDirectories))
+            qrcFilePaths.append(p.toString());
+        for (const QString &qrcFilePathStr : qAsConst(qrcFilePaths)) {
+            auto qrcFilePath = Utils::FilePath::fromString(qrcFilePathStr);
             if (pathsChecked.contains(qrcFilePath))
                 continue;
             pathsChecked.insert(qrcFilePath);
-            QrcParser::ConstPtr qrcFile = m_qrcCache.parsedPath(qrcFilePath);
+            QrcParser::ConstPtr qrcFile = m_qrcCache.parsedPath(qrcFilePath.toString());
             if (qrcFile.isNull())
                 continue;
             callback(qrcFile);
@@ -588,6 +606,8 @@ void ModelManagerInterface::updateProjectInfo(const ProjectInfo &pinfo, ProjectE
     m_qrcContents = pinfo.resourceFileContents;
     for (const QString &newQrc : qAsConst(pinfo.allResourceFiles))
         m_qrcCache.addPath(newQrc, m_qrcContents.value(newQrc));
+    for (const Utils::FilePath &newQrc : generatedQrc(pinfo.applicationDirectories))
+        m_qrcCache.addPath(newQrc.toString(), m_qrcContents.value(newQrc.toString()));
     for (const QString &oldQrc : qAsConst(oldInfo.allResourceFiles))
         m_qrcCache.removePath(oldQrc);
 
@@ -684,19 +704,19 @@ void ModelManagerInterface::updateDocument(const Document::Ptr &doc)
     emit documentUpdated(doc);
 }
 
-void ModelManagerInterface::updateLibraryInfo(const QString &path, const LibraryInfo &info)
+void ModelManagerInterface::updateLibraryInfo(const FilePath &path, const LibraryInfo &info)
 {
     if (!info.pluginTypeInfoError().isEmpty())
         qCDebug(qmljsLog) << "Dumping errors for " << path << ":" << info.pluginTypeInfoError();
 
     {
         QMutexLocker locker(&m_mutex);
-        m_validSnapshot.insertLibraryInfo(path, info);
-        m_newestSnapshot.insertLibraryInfo(path, info);
+        m_validSnapshot.insertLibraryInfo(path.toString(), info);
+        m_newestSnapshot.insertLibraryInfo(path.toString(), info);
     }
     // only emit if we got new useful information
     if (info.isValid())
-        emit libraryInfoUpdated(path, info);
+        emit libraryInfoUpdated(path.toString(), info);
 }
 
 static QStringList filesInDirectoryForLanguages(const QString &path,
@@ -773,7 +793,7 @@ enum class LibraryStatus {
     Unknown
 };
 
-static LibraryStatus libraryStatus(const QString &path, const Snapshot &snapshot,
+static LibraryStatus libraryStatus(const FilePath &path, const Snapshot &snapshot,
                                    QSet<QString> *newLibraries)
 {
     if (path.isEmpty())
@@ -782,7 +802,7 @@ static LibraryStatus libraryStatus(const QString &path, const Snapshot &snapshot
     const LibraryInfo &existingInfo = snapshot.libraryInfo(path);
     if (existingInfo.isValid())
         return LibraryStatus::Accepted;
-    if (newLibraries->contains(path))
+    if (newLibraries->contains(path.toString()))
         return LibraryStatus::Accepted;
     // if we looked at the path before, done
     return existingInfo.wasScanned()
@@ -790,7 +810,7 @@ static LibraryStatus libraryStatus(const QString &path, const Snapshot &snapshot
             : LibraryStatus::Unknown;
 }
 
-static bool findNewQmlApplicationInPath(const QString &path,
+static bool findNewQmlApplicationInPath(const FilePath &path,
                                         const Snapshot &snapshot,
                                         ModelManagerInterface *modelManager,
                                         QSet<QString> *newLibraries)
@@ -803,8 +823,8 @@ static bool findNewQmlApplicationInPath(const QString &path,
 
     QString qmltypesFile;
 
-    QDir dir(path);
-    QDirIterator it(path, QStringList { "*.qmltypes" }, QDir::Files);
+    QDir dir(path.toString());
+    QDirIterator it(path.toString(), QStringList { "*.qmltypes" }, QDir::Files);
 
     if (!it.hasNext())
         return false;
@@ -828,7 +848,7 @@ static bool findNewQmlLibraryInPath(const QString &path,
                                     QSet<QString> *newLibraries,
                                     bool ignoreMissing)
 {
-    switch (libraryStatus(path, snapshot, newLibraries)) {
+    switch (libraryStatus(FilePath::fromString(path), snapshot, newLibraries)) {
     case LibraryStatus::Accepted: return true;
     case LibraryStatus::Rejected: return false;
     default: break;
@@ -839,7 +859,7 @@ static bool findNewQmlLibraryInPath(const QString &path,
     if (!qmldirFile.exists()) {
         if (!ignoreMissing) {
             LibraryInfo libraryInfo(LibraryInfo::NotFound);
-            modelManager->updateLibraryInfo(path, libraryInfo);
+            modelManager->updateLibraryInfo(FilePath::fromString(path), libraryInfo);
         }
         return false;
     }
@@ -858,7 +878,7 @@ static bool findNewQmlLibraryInPath(const QString &path,
 
     const QString libraryPath = QFileInfo(qmldirFile).absolutePath();
     newLibraries->insert(libraryPath);
-    modelManager->updateLibraryInfo(libraryPath, LibraryInfo(qmldirParser));
+    modelManager->updateLibraryInfo(FilePath::fromString(libraryPath), LibraryInfo(qmldirParser));
     modelManager->loadPluginTypes(QFileInfo(libraryPath).canonicalFilePath(), libraryPath,
                 QString(), QString());
 
@@ -968,7 +988,8 @@ void ModelManagerInterface::parseLoop(QSet<QString> &scannedPaths,
         doc->parse();
 
 #ifdef WITH_TESTS
-        if (ExtensionSystem::PluginManager::isScenarioRunning("TestModelManagerInterface")) {
+        if (ExtensionSystem::PluginManager::instance() // we might run as an auto-test
+            && ExtensionSystem::PluginManager::isScenarioRunning("TestModelManagerInterface")) {
             ExtensionSystem::PluginManager::waitForScenarioFullyInitialized();
             if (ExtensionSystem::PluginManager::finishScenario()) {
                 qDebug() << "Point 1: Shutdown triggered";
@@ -1178,6 +1199,29 @@ void ModelManagerInterface::maybeScan(const PathsAndLanguages &importPaths)
     }
 }
 
+static QList<Utils::FilePath> minimalPrefixPaths(const QStringList &paths)
+{
+    QList<Utils::FilePath> sortedPaths;
+    // find minimal prefix, ensure '/' at end
+    for (const QString &pathStr : qAsConst(paths)) {
+        Utils::FilePath path = Utils::FilePath::fromString(pathStr);
+        if (!path.endsWith("/"))
+            path.setPath(path.path() + "/");
+        if (path.path().length() > 1)
+            sortedPaths.append(path);
+    }
+    std::sort(sortedPaths.begin(), sortedPaths.end());
+    QList<Utils::FilePath> res;
+    QString lastPrefix;
+    for (auto it = sortedPaths.begin(); it != sortedPaths.end(); ++it) {
+        if (lastPrefix.isEmpty() || !it->startsWith(lastPrefix)) {
+            lastPrefix = it->path();
+            res.append(*it);
+        }
+    }
+    return res;
+}
+
 void ModelManagerInterface::updateImportPaths()
 {
     if (m_indexerDisabled)
@@ -1217,16 +1261,14 @@ void ModelManagerInterface::updateImportPaths()
     }
 
     for (const ProjectInfo &pInfo : qAsConst(m_projects)) {
-        if (!pInfo.qtQmlPath.isEmpty()) {
-            allImportPaths.maybeInsert(Utils::FilePath::fromString(pInfo.qtQmlPath),
-                                       Dialect::QmlQtQuick2);
-        }
+        if (!pInfo.qtQmlPath.isEmpty())
+            allImportPaths.maybeInsert(pInfo.qtQmlPath, Dialect::QmlQtQuick2);
     }
 
     {
-        const QString pathAtt = defaultProjectInfo().qtQmlPath;
+        const FilePath pathAtt = defaultProjectInfo().qtQmlPath;
         if (!pathAtt.isEmpty())
-            allImportPaths.maybeInsert(Utils::FilePath::fromString(pathAtt), Dialect::QmlQtQuick2);
+            allImportPaths.maybeInsert(pathAtt, Dialect::QmlQtQuick2);
     }
 
     for (const auto &importPath : defaultProjectInfo().importPaths) {
@@ -1243,6 +1285,7 @@ void ModelManagerInterface::updateImportPaths()
         m_allImportPaths = allImportPaths;
         m_activeBundles = activeBundles;
         m_extendedBundles = extendedBundles;
+        m_applicationPaths = minimalPrefixPaths(allApplicationDirectories);
     }
 
 
@@ -1253,8 +1296,13 @@ void ModelManagerInterface::updateImportPaths()
     QSet<QString> newLibraries;
     for (const Document::Ptr &doc : qAsConst(snapshot))
         findNewLibraryImports(doc, snapshot, this, &importedFiles, &scannedPaths, &newLibraries);
-    for (const QString &path : qAsConst(allApplicationDirectories))
+    for (const QString &pathStr : qAsConst(allApplicationDirectories)) {
+        Utils::FilePath path = Utils::FilePath::fromString(pathStr);
+        allImportPaths.maybeInsert(path, Dialect::Qml);
         findNewQmlApplicationInPath(path, snapshot, this, &newLibraries);
+    }
+    for (const Utils::FilePath &qrcPath : generatedQrc(allApplicationDirectories))
+        updateQrcFile(qrcPath.toString());
 
     updateSourceFiles(importedFiles, true);
 
@@ -1329,7 +1377,7 @@ bool rescanExports(const QString &fileName, FindExportedCppTypes &finder,
     QList<LanguageUtils::FakeMetaObject::ConstPtr> exported = finder.exportedTypes();
     QHash<QString, QString> contextProperties = finder.contextProperties();
     if (exported.isEmpty() && contextProperties.isEmpty()) {
-        hasNewInfo = hasNewInfo || newData.remove(fileName) > 0;
+        hasNewInfo = hasNewInfo || newData.remove(fileName);
     } else {
         ModelManagerInterface::CppData &data = newData[fileName];
         if (!hasNewInfo && (data.exportedTypes.size() != exported.size()
@@ -1382,7 +1430,7 @@ void ModelManagerInterface::updateCppQmlTypes(
         const bool scan = pair.second;
         const QString fileName = doc->fileName();
         if (!scan) {
-            hasNewInfo = newData.remove(fileName) > 0 || hasNewInfo;
+            hasNewInfo = newData.remove(fileName) || hasNewInfo;
             const auto savedDocs = newDeclarations.value(fileName);
             for (const CPlusPlus::Document::Ptr &savedDoc : savedDocs) {
                 finder(savedDoc);
@@ -1483,13 +1531,13 @@ ViewerContext ModelManagerInterface::getVContext(const ViewerContext &vCtx,
         switch (res.language.dialect()) {
         case Dialect::AnyLanguage:
         case Dialect::Qml:
-            maybeAddPath(res, info.qtQmlPath);
+            maybeAddPath(res, info.qtQmlPath.toString());
             Q_FALLTHROUGH();
         case Dialect::QmlQtQuick2:
         case Dialect::QmlQtQuick2Ui:
         {
             if (res.language == Dialect::QmlQtQuick2 || res.language == Dialect::QmlQtQuick2Ui)
-                maybeAddPath(res, info.qtQmlPath);
+                maybeAddPath(res, info.qtQmlPath.toString());
 
             QList<Dialect> languages = res.language.companionLanguages();
             auto addPathsOnLanguageMatch = [&](const PathsAndLanguages &importPaths) {
@@ -1534,7 +1582,7 @@ ViewerContext ModelManagerInterface::getVContext(const ViewerContext &vCtx,
         for (const QString &path : qAsConst(defaultVCtx.paths))
             maybeAddPath(res, path);
         if (res.language == Dialect::AnyLanguage || res.language == Dialect::Qml)
-            maybeAddPath(res, info.qtQmlPath);
+            maybeAddPath(res, info.qtQmlPath.toString());
         if (res.language == Dialect::AnyLanguage || res.language == Dialect::Qml
                 || res.language == Dialect::QmlQtQuick2 || res.language == Dialect::QmlQtQuick2Ui) {
             const auto environemntPaths = environmentImportPaths();
@@ -1666,4 +1714,47 @@ void ModelManagerInterface::resetCodeModel()
     updateImportPaths();
 }
 
+Utils::FilePath ModelManagerInterface::fileToSource(const Utils::FilePath &path)
+{
+    if (!path.scheme().isEmpty())
+        return path;
+    for (const Utils::FilePath &p : m_applicationPaths) {
+        if (!p.isEmpty() && path.startsWith(p.path())) {
+            // if it is an applicationPath (i.e. in the build directory)
+            // try to use the path from the build dir as resource path
+            // and recover the path of the corresponding source file
+            QString reducedPath = path.path().mid(p.path().size());
+            QString reversePath(reducedPath);
+            std::reverse(reversePath.begin(), reversePath.end());
+            if (!reversePath.endsWith('/'))
+                reversePath.append('/');
+            QrcParser::MatchResult res;
+            iterateQrcFiles(nullptr,
+                            QrcResourceSelector::AllQrcResources,
+                            [&](const QrcParser::ConstPtr &qrcFile) {
+                                if (!qrcFile)
+                                    return;
+                                QrcParser::MatchResult matchNow = qrcFile->longestReverseMatches(
+                                    reversePath);
+
+                                if (matchNow.matchDepth < res.matchDepth)
+                                    return;
+                                if (matchNow.matchDepth == res.matchDepth) {
+                                    res.reversedPaths += matchNow.reversedPaths;
+                                    res.sourceFiles += matchNow.sourceFiles;
+                                } else {
+                                    res = matchNow;
+                                }
+                            });
+            std::sort(res.sourceFiles.begin(), res.sourceFiles.end());
+            if (!res.sourceFiles.isEmpty()) {
+                return res.sourceFiles.first();
+            }
+            qCWarning(qmljsLog) << "Could not find source file for file" << path
+                                << "in application path" << p;
+        }
+    }
+    return path;
+}
+
 } // namespace QmlJS
