@@ -26,11 +26,12 @@
 #include "qmljsstaticanalysismessage.h"
 #include "qmljsconstants.h"
 #include "parser/qmljsengine_p.h"
+#include "parser/qmljsdiagnosticmessage_p.h"
 
 #include <utils/qtcassert.h>
 
 #include <QCoreApplication>
-#include <QRegExp>
+#include <QRegularExpression>
 
 using namespace QmlJS;
 using namespace QmlJS::StaticAnalysis;
@@ -70,7 +71,7 @@ static inline QString msgInvalidConstructor(const char *what)
 StaticAnalysisMessages::StaticAnalysisMessages()
 {
     // When changing a message or severity, update the documentation, currently
-    // in creator-editors.qdoc, accordingly.
+    // in creator-code-syntax.qdoc, accordingly.
     newMsg(ErrInvalidEnumValue, Error,
            tr("Invalid value for enum."));
     newMsg(ErrEnumValueMustBeStringOrNumber, Error,
@@ -155,7 +156,7 @@ StaticAnalysisMessages::StaticAnalysisMessages()
            tr("Unnecessary parentheses."));
     newMsg(MaybeWarnEqualityTypeCoercion, MaybeWarning,
            tr("== and != may perform type coercion, use === or !== to avoid it."));
-    newMsg(WarnConfusingExpressionStatement, Error,
+    newMsg(WarnConfusingExpressionStatement, Warning,
            tr("Expression statements should be assignments, calls or delete expressions only."));
     newMsg(HintDeclarationsShouldBeAtStartOfFunction, Hint,
            tr("Place var declarations at the start of a function."));
@@ -210,38 +211,54 @@ StaticAnalysisMessages::StaticAnalysisMessages()
     newMsg(ErrInvalidArrayValueLength, Error,
            tr("%1 elements expected in array value."), 1);
     newMsg(WarnImperativeCodeNotEditableInVisualDesigner, Warning,
-            tr("Imperative code is not supported in the Qt Quick Designer."));
+            tr("Imperative code is not supported in Qt Design Studio."));
     newMsg(WarnUnsupportedTypeInVisualDesigner, Warning,
-            tr("This type (%1) is not supported in the Qt Quick Designer."), 1);
+            tr("This type (%1) is not supported in Qt Design Studio."), 1);
     newMsg(WarnReferenceToParentItemNotSupportedByVisualDesigner, Warning,
-            tr("Reference to parent item cannot be resolved correctly by the Qt Quick Designer."));
+            tr("Reference to parent item cannot be resolved correctly by Qt Design Studio."));
     newMsg(WarnUndefinedValueForVisualDesigner, Warning,
             tr("This visual property binding cannot be evaluated in the local context "
-               "and might not show up in Qt Quick Designer as expected."));
+               "and might not show up in Qt Design Studio as expected."));
     newMsg(WarnStatesOnlyInRootItemForVisualDesigner, Warning,
-            tr("Qt Quick Designer only supports states in the root item."));
+            tr("Qt Design Studio only supports states in the root item."));
     newMsg(ErrInvalidIdeInVisualDesigner, Error,
-           tr("This id might be ambiguous and is not supported in the Qt Quick Designer."));
-    newMsg(WarnAboutQtQuick1InsteadQtQuick2, Warning,
-            tr("Using Qt Quick 1 code model instead of Qt Quick 2."));
+           tr("This id might be ambiguous and is not supported in Qt Design Studio."));
     newMsg(ErrUnsupportedRootTypeInVisualDesigner, Error,
-           tr("This type (%1) is not supported as a root element by Qt Quick Designer."), 1);
+           tr("This type (%1) is not supported as a root element by Qt Design Studio."), 1);
     newMsg(ErrUnsupportedRootTypeInQmlUi, Error,
-           tr("This type (%1) is not supported as a root element of a Qt Quick UI form."), 1);
+           tr("This type (%1) is not supported as a root element of a UI file (.ui.qml)."), 1);
     newMsg(ErrUnsupportedTypeInQmlUi, Error,
-            tr("This type (%1) is not supported in a Qt Quick UI form."), 1);
+            tr("This type (%1) is not supported in a UI file (.ui.qml)."), 1);
     newMsg(ErrFunctionsNotSupportedInQmlUi, Error,
-            tr("Functions are not supported in a Qt Quick UI form."));
+            tr("Functions are not supported in a UI file (.ui.qml)."));
     newMsg(ErrBlocksNotSupportedInQmlUi, Error,
-            tr("JavaScript blocks are not supported in a Qt Quick UI form."));
+            tr("JavaScript blocks are not supported in a UI file (.ui.qml)."));
     newMsg(ErrBehavioursNotSupportedInQmlUi, Error,
-            tr("Behavior type is not supported in a Qt Quick UI form."));
+            tr("Behavior type is not supported in a UI file (.ui.qml)."));
     newMsg(ErrStatesOnlyInRootItemInQmlUi, Error,
-            tr("States are only supported in the root item in a Qt Quick UI form."));
+            tr("States are only supported in the root item in a UI file (.ui.qml)."));
     newMsg(ErrReferenceToParentItemNotSupportedInQmlUi, Error,
-            tr("Referencing the parent of the root item is not supported in a Qt Quick UI form."));
+            tr("Referencing the parent of the root item is not supported in a UI file (.ui.qml)."));
+    newMsg(ErrDoNotMixTranslationFunctionsInQmlUi, Error,
+           tr("Do not mix translation functions in a UI file (.ui.qml)."));
     newMsg(StateCannotHaveChildItem, Error,
             tr("A State cannot have a child item (%1)."), 1);
+    newMsg(WarnDuplicateImport, Warning,
+           tr("Duplicate import (%1)."), 1);
+    newMsg(ErrHitMaximumRecursion, Error,
+           tr("Hit maximum recursion limit when visiting AST."));
+    newMsg(ErrTypeIsInstantiatedRecursively, Error,
+            tr("Type cannot be instantiated recursively (%1)."), 1);
+    newMsg(WarnLogicalValueDoesNotDependOnValues, Warning,
+           tr("Logical value does not depend on actual values."));
+    newMsg(ErrToManyComponentChildren, Error,
+           tr("Components are only allowed to have a single child element."));
+    newMsg(WarnComponentRequiresChildren, Warning,
+           tr("Components require a child element."));
+    newMsg(ErrAliasReferRoot, Error,
+           tr("Do not reference the root item as alias."));
+    newMsg(WarnAliasReferRootHierarchy, Warning,
+           tr("Avoid referencing the root item in a hierarchy."));
 }
 
 } // anonymous namespace
@@ -254,11 +271,11 @@ QList<Type> Message::allMessageTypes()
 }
 
 Message::Message()
-    : type(UnknownType), severity(Hint)
+    : type(UnknownType)
 {}
 
 Message::Message(Type type,
-                 AST::SourceLocation location,
+                 SourceLocation location,
                  const QString &arg1,
                  const QString &arg2,
                  bool appendTypeId)
@@ -312,9 +329,9 @@ QString Message::suppressionString() const
     return QString::fromLatin1("@disable-check M%1").arg(QString::number(type));
 }
 
-QRegExp Message::suppressionPattern()
+QRegularExpression Message::suppressionPattern()
 {
-    return QRegExp(QLatin1String("@disable-check M(\\d+)"));
+    return QRegularExpression("@disable-check M(\\d+)");
 }
 
 const PrototypeMessageData Message::prototypeForMessageType(Type type)
