@@ -26,49 +26,64 @@
 #include "environment.h"
 
 #include "algorithm.h"
+#include "fileutils.h"
 #include "qtcassert.h"
 
-#include <QDebug>
 #include <QDir>
 #include <QProcessEnvironment>
 #include <QSet>
-#include <QCoreApplication>
-
-Q_GLOBAL_STATIC_WITH_ARGS(Utils::Environment, staticSystemEnvironment,
-                          (QProcessEnvironment::systemEnvironment().toStringList()))
-
-Q_GLOBAL_STATIC(QVector<Utils::EnvironmentProvider>, environmentProviders)
 
 namespace Utils {
 
+Q_GLOBAL_STATIC_WITH_ARGS(Environment, staticSystemEnvironment,
+                          (QProcessEnvironment::systemEnvironment().toStringList()))
+
+Q_GLOBAL_STATIC(QVector<EnvironmentProvider>, environmentProviders)
+
+NameValueItems Environment::diff(const Environment &other, bool checkAppendPrepend) const
+{
+    return m_dict.diff(other.m_dict, checkAppendPrepend);
+}
+
+int Environment::isValid() const
+{
+    return m_dict.size() != 0;
+}
+
 QProcessEnvironment Environment::toProcessEnvironment() const
 {
     QProcessEnvironment result;
-    for (auto it = m_values.constBegin(); it != m_values.constEnd(); ++it) {
+    for (auto it = m_dict.m_values.constBegin(); it != m_dict.m_values.constEnd(); ++it) {
         if (it.value().second)
             result.insert(it.key().name, expandedValueForKey(key(it)));
     }
     return result;
 }
 
-void Environment::appendOrSetPath(const QString &value)
+void Environment::appendOrSetPath(const FilePath &value)
 {
-    appendOrSet("PATH", QDir::toNativeSeparators(value),
-                QString(OsSpecificAspects::pathListSeparator(m_osType)));
+    QTC_CHECK(value.osType() == osType());
+    if (value.isEmpty())
+        return;
+    appendOrSet("PATH", value.nativePath(),
+                QString(OsSpecificAspects::pathListSeparator(osType())));
 }
 
-void Environment::prependOrSetPath(const QString &value)
+void Environment::prependOrSetPath(const FilePath &value)
 {
-    prependOrSet("PATH", QDir::toNativeSeparators(value),
-            QString(OsSpecificAspects::pathListSeparator(m_osType)));
+    QTC_CHECK(value.osType() == osType());
+    if (value.isEmpty())
+        return;
+    prependOrSet("PATH", value.nativePath(),
+                 QString(OsSpecificAspects::pathListSeparator(osType())));
 }
 
 void Environment::appendOrSet(const QString &key, const QString &value, const QString &sep)
 {
     QTC_ASSERT(!key.contains('='), return );
-    const auto it = findKey(key);
-    if (it == m_values.end()) {
-        m_values.insert(DictKey(key, nameCaseSensitivity()), qMakePair(value, true));
+    const auto it = m_dict.findKey(key);
+    if (it == m_dict.m_values.end()) {
+        m_dict.m_values.insert(DictKey(key, m_dict.nameCaseSensitivity()), qMakePair(value, true));
     } else {
         // Append unless it is already there
         const QString toAppend = sep + value;
@@ -80,9 +95,9 @@ void Environment::appendOrSet(const QString &key, const QString &value, const QS
 void Environment::prependOrSet(const QString &key, const QString &value, const QString &sep)
 {
     QTC_ASSERT(!key.contains('='), return );
-    const auto it = findKey(key);
-    if (it == m_values.end()) {
-        m_values.insert(DictKey(key, nameCaseSensitivity()), qMakePair(value, true));
+    const auto it = m_dict.findKey(key);
+    if (it == m_dict.m_values.end()) {
+        m_dict.m_values.insert(DictKey(key, m_dict.nameCaseSensitivity()), qMakePair(value, true));
     } else {
         // Prepend unless it is already there
         const QString toPrepend = value + sep;
@@ -91,17 +106,18 @@ void Environment::prependOrSet(const QString &key, const QString &value, const Q
     }
 }
 
-void Environment::prependOrSetLibrarySearchPath(const QString &value)
+void Environment::prependOrSetLibrarySearchPath(const FilePath &value)
 {
-    switch (m_osType) {
+    QTC_CHECK(value.osType() == osType());
+    switch (osType()) {
     case OsTypeWindows: {
         const QChar sep = ';';
-        prependOrSet("PATH", QDir::toNativeSeparators(value), QString(sep));
+        prependOrSet("PATH", value.nativePath(), QString(sep));
         break;
     }
     case OsTypeMac: {
         const QString sep =  ":";
-        const QString nativeValue = QDir::toNativeSeparators(value);
+        const QString nativeValue = value.nativePath();
         prependOrSet("DYLD_LIBRARY_PATH", nativeValue, sep);
         prependOrSet("DYLD_FRAMEWORK_PATH", nativeValue, sep);
         break;
@@ -109,7 +125,7 @@ void Environment::prependOrSetLibrarySearchPath(const QString &value)
     case OsTypeLinux:
     case OsTypeOtherUnix: {
         const QChar sep = ':';
-        prependOrSet("LD_LIBRARY_PATH", QDir::toNativeSeparators(value), QString(sep));
+        prependOrSet("LD_LIBRARY_PATH", value.nativePath(), QString(sep));
         break;
     }
     default:
@@ -117,9 +133,9 @@ void Environment::prependOrSetLibrarySearchPath(const QString &value)
     }
 }
 
-void Environment::prependOrSetLibrarySearchPaths(const QStringList &values)
+void Environment::prependOrSetLibrarySearchPaths(const FilePaths &values)
 {
-    Utils::reverseForeach(values, [this](const QString &value) {
+    reverseForeach(values, [this](const FilePath &value) {
         prependOrSetLibrarySearchPath(value);
     });
 }
@@ -129,35 +145,15 @@ Environment Environment::systemEnvironment()
     return *staticSystemEnvironment();
 }
 
-const char lcMessages[] = "LC_MESSAGES";
-const char language[] = "LANGUAGE";
-const char englishLocale[] = "en_US.utf8";
-const char languageEnglishLocales[] = "en_US:en";
-
-void Environment::setupEnglishOutput(Environment *environment)
+void Environment::setupEnglishOutput()
 {
-    QTC_ASSERT(environment, return);
-    environment->set(lcMessages, englishLocale);
-    environment->set(language, languageEnglishLocales);
+    m_dict.set("LC_MESSAGES", "en_US.utf8");
+    m_dict.set("LANGUAGE", "en_US:en");
 }
 
-void Environment::setupEnglishOutput(QProcessEnvironment *environment)
-{
-    QTC_ASSERT(environment, return);
-    environment->insert(lcMessages, englishLocale);
-    environment->insert(language, languageEnglishLocales);
-}
-
-void Environment::setupEnglishOutput(QStringList *environment)
-{
-    QTC_ASSERT(environment, return);
-    Environment env(*environment);
-    setupEnglishOutput(&env);
-    *environment = env.toStringList();
-}
-
-FilePath Environment::searchInDirectory(const QStringList &execs, const FilePath &directory,
-                                        QSet<FilePath> &alreadyChecked) const
+static FilePath searchInDirectory(const QStringList &execs,
+                                  const FilePath &directory,
+                                  QSet<FilePath> &alreadyChecked)
 {
     const int checkedCount = alreadyChecked.count();
     alreadyChecked.insert(directory);
@@ -180,7 +176,7 @@ QStringList Environment::appendExeExtensions(const QString &executable) const
 {
     QStringList execs(executable);
     const QFileInfo fi(executable);
-    if (m_osType == OsTypeWindows) {
+    if (osType() == OsTypeWindows) {
         // Check all the executable extensions on windows:
         // PATHEXT is only used if the executable has no extension
         if (fi.suffix().isEmpty()) {
@@ -203,7 +199,9 @@ bool Environment::isSameExecutable(const QString &exe1, const QString &exe2) con
             const FilePath f2 = FilePath::fromString(i2);
             if (f1 == f2)
                 return true;
-            if (FileUtils::resolveSymlinks(f1) == FileUtils::resolveSymlinks(f2))
+            if (f1.needsDevice() != f2.needsDevice() || f1.scheme() != f2.scheme())
+                return false;
+            if (f1.resolveSymlinks() == f2.resolveSymlinks())
                 return true;
             if (FileUtils::fileId(f1) == FileUtils::fileId(f2))
                 return true;
@@ -214,20 +212,22 @@ bool Environment::isSameExecutable(const QString &exe1, const QString &exe2) con
 
 QString Environment::expandedValueForKey(const QString &key) const
 {
-    return expandVariables(value(key));
+    return expandVariables(m_dict.value(key));
 }
 
-FilePath Environment::searchInPath(const QString &executable,
-                                   const FilePathList &additionalDirs,
-                                   const PathFilter &func) const
+static FilePath searchInDirectoriesHelper(const Environment &env,
+                                          const QString &executable,
+                                          const FilePaths &dirs,
+                                          const Environment::PathFilter &func,
+                                          bool usePath)
 {
     if (executable.isEmpty())
         return FilePath();
 
-    const QString exec = QDir::cleanPath(expandVariables(executable));
+    const QString exec = QDir::cleanPath(env.expandVariables(executable));
     const QFileInfo fi(exec);
 
-    const QStringList execs = appendExeExtensions(exec);
+    const QStringList execs = env.appendExeExtensions(exec);
 
     if (fi.isAbsolute()) {
         for (const QString &path : execs) {
@@ -239,25 +239,40 @@ FilePath Environment::searchInPath(const QString &executable,
     }
 
     QSet<FilePath> alreadyChecked;
-    for (const FilePath &dir : additionalDirs) {
+    for (const FilePath &dir : dirs) {
         FilePath tmp = searchInDirectory(execs, dir, alreadyChecked);
         if (!tmp.isEmpty() && (!func || func(tmp)))
             return tmp;
     }
 
-    if (executable.contains('/'))
-        return FilePath();
+    if (usePath) {
+        if (executable.contains('/'))
+            return FilePath();
 
-    for (const FilePath &p : path()) {
-        FilePath tmp = searchInDirectory(execs, p, alreadyChecked);
-        if (!tmp.isEmpty() && (!func || func(tmp)))
-            return tmp;
+        for (const FilePath &p : env.path()) {
+            FilePath tmp = searchInDirectory(execs, p, alreadyChecked);
+            if (!tmp.isEmpty() && (!func || func(tmp)))
+                return tmp;
+        }
     }
     return FilePath();
 }
 
-FilePathList Environment::findAllInPath(const QString &executable,
-                                        const FilePathList &additionalDirs,
+FilePath Environment::searchInDirectories(const QString &executable,
+                                          const FilePaths &dirs) const
+{
+    return searchInDirectoriesHelper(*this, executable, dirs, {}, false);
+}
+
+FilePath Environment::searchInPath(const QString &executable,
+                                   const FilePaths &additionalDirs,
+                                   const PathFilter &func) const
+{
+    return searchInDirectoriesHelper(*this, executable, additionalDirs, func, true);
+}
+
+FilePaths Environment::findAllInPath(const QString &executable,
+                                        const FilePaths &additionalDirs,
                                         const Environment::PathFilter &func) const
 {
     if (executable.isEmpty())
@@ -295,15 +310,15 @@ FilePathList Environment::findAllInPath(const QString &executable,
     return result.values();
 }
 
-FilePathList Environment::path() const
+FilePaths Environment::path() const
 {
     return pathListValue("PATH");
 }
 
-FilePathList Environment::pathListValue(const QString &varName) const
+FilePaths Environment::pathListValue(const QString &varName) const
 {
-    const QStringList pathComponents = expandedValueForKey(varName)
-            .split(OsSpecificAspects::pathListSeparator(m_osType), QString::SkipEmptyParts);
+    const QStringList pathComponents = expandedValueForKey(varName).split(
+        OsSpecificAspects::pathListSeparator(osType()), Qt::SkipEmptyParts);
     return transform(pathComponents, &FilePath::fromUserInput);
 }
 
@@ -312,6 +327,11 @@ void Environment::modifySystemEnvironment(const EnvironmentItems &list)
     staticSystemEnvironment->modify(list);
 }
 
+void Environment::setSystemEnvironment(const Environment &environment)
+{
+    *staticSystemEnvironment = environment;
+}
+
 /** Expand environment variables in a string.
  *
  * Environment variables are accepted in the following forms:
@@ -323,12 +343,12 @@ QString Environment::expandVariables(const QString &input) const
 {
     QString result = input;
 
-    if (m_osType == OsTypeWindows) {
+    if (osType() == OsTypeWindows) {
         for (int vStart = -1, i = 0; i < result.length(); ) {
             if (result.at(i++) == '%') {
                 if (vStart > 0) {
-                    const auto it = findKey(result.mid(vStart, i - vStart - 1));
-                    if (it != m_values.constEnd()) {
+                    const auto it = m_dict.findKey(result.mid(vStart, i - vStart - 1));
+                    if (it != m_dict.m_values.constEnd()) {
                         result.replace(vStart - 1, i - vStart + 1, it->first);
                         i = vStart - 1 + it->first.length();
                         vStart = -1;
@@ -395,7 +415,7 @@ FilePath Environment::expandVariables(const FilePath &variables) const
 
 QStringList Environment::expandVariables(const QStringList &variables) const
 {
-    return Utils::transform(variables, [this](const QString &i) { return expandVariables(i); });
+    return transform(variables, [this](const QString &i) { return expandVariables(i); });
 }
 
 void EnvironmentProvider::addProvider(EnvironmentProvider &&provider)
@@ -416,4 +436,63 @@ optional<EnvironmentProvider> EnvironmentProvider::provider(const QByteArray &id
     return nullopt;
 }
 
+void EnvironmentChange::addSetValue(const QString &key, const QString &value)
+{
+    m_changeItems.append({Item::SetValue, QVariant::fromValue(QPair<QString, QString>(key, value))});
+}
+
+void EnvironmentChange::addUnsetValue(const QString &key)
+{
+    m_changeItems.append({Item::UnsetValue, key});
+}
+
+void EnvironmentChange::addPrependToPath(const FilePaths &values)
+{
+    for (int i = values.size(); --i >= 0; ) {
+        const FilePath value = values.at(i);
+        m_changeItems.append({Item::PrependToPath, value.toVariant()});
+    }
+}
+
+void EnvironmentChange::addAppendToPath(const FilePaths &values)
+{
+    for (const FilePath &value : values)
+        m_changeItems.append({Item::AppendToPath, value.toVariant()});
+}
+
+EnvironmentChange EnvironmentChange::fromFixedEnvironment(const Environment &fixedEnv)
+{
+    EnvironmentChange change;
+    change.m_changeItems.append({Item::SetFixedEnvironment, QVariant::fromValue(fixedEnv)});
+    return change;
+}
+
+void EnvironmentChange::applyToEnvironment(Environment &env) const
+{
+    for (const Item &item : m_changeItems) {
+        switch (item.type) {
+        case Item::SetSystemEnvironment:
+            env = Environment::systemEnvironment();
+            break;
+        case Item::SetFixedEnvironment:
+            env = item.data.value<Environment>();
+            break;
+        case Item::SetValue: {
+            auto data = item.data.value<QPair<QString, QString>>();
+            env.set(data.first, data.second);
+            break;
+        }
+        case Item::UnsetValue:
+            env.unset(item.data.toString());
+            break;
+        case Item::PrependToPath:
+            env.prependOrSetPath(FilePath::fromVariant(item.data));
+            break;
+        case Item::AppendToPath:
+            env.appendOrSetPath(FilePath::fromVariant(item.data));
+            break;
+        }
+    }
+}
+
 } // namespace Utils
