@@ -26,6 +26,7 @@
 #pragma once
 
 #include "utils_global.h"
+
 #include "completinglineedit.h"
 
 #include <QAbstractButton>
@@ -34,6 +35,7 @@
 
 QT_BEGIN_NAMESPACE
 class QEvent;
+class QKeySequence;
 QT_END_NAMESPACE
 
 namespace Utils {
@@ -46,8 +48,8 @@ class QTCREATOR_UTILS_EXPORT IconButton: public QAbstractButton
     Q_PROPERTY(float iconOpacity READ iconOpacity WRITE setIconOpacity)
     Q_PROPERTY(bool autoHide READ hasAutoHide WRITE setAutoHide)
 public:
-    explicit IconButton(QWidget *parent = 0);
-    void paintEvent(QPaintEvent *event);
+    explicit IconButton(QWidget *parent = nullptr);
+    void paintEvent(QPaintEvent *event) override;
     float iconOpacity() { return m_iconOpacity; }
     void setIconOpacity(float value) { m_iconOpacity = value; update(); }
     void animateShow(bool visible);
@@ -55,11 +57,11 @@ public:
     void setAutoHide(bool hide) { m_autoHide = hide; }
     bool hasAutoHide() const { return m_autoHide; }
 
-    QSize sizeHint() const;
+    QSize sizeHint() const override;
 
 protected:
-    void keyPressEvent(QKeyEvent *ke);
-    void keyReleaseEvent(QKeyEvent *ke);
+    void keyPressEvent(QKeyEvent *ke) override;
+    void keyReleaseEvent(QKeyEvent *ke) override;
 
 private:
     float m_iconOpacity;
@@ -70,18 +72,14 @@ private:
 class QTCREATOR_UTILS_EXPORT FancyLineEdit : public CompletingLineEdit
 {
     Q_OBJECT
-    Q_ENUMS(Side)
-
-    // Validation.
-    Q_PROPERTY(QString initialText READ initialText WRITE setInitialText DESIGNABLE true)
-    Q_PROPERTY(QColor errorColor READ errorColor WRITE setErrorColor DESIGNABLE true)
-    Q_PROPERTY(QColor okColor READ okColor WRITE setOkColor DESIGNABLE true)
-
 public:
     enum Side {Left = 0, Right = 1};
+    Q_ENUM(Side)
 
-    explicit FancyLineEdit(QWidget *parent = 0);
-    ~FancyLineEdit();
+    explicit FancyLineEdit(QWidget *parent = nullptr);
+    ~FancyLineEdit() override;
+
+    void setTextKeepingActiveCursor(const QString &text);
 
     QIcon buttonIcon(Side side) const;
     void setButtonIcon(Side side, const QIcon &icon);
@@ -122,33 +120,28 @@ public:
     //  Validation
 
     // line edit, (out)errorMessage -> valid?
-    typedef std::function<bool(FancyLineEdit *, QString *)> ValidationFunction;
-    enum State { Invalid, DisplayingInitialText, Valid };
+    using ValidationFunction = std::function<bool(FancyLineEdit *, QString *)>;
+    enum State { Invalid, DisplayingPlaceholderText, Valid };
 
     State state() const;
     bool isValid() const;
     QString errorMessage() const;
 
-    QString initialText() const;
-    void setInitialText(const QString &);
-
-    QColor errorColor() const;
-    void setErrorColor(const  QColor &c);
-
-    QColor okColor() const;
-    void setOkColor(const  QColor &c);
-
     void setValidationFunction(const ValidationFunction &fn);
     static ValidationFunction defaultValidationFunction();
     void validate();
     void onEditingFinished();
 
+    static void setCamelCaseNavigationEnabled(bool enabled);
+    static void setCompletionShortcut(const QKeySequence &shortcut);
+
 protected:
     // Custom behaviour can be added here.
     virtual void handleChanged(const QString &) {}
+    void keyPressEvent(QKeyEvent *event) override;
 
 signals:
-    void buttonClicked(Utils::FancyLineEdit::Side side);
+    void buttonClicked(FancyLineEdit::Side side);
     void leftButtonClicked();
     void rightButtonClicked();
 
@@ -158,7 +151,7 @@ signals:
     void validReturnPressed();
 
 protected:
-    void resizeEvent(QResizeEvent *e);
+    void resizeEvent(QResizeEvent *e) override;
 
     virtual QString fixInputString(const QString &string);
 
@@ -172,6 +165,8 @@ private:
 
     void updateMargins();
     void updateButtonPositions();
+    bool camelCaseBackward(bool mark);
+    bool camelCaseForward(bool mark);
     friend class FancyLineEditPrivate;
 
     FancyLineEditPrivate *d;
