@@ -26,6 +26,7 @@
 #include "fancymainwindow.h"
 
 #include "algorithm.h"
+#include "porting.h"
 #include "qtcassert.h"
 #include "stringutils.h"
 
@@ -41,7 +42,6 @@
 #include <QStyle>
 #include <QStyleOption>
 #include <QTimer>
-#include <QToolButton>
 
 static const char AutoHideTitleBarsKey[] = "AutoHideTitleBars";
 static const char ShowCentralWidgetKey[] = "ShowCentralWidget";
@@ -74,7 +74,7 @@ public:
     DockWidget(QWidget *inner, FancyMainWindow *parent, bool immutable = false);
 
     bool eventFilter(QObject *, QEvent *event) override;
-    void enterEvent(QEvent *event) override;
+    void enterEvent(EnterEvent *event) override;
     void leaveEvent(QEvent *event) override;
     void handleMouseTimeout();
     void handleToplevelChanged(bool floating);
@@ -114,7 +114,7 @@ public:
 
     QSize minimumSizeHint() const override { return sizeHint(); }
 
-    void enterEvent(QEvent *event) override
+    void enterEvent(EnterEvent *event) override
     {
         if (isEnabled())
             update();
@@ -136,11 +136,11 @@ void DockWidgetTitleButton::paintEvent(QPaintEvent *)
     QPainter p(this);
 
     QStyleOptionToolButton opt;
-    opt.init(this);
+    opt.initFrom(this);
     opt.state |= QStyle::State_AutoRaise;
     opt.icon = icon();
-    opt.subControls = nullptr;
-    opt.activeSubControls = nullptr;
+    opt.subControls = {};
+    opt.activeSubControls = {};
     opt.features = QStyleOptionToolButton::None;
     opt.arrowType = Qt::NoArrow;
     int size = style()->pixelMetric(QStyle::PM_SmallIconSize, nullptr, this);
@@ -182,7 +182,6 @@ public:
         m_maximumActiveSize   = QSize(maxWidth, activeHeight);
 
         auto layout = new QHBoxLayout(this);
-        layout->setMargin(0);
         layout->setSpacing(0);
         layout->setContentsMargins(4, 0, 0, 0);
         layout->addWidget(m_titleLabel);
@@ -194,7 +193,7 @@ public:
         setProperty("managed_titlebar", 1);
     }
 
-    void enterEvent(QEvent *event) override
+    void enterEvent(EnterEvent *event) override
     {
         setActive(true);
         QWidget::enterEvent(event);
@@ -255,7 +254,7 @@ DockWidget::DockWidget(QWidget *inner, FancyMainWindow *parent, bool immutable)
 
     QString title = inner->windowTitle();
     toggleViewAction()->setProperty("original_title", title);
-    title = Utils::stripAccelerator(title);
+    title = stripAccelerator(title);
     setWindowTitle(title);
 
     QStyleOptionDockWidget opt;
@@ -304,7 +303,7 @@ bool DockWidget::eventFilter(QObject *, QEvent *event)
     return false;
 }
 
-void DockWidget::enterEvent(QEvent *event)
+void DockWidget::enterEvent(EnterEvent *event)
 {
     if (!m_immutable)
         QApplication::instance()->installEventFilter(this);
@@ -468,20 +467,17 @@ void FancyMainWindow::handleVisibilityChanged(bool visible)
 
 void FancyMainWindow::saveSettings(QSettings *settings) const
 {
-    QHash<QString, QVariant> hash = saveSettings();
-    QHashIterator<QString, QVariant> it(hash);
-    while (it.hasNext()) {
-        it.next();
+    const QHash<QString, QVariant> hash = saveSettings();
+    for (auto it = hash.cbegin(), end = hash.cend(); it != end; ++it)
         settings->setValue(it.key(), it.value());
-    }
 }
 
 void FancyMainWindow::restoreSettings(const QSettings *settings)
 {
     QHash<QString, QVariant> hash;
-    foreach (const QString &key, settings->childKeys()) {
+    const QStringList childKeys = settings->childKeys();
+    for (const QString &key : childKeys)
         hash.insert(key, settings->value(key));
-    }
     restoreSettings(hash);
 }
 
@@ -551,12 +547,12 @@ void FancyMainWindow::addDockActionsToMenu(QMenu *menu)
             actions.append(action);
         }
     }
-    Utils::sort(actions, [](const QAction *action1, const QAction *action2) {
+    sort(actions, [](const QAction *action1, const QAction *action2) {
         QTC_ASSERT(action1, return true);
         QTC_ASSERT(action2, return false);
         return stripAccelerator(action1->text()).toLower() < stripAccelerator(action2->text()).toLower();
     });
-    foreach (QAction *action, actions)
+    for (QAction *action : qAsConst(actions))
         menu->addAction(action);
     menu->addAction(&d->m_showCentralWidget);
     menu->addAction(&d->m_menuSeparator1);
