@@ -47,6 +47,8 @@
 #include "mimeprovider_p.h"
 #include "mimetype_p.h"
 
+#include <utils/fileutils.h>
+
 #include <QtCore/QCoreApplication>
 #include <QtCore/QFile>
 #include <QtCore/QFileInfo>
@@ -59,12 +61,6 @@
 #include <algorithm>
 #include <functional>
 
-#ifdef Q_OS_WIN
-static struct {const char *source; const char *comment; } ALL_FILES_FILTER = QT_TRANSLATE_NOOP3("Core", "All Files (*.*)", "On Windows");
-#else
-static struct {const char *source; const char *comment; } ALL_FILES_FILTER = QT_TRANSLATE_NOOP3("Core", "All Files (*)", "On Linux/macOS");
-#endif
-
 using namespace Utils;
 using namespace Utils::Internal;
 
@@ -207,7 +203,7 @@ MimeType MimeDatabasePrivate::mimeTypeForFileNameAndData(const QString &fileName
                 return candidateByData;
             }
             // If there is a glob match that is a sub class of sniffedMime, use it
-            foreach (const QString &m, candidatesByName) {
+            for (const QString &m : qAsConst(candidatesByName)) {
                 if (inherits(m, sniffedMime)) {
                     // We have magic + pattern pointing to this, so it's a pretty good match
                     *accuracyPtr = 100;
@@ -247,7 +243,8 @@ bool MimeDatabasePrivate::inherits(const QString &mime, const QString &parent)
         const QString current = toCheck.pop();
         if (current == resolvedParent)
             return true;
-        foreach (const QString &par, provider()->parents(current)) {
+        const QStringList parents = provider()->parents(current);
+        for (const QString &par : parents) {
             int seenSize = seen.size();
             seen.insert(par);
             if (seen.size() != seenSize) // haven't seen before, so add
@@ -328,65 +325,7 @@ MimeDatabase::MimeDatabase() :
  */
 MimeDatabase::~MimeDatabase()
 {
-    d = 0;
-}
-
-void Utils::addMimeTypes(const QString &fileName, const QByteArray &data)
-{
-    auto d = MimeDatabasePrivate::instance();
-    QMutexLocker locker(&d->mutex);
-
-    if (d->m_startupPhase >= MimeDatabase::PluginsDelayedInitializing)
-        qWarning("Adding items from %s to MimeDatabase after initialization time",
-                 qPrintable(fileName));
-
-    auto xmlProvider = static_cast<MimeXMLProvider *>(d->provider());
-    xmlProvider->addData(fileName, data);
-}
-
-QString Utils::allFiltersString(QString *allFilesFilter)
-{
-    MimeDatabase mdb;
-    QSet<QString> uniqueFilters;
-    foreach (const MimeType &mt, mdb.allMimeTypes()) {
-        const QString &filterString = mt.filterString();
-        if (!filterString.isEmpty())
-            uniqueFilters.insert(mt.filterString());
-    }
-    QStringList filters;
-    foreach (const QString &filter, uniqueFilters)
-        filters.append(filter);
-    filters.sort();
-    const QString allFiles = allFilesFilterString();
-    if (allFilesFilter)
-        *allFilesFilter = allFiles;
-
-    // Prepend all files filter
-    filters.prepend(allFiles);
-
-    return filters.join(QLatin1String(";;"));
-}
-
-QString Utils::allFilesFilterString()
-{
-    auto d = MimeDatabasePrivate::instance();
-    if (d->m_startupPhase <= MimeDatabase::PluginsInitializing)
-        qWarning("Accessing MimeDatabase files filter strings before plugins are initialized");
-
-    return QCoreApplication::translate("Core", ALL_FILES_FILTER.source, ALL_FILES_FILTER.comment);
-}
-
-QStringList Utils::allGlobPatterns()
-{
-    auto d = MimeDatabasePrivate::instance();
-    if (d->m_startupPhase <= MimeDatabase::PluginsInitializing)
-        qWarning("Accessing MimeDatabase glob patterns before plugins are initialized");
-
-    MimeDatabase mdb;
-    QStringList patterns;
-    foreach (const MimeType &mt, mdb.allMimeTypes())
-        patterns.append(mt.globPatterns());
-    return patterns;
+    d = nullptr;
 }
 
 /*!
@@ -522,7 +461,7 @@ QList<MimeType> MimeDatabase::mimeTypesForFileName(const QString &fileName) cons
     QStringList matches = d->mimeTypeForFileName(fileName);
     QList<MimeType> mimes;
     matches.sort(); // Make it deterministic
-    foreach (const QString &mime, matches)
+    for (const QString &mime : qAsConst(matches))
         mimes.append(d->mimeTypeForName(mime));
     return mimes;
 }
@@ -681,69 +620,3 @@ QList<MimeType> MimeDatabase::allMimeTypes() const
 
     \value MatchContent The file content is used to look for a match
 */
-
-QMap<int, QList<MimeMagicRule> > Utils::magicRulesForMimeType(const MimeType &mimeType)
-{
-    auto d = MimeDatabasePrivate::instance();
-    QMutexLocker locker(&d->mutex);
-    return d->provider()->magicRulesForMimeType(mimeType);
-}
-
-void Utils::setGlobPatternsForMimeType(const MimeType &mimeType, const QStringList &patterns)
-{
-    auto d = MimeDatabasePrivate::instance();
-    QMutexLocker locker(&d->mutex);
-    d->provider()->setGlobPatternsForMimeType(mimeType, patterns);
-}
-
-void Utils::setMagicRulesForMimeType(const MimeType &mimeType, const QMap<int, QList<MimeMagicRule> > &rules)
-{
-    auto d = MimeDatabasePrivate::instance();
-    QMutexLocker locker(&d->mutex);
-    d->provider()->setMagicRulesForMimeType(mimeType, rules);
-}
-
-void Utils::setMimeStartupPhase(MimeStartupPhase phase)
-{
-    auto d = MimeDatabasePrivate::instance();
-    QMutexLocker locker(&d->mutex);
-    if (int(phase) != d->m_startupPhase + 1)
-        qWarning("Unexpected jump in MimedDatabase lifetime from %d to %d", d->m_startupPhase, int(phase));
-    d->m_startupPhase = int(phase);
-}
-
-MimeType Utils::mimeTypeForName(const QString &nameOrAlias)
-{
-    MimeDatabase mdb;
-    return mdb.mimeTypeForName(nameOrAlias);
-}
-
-MimeType Utils::mimeTypeForFile(const QString &fileName, MimeMatchMode mode)
-{
-    MimeDatabase mdb;
-    return mdb.mimeTypeForFile(fileName, MimeDatabase::MatchMode(mode));
-}
-
-MimeType Utils::mimeTypeForFile(const QFileInfo &fileInfo, MimeMatchMode mode)
-{
-    MimeDatabase mdb;
-    return mdb.mimeTypeForFile(fileInfo, MimeDatabase::MatchMode(mode));
-}
-
-QList<MimeType> Utils::mimeTypesForFileName(const QString &fileName)
-{
-    MimeDatabase mdb;
-    return mdb.mimeTypesForFileName(fileName);
-}
-
-MimeType Utils::mimeTypeForData(const QByteArray &data)
-{
-    MimeDatabase mdb;
-    return mdb.mimeTypeForData(data);
-}
-
-QList<MimeType> Utils::allMimeTypes()
-{
-    MimeDatabase mdb;
-    return mdb.allMimeTypes();
-}
