@@ -25,15 +25,14 @@
 
 #include "pathchooser.h"
 
+#include "commandline.h"
 #include "environment.h"
-#include "qtcassert.h"
-#include "macroexpander.h"
-
-#include "synchronousprocess.h"
+#include "fileutils.h"
 #include "hostosinfo.h"
-#include "theme/theme.h"
+#include "macroexpander.h"
+#include "qtcassert.h"
+#include "qtcprocess.h"
 
-#include <QDebug>
 #include <QFileDialog>
 #include <QHBoxLayout>
 #include <QMenu>
@@ -42,24 +41,41 @@
 
 /*!
     \class Utils::PathChooser
+    \inmodule QtCreator
 
-    \brief The PathChooser class is a control that lets the user choose a path,
-    consisting of a QLineEdit and
-    a "Browse" button.
+    \brief The PathChooser class is a control that lets the user choose a path.
+    The control consist of a QLineEdit and a "Browse" button, and is optionally
+    able to perform variable substitution.
 
     This class has some validation logic for embedding into QWizardPage.
 */
 
+/*!
+    \enum Utils::PathChooser::Kind
+    \inmodule QtCreator
+
+    The Kind enum describes the kind of path a PathChooser considers valid.
+
+    \value ExistingDirectory An existing directory
+    \value Directory A directory that does not need to exist
+    \value File An existing file
+    \value SaveFile A file that does not need to exist
+    \value ExistingCommand An executable file that must exist at the time of selection
+    \value Command An executable file that may or may not exist at the time of selection (e.g. result of a build)
+    \value Any No restriction on the selected path
+
+    \sa setExpectedKind(), expectedKind()
+*/
+
 namespace Utils {
 
-static QString appBundleExpandedPath(const QString &path)
+static FilePath appBundleExpandedPath(const FilePath &path)
 {
-    if (HostOsInfo::hostOs() == OsTypeMac && path.endsWith(".app")) {
+    if (path.osType() == OsTypeMac && path.endsWith(".app")) {
         // possibly expand to Foo.app/Contents/MacOS/Foo
-        QFileInfo info(path);
-        if (info.isDir()) {
-            QString exePath = path + "/Contents/MacOS/" + info.completeBaseName();
-            if (QFileInfo::exists(exePath))
+        if (path.isDir()) {
+            const FilePath exePath = path / "Contents/MacOS" / path.completeBaseName();
+            if (exePath.exists())
                 return exePath;
         }
     }
@@ -109,7 +125,7 @@ bool BinaryVersionToolTipEventFilter::eventFilter(QObject *o, QEvent *e)
     const QString binary = le->text();
     if (!binary.isEmpty()) {
         const QString version = BinaryVersionToolTipEventFilter::toolVersion(
-                    CommandLine(QDir::cleanPath(binary), m_arguments));
+                    CommandLine(FilePath::fromString(QDir::cleanPath(binary)), m_arguments));
         if (!version.isEmpty()) {
             // Concatenate tooltips.
             QString tooltip = "<html><head/><body>";
@@ -132,12 +148,13 @@ QString BinaryVersionToolTipEventFilter::toolVersion(const CommandLine &cmd)
 {
     if (cmd.executable().isEmpty())
         return QString();
-    SynchronousProcess proc;
+    QtcProcess proc;
     proc.setTimeoutS(1);
-    SynchronousProcessResponse response = proc.runBlocking(cmd);
-    if (response.result != SynchronousProcessResponse::Finished)
+    proc.setCommand(cmd);
+    proc.runBlocking();
+    if (proc.result() != ProcessResult::FinishedWithSuccess)
         return QString();
-    return response.allOutput();
+    return proc.allOutput();
 }
 
 // Extends BinaryVersionToolTipEventFilter to prepend the existing pathchooser
@@ -149,8 +166,7 @@ public:
         BinaryVersionToolTipEventFilter(pe->lineEdit()), m_pathChooser(pe) {}
 
 private:
-    QString defaultToolTip() const override
-        { return m_pathChooser->errorMessage(); }
+    QString defaultToolTip() const override { return m_pathChooser->errorMessage(); }
 
     const PathChooser *m_pathChooser = nullptr;
 };
@@ -162,7 +178,7 @@ class PathChooserPrivate
 public:
     PathChooserPrivate();
 
-    QString expandedPath(const QString &path) const;
+    FilePath expandedPath(const FilePath &path) const;
 
     QHBoxLayout *m_hLayout = nullptr;
     FancyLineEdit *m_lineEdit = nullptr;
@@ -170,39 +186,44 @@ public:
     PathChooser::Kind m_acceptingKind = PathChooser::ExistingDirectory;
     QString m_dialogTitleOverride;
     QString m_dialogFilter;
-    QString m_initialBrowsePathOverride;
+    FilePath m_initialBrowsePathOverride;
+    QString m_defaultValue;
     FilePath m_baseDirectory;
-    Environment m_environment;
+    EnvironmentChange m_environmentChange;
     BinaryVersionToolTipEventFilter *m_binaryVersionToolTipEventFilter = nullptr;
     QList<QAbstractButton *> m_buttons;
-    MacroExpander *m_macroExpander;
+    const MacroExpander *m_macroExpander = globalMacroExpander();
+    std::function<void()> m_openTerminal;
 };
 
-PathChooserPrivate::PathChooserPrivate() :
-    m_hLayout(new QHBoxLayout),
-    m_lineEdit(new FancyLineEdit),
-    m_macroExpander(globalMacroExpander())
+PathChooserPrivate::PathChooserPrivate()
+    : m_hLayout(new QHBoxLayout)
+    , m_lineEdit(new FancyLineEdit)
 {
 }
 
-QString PathChooserPrivate::expandedPath(const QString &input) const
+FilePath PathChooserPrivate::expandedPath(const FilePath &input) const
 {
     if (input.isEmpty())
-        return input;
+        return {};
+
+    FilePath path = input;
+
+    Environment env = path.deviceEnvironment();
+    m_environmentChange.applyToEnvironment(env);
+    path = env.expandVariables(path);
 
-    QString expandedInput = m_environment.expandVariables(input);
     if (m_macroExpander)
-        expandedInput = m_macroExpander->expand(expandedInput);
+        path = m_macroExpander->expand(path);
 
-    const QString path = FilePath::fromUserInput(expandedInput).toString();
     if (path.isEmpty())
         return path;
 
     switch (m_acceptingKind) {
     case PathChooser::Command:
     case PathChooser::ExistingCommand: {
-        const FilePath expanded = m_environment.searchInPath(path, {m_baseDirectory});
-        return expanded.isEmpty() ? path : expanded.toString();
+        const FilePath expanded = path.searchInPath({m_baseDirectory});
+        return expanded.isEmpty() ? path : expanded;
     }
     case PathChooser::Any:
         break;
@@ -210,8 +231,19 @@ QString PathChooserPrivate::expandedPath(const QString &input) const
     case PathChooser::ExistingDirectory:
     case PathChooser::File:
     case PathChooser::SaveFile:
-        if (!m_baseDirectory.isEmpty() && QFileInfo(path).isRelative())
-            return m_baseDirectory.pathAppended(path).toFileInfo().absoluteFilePath();
+        if (!m_baseDirectory.isEmpty()) {
+            FilePath fp = m_baseDirectory.resolvePath(path.path()).absoluteFilePath();
+            // FIXME bad hotfix for manually editing PathChooser (invalid paths, jumping cursor)
+            // examples: have an absolute path and try to change the device letter by typing the new
+            // letter and removing the original afterwards ends up in
+            // D:\\dev\\project\\cD:\\dev\\build-project (before trying to remove the original)
+            // as 'cD:\\dev\\build-project' is considered is handled as being relative
+            // input = "cD:\\dev\build-project"; // prepended 'c' to change the device letter
+            // m_baseDirectory = "D:\\dev\\project"
+            if (!fp.needsDevice() && HostOsInfo::isWindowsHost() && fp.toString().count(':') > 1)
+                return path;
+            return fp;
+        }
         break;
     }
     return path;
@@ -225,13 +257,20 @@ PathChooser::PathChooser(QWidget *parent) :
 
     d->m_lineEdit->setContextMenuPolicy(Qt::CustomContextMenu);
 
-    connect(d->m_lineEdit, &FancyLineEdit::customContextMenuRequested, this, &PathChooser::contextMenuRequested);
+    connect(d->m_lineEdit,
+            &FancyLineEdit::customContextMenuRequested,
+            this,
+            &PathChooser::contextMenuRequested);
     connect(d->m_lineEdit, &FancyLineEdit::validReturnPressed, this, &PathChooser::returnPressed);
     connect(d->m_lineEdit, &QLineEdit::textChanged, this,
             [this] { emit rawPathChanged(rawPath()); });
     connect(d->m_lineEdit, &FancyLineEdit::validChanged, this, &PathChooser::validChanged);
     connect(d->m_lineEdit, &QLineEdit::editingFinished, this, &PathChooser::editingFinished);
-    connect(d->m_lineEdit, &QLineEdit::textChanged, this, [this] { emit pathChanged(d->m_lineEdit->text()); });
+    connect(d->m_lineEdit, &QLineEdit::textChanged, this, [this] {
+        const QString text = d->m_lineEdit->text();
+        emit pathChanged(text);
+        emit filePathChanged(FilePath::fromUserInput(text));
+    });
 
     d->m_lineEdit->setMinimumWidth(120);
     d->m_hLayout->addWidget(d->m_lineEdit);
@@ -242,7 +281,6 @@ PathChooser::PathChooser(QWidget *parent) :
     setLayout(d->m_hLayout);
     setFocusProxy(d->m_lineEdit);
     setFocusPolicy(d->m_lineEdit->focusPolicy());
-    setEnvironment(Environment::systemEnvironment());
 
     d->m_lineEdit->setValidationFunction(defaultValidationFunction());
 }
@@ -293,11 +331,11 @@ FilePath PathChooser::baseDirectory() const
     return d->m_baseDirectory;
 }
 
-void PathChooser::setEnvironment(const Environment &env)
+void PathChooser::setEnvironmentChange(const EnvironmentChange &env)
 {
-    QString oldExpand = path();
-    d->m_environment = env;
-    if (path() != oldExpand) {
+    QString oldExpand = filePath().toString();
+    d->m_environmentChange = env;
+    if (filePath().toString() != oldExpand) {
         triggerChanged();
         emit rawPathChanged(rawPath());
     }
@@ -305,22 +343,22 @@ void PathChooser::setEnvironment(const Environment &env)
 
 QString PathChooser::rawPath() const
 {
-    return rawFileName().toString();
+    return rawFilePath().toString();
 }
 
-QString PathChooser::path() const
+FilePath PathChooser::rawFilePath() const
 {
-    return fileName().toString();
+    return FilePath::fromUserInput(d->m_lineEdit->text());
 }
 
-FilePath PathChooser::rawFileName() const
+FilePath PathChooser::filePath() const
 {
-    return FilePath::fromString(QDir::fromNativeSeparators(d->m_lineEdit->text()));
+    return d->expandedPath(rawFilePath());
 }
 
-FilePath PathChooser::fileName() const
+FilePath PathChooser::absoluteFilePath() const
 {
-    return FilePath::fromUserInput(d->expandedPath(rawFileName().toString()));
+    return d->m_baseDirectory.resolvePath(filePath());
 }
 
 // FIXME: try to remove again
@@ -342,7 +380,7 @@ void PathChooser::setPath(const QString &path)
     d->m_lineEdit->setTextKeepingActiveCursor(QDir::toNativeSeparators(path));
 }
 
-void PathChooser::setFileName(const FilePath &fn)
+void PathChooser::setFilePath(const FilePath &fn)
 {
     d->m_lineEdit->setTextKeepingActiveCursor(fn.toUserOutput());
 }
@@ -364,63 +402,54 @@ void PathChooser::slotBrowse()
 {
     emit beforeBrowsing();
 
-    QString predefined = path();
-    QFileInfo fi(predefined);
+    FilePath predefined = filePath();
 
-    if (!predefined.isEmpty() && !fi.isDir()) {
-        predefined = fi.path();
-        fi.setFile(predefined);
+    if (!predefined.isEmpty() && !predefined.isDir()) {
+        predefined = predefined.parentDir();
     }
 
-    if ((predefined.isEmpty() || !fi.isDir())
-            && !d->m_initialBrowsePathOverride.isNull()) {
+    if ((predefined.isEmpty() || !predefined.isDir()) && !d->m_initialBrowsePathOverride.isEmpty()) {
         predefined = d->m_initialBrowsePathOverride;
-        fi.setFile(predefined);
-        if (!fi.isDir()) {
+        if (!predefined.isDir())
             predefined.clear();
-            fi.setFile(QString());
-        }
     }
 
     // Prompt for a file/dir
-    QString newPath;
+    FilePath newPath;
     switch (d->m_acceptingKind) {
     case PathChooser::Directory:
     case PathChooser::ExistingDirectory:
-        newPath = QFileDialog::getExistingDirectory(this,
-                makeDialogTitle(tr("Choose Directory")), predefined);
+        newPath = FileUtils::getExistingDirectory(this,
+                        makeDialogTitle(tr("Choose Directory")), predefined);
         break;
     case PathChooser::ExistingCommand:
     case PathChooser::Command:
-        newPath = QFileDialog::getOpenFileName(this,
-                makeDialogTitle(tr("Choose Executable")), predefined,
-                d->m_dialogFilter);
+        newPath = FileUtils::getOpenFilePath(this,
+                        makeDialogTitle(tr("Choose Executable")), predefined, d->m_dialogFilter);
         newPath = appBundleExpandedPath(newPath);
         break;
     case PathChooser::File: // fall through
-        newPath = QFileDialog::getOpenFileName(this,
-                makeDialogTitle(tr("Choose File")), predefined,
-                d->m_dialogFilter);
+        newPath = FileUtils::getOpenFilePath(this,
+                        makeDialogTitle(tr("Choose File")), predefined, d->m_dialogFilter);
         newPath = appBundleExpandedPath(newPath);
         break;
     case PathChooser::SaveFile:
-        newPath = QFileDialog::getSaveFileName(this,
-                makeDialogTitle(tr("Choose File")), predefined,
-                d->m_dialogFilter);
+        newPath = FileUtils::getSaveFilePath(this,
+                        makeDialogTitle(tr("Choose File")), predefined, d->m_dialogFilter);
         break;
     case PathChooser::Any: {
         QFileDialog dialog(this);
         dialog.setFileMode(QFileDialog::AnyFile);
         dialog.setWindowTitle(makeDialogTitle(tr("Choose File")));
-        if (fi.exists())
-            dialog.setDirectory(fi.absolutePath());
+        if (predefined.exists())
+            dialog.setDirectory(predefined.absolutePath().toDir());
         // FIXME: fix QFileDialog so that it filters properly: lib*.a
         dialog.setNameFilter(d->m_dialogFilter);
         if (dialog.exec() == QDialog::Accepted) {
             // probably loop here until the *.framework dir match
             QStringList paths = dialog.selectedFiles();
             if (!paths.isEmpty())
-                newPath = paths.at(0);
+                newPath = FilePath::fromString(paths.at(0));
         }
         break;
         }
@@ -433,12 +462,11 @@ void PathChooser::slotBrowse()
     window()->raise();
     window()->activateWindow();
 
-    // Delete trailing slashes unless it is "/"|"\\", only
+    // Delete trailing slashes unless it is "/" only.
     if (!newPath.isEmpty()) {
-        newPath = QDir::toNativeSeparators(newPath);
-        if (newPath.size() > 1 && newPath.endsWith(QDir::separator()))
-            newPath.truncate(newPath.size() - 1);
-        setPath(newPath);
+        if (newPath.endsWith("/") && newPath.path().size() > 1)
+            newPath = newPath.withNewPath(newPath.path().chopped(1));
+        setFilePath(newPath);
     }
 
     emit browsingFinished();
@@ -477,6 +505,23 @@ void PathChooser::setAboutToShowContextMenuHandler(PathChooser::AboutToShowConte
     s_aboutToShowContextMenuHandler = handler;
 }
 
+void PathChooser::setOpenTerminalHandler(const std::function<void ()> &openTerminal)
+{
+    d->m_openTerminal = openTerminal;
+}
+
+std::function<void()> PathChooser::openTerminalHandler() const
+{
+    return d->m_openTerminal;
+}
+
+void PathChooser::setDefaultValue(const QString &defaultValue)
+{
+    d->m_defaultValue = defaultValue;
+    d->m_lineEdit->setPlaceholderText(defaultValue);
+    d->m_lineEdit->validate();
+}
+
 FancyLineEdit::ValidationFunction PathChooser::defaultValidationFunction() const
 {
     return std::bind(&PathChooser::validatePath, this, std::placeholders::_1, std::placeholders::_2);
@@ -484,83 +529,92 @@ FancyLineEdit::ValidationFunction PathChooser::defaultValidationFunction() const
 
 bool PathChooser::validatePath(FancyLineEdit *edit, QString *errorMessage) const
 {
-    const QString path = edit->text();
-    QString expandedPath = d->expandedPath(path);
+    QString input = edit->text();
 
-    if (path.isEmpty()) {
-        if (errorMessage)
-            *errorMessage = tr("The path must not be empty.");
-        return false;
+    if (input.isEmpty()) {
+        if (!d->m_defaultValue.isEmpty()) {
+            input = d->m_defaultValue;
+        } else {
+            if (errorMessage)
+                *errorMessage = tr("The path must not be empty.");
+            return false;
+        }
     }
 
-    if (expandedPath.isEmpty()) {
+    const FilePath filePath = d->expandedPath(FilePath::fromUserInput(input));
+    if (filePath.isEmpty()) {
         if (errorMessage)
-            *errorMessage = tr("The path \"%1\" expanded to an empty string.").arg(QDir::toNativeSeparators(path));
+            *errorMessage = tr("The path \"%1\" expanded to an empty string.").arg(input);
         return false;
     }
-    const QFileInfo fi(expandedPath);
 
     // Check if existing
     switch (d->m_acceptingKind) {
     case PathChooser::ExistingDirectory:
-        if (!fi.exists()) {
+        if (!filePath.exists()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" does not exist.").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" does not exist.").arg(filePath.toUserOutput());
             return false;
         }
-        if (!fi.isDir()) {
+        if (!filePath.isDir()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not a directory.").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" is not a directory.").arg(filePath.toUserOutput());
             return false;
         }
         break;
     case PathChooser::File:
-        if (!fi.exists()) {
+        if (!filePath.exists()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" does not exist.").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" does not exist.").arg(filePath.toUserOutput());
             return false;
         }
-        if (!fi.isFile()) {
+        if (!filePath.isFile()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not a file.").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" is not a file.").arg(filePath.toUserOutput());
             return false;
         }
         break;
     case PathChooser::SaveFile:
-        if (!fi.absoluteDir().exists()) {
+        if (!filePath.parentDir().exists()) {
             if (errorMessage)
-                *errorMessage = tr("The directory \"%1\" does not exist.").arg(QDir::toNativeSeparators(fi.absolutePath()));
+                *errorMessage = tr("The directory \"%1\" does not exist.").arg(filePath.toUserOutput());
             return false;
         }
-        if (fi.exists() && fi.isDir()) {
+        if (filePath.exists() && filePath.isDir()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not a file.").arg(QDir::toNativeSeparators(fi.absolutePath()));
+                *errorMessage = tr("The path \"%1\" is not a file.").arg(filePath.toUserOutput());
             return false;
         }
         break;
     case PathChooser::ExistingCommand:
-        if (!fi.exists()) {
+        if (!filePath.exists()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" does not exist.").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" does not exist.").arg(filePath.toUserOutput());
             return false;
         }
-        if (!fi.isFile() || !fi.isExecutable()) {
+        if (!filePath.isExecutableFile()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not an executable file.").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" is not an executable file.").arg(filePath.toUserOutput());
             return false;
         }
         break;
     case PathChooser::Directory:
-        if (fi.exists() && !fi.isDir()) {
+        if (filePath.exists() && !filePath.isDir()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not a directory.").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" is not a directory.").arg(filePath.toUserOutput());
+            return false;
+        }
+        if (HostOsInfo::isWindowsHost() && !filePath.startsWithDriveLetter()
+                && !filePath.startsWith("\\\\") && !filePath.startsWith("//")) {
+            if (errorMessage)
+                *errorMessage = tr("Invalid path \"%1\".").arg(filePath.toUserOutput());
             return false;
         }
         break;
     case PathChooser::Command:
-        if (fi.exists() && !fi.isExecutable()) {
+        if (filePath.exists() && !filePath.isExecutableFile()) {
             if (errorMessage)
-                *errorMessage = tr("Cannot execute \"%1\".").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("Cannot execute \"%1\".").arg(filePath.toUserOutput());
             return false;
         }
         break;
@@ -570,7 +624,7 @@ bool PathChooser::validatePath(FancyLineEdit *edit, QString *errorMessage) const
     }
 
     if (errorMessage)
-        *errorMessage = tr("Full path: \"%1\"").arg(QDir::toNativeSeparators(expandedPath));
+        *errorMessage = tr("Full path: \"%1\"").arg(filePath.toUserOutput());
     return true;
 }
 
@@ -584,16 +638,23 @@ QString PathChooser::label()
     return tr("Path:");
 }
 
-QString PathChooser::homePath()
+FilePath PathChooser::homePath()
 {
     // Return 'users/<name>/Documents' on Windows, since Windows explorer
     // does not let people actually display the contents of their home
     // directory. Alternatively, create a QtCreator-specific directory?
     if (HostOsInfo::isWindowsHost())
-        return QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);
-    return QDir::homePath();
+        return FilePath::fromString(QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation));
+    return FilePath::fromString(QDir::homePath());
 }
 
+/*!
+    Sets the kind of path the PathChooser will consider valid to select
+    to \a expected.
+
+    \sa Utils::PathChooser::Kind, expectedKind()
+*/
+
 void PathChooser::setExpectedKind(Kind expected)
 {
     if (d->m_acceptingKind == expected)
@@ -602,6 +663,12 @@ void PathChooser::setExpectedKind(Kind expected)
     d->m_lineEdit->validate();
 }
 
+/*!
+    Returns the kind of path the PathChooser considers valid to select.
+
+    \sa Utils::PathChooser::Kind, setExpectedKind()
+*/
+
 PathChooser::Kind PathChooser::expectedKind() const
 {
     return d->m_acceptingKind;
@@ -628,7 +695,7 @@ QString PathChooser::promptDialogFilter() const
     return d->m_dialogFilter;
 }
 
-void PathChooser::setInitialBrowsePathBackup(const QString &path)
+void PathChooser::setInitialBrowsePathBackup(const FilePath &path)
 {
     d->m_initialBrowsePathOverride = path;
 }
@@ -649,9 +716,9 @@ FancyLineEdit *PathChooser::lineEdit() const
     return d->m_lineEdit;
 }
 
-QString PathChooser::toolVersion(const QString &binary, const QStringList &arguments)
+QString PathChooser::toolVersion(const CommandLine &cmd)
 {
-    return BinaryVersionToolTipEventFilter::toolVersion({binary, arguments});
+    return BinaryVersionToolTipEventFilter::toolVersion(cmd);
 }
 
 void PathChooser::installLineEditVersionToolTip(QLineEdit *le, const QStringList &arguments)
@@ -665,7 +732,7 @@ void PathChooser::setHistoryCompleter(const QString &historyKey, bool restoreLas
     d->m_lineEdit->setHistoryCompleter(historyKey, restoreLastItemFromHistory);
 }
 
-void PathChooser::setMacroExpander(MacroExpander *macroExpander)
+void PathChooser::setMacroExpander(const MacroExpander *macroExpander)
 {
     d->m_macroExpander = macroExpander;
 }
