@@ -25,38 +25,48 @@
 
 #include "persistentsettings.h"
 
+#include "fileutils.h"
+#include "qtcassert.h"
+
+#include <QCoreApplication>
+#include <QDateTime>
 #include <QDebug>
 #include <QDir>
+#include <QRect>
+#include <QRegularExpression>
 #include <QStack>
+#include <QTextStream>
 #include <QXmlStreamAttributes>
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
-#include <QDateTime>
-#include <QTextStream>
-#include <QRegExp>
-#include <QRect>
 
 #ifdef QT_GUI_LIB
 #include <QMessageBox>
 #endif
 
-#include <utils/qtcassert.h>
-
 // Read and write rectangle in X11 resource syntax "12x12+4+3"
 static QString rectangleToString(const QRect &r)
 {
     QString result;
-    QTextStream(&result) << r.width() << 'x' << r.height() << forcesign << r.x() << r.y();
+    QTextStream str(&result);
+    str << r.width() << 'x' << r.height();
+    if (r.x() >= 0)
+        str << '+';
+    str << r.x();
+    if (r.y() >= 0)
+        str << '+';
+    str << r.y();
     return result;
 }
 
 static QRect stringToRectangle(const QString &v)
 {
-    static QRegExp pattern(QLatin1String("(\\d+)x(\\d+)([-+]\\d+)([-+]\\d+)"));
+    static QRegularExpression pattern("^(\\d+)x(\\d+)([-+]\\d+)([-+]\\d+)$");
     Q_ASSERT(pattern.isValid());
-    return pattern.exactMatch(v) ?
-        QRect(QPoint(pattern.cap(3).toInt(), pattern.cap(4).toInt()),
-              QSize(pattern.cap(1).toInt(), pattern.cap(2).toInt())) :
+    const QRegularExpressionMatch match = pattern.match(v);
+    return match.hasMatch() ?
+        QRect(QPoint(match.captured(3).toInt(), match.captured(4).toInt()),
+              QSize(match.captured(1).toInt(), match.captured(2).toInt())) :
         QRect();
 }
 
@@ -174,19 +184,19 @@ void ParseValueStackEntry::addChild(const QString &key, const QVariant &v)
 class ParseContext : public Context
 {
 public:
-    QVariantMap parse(QFile &file);
+    QVariantMap parse(const FilePath &file);
 
 private:
     enum Element { QtCreatorElement, DataElement, VariableElement,
                    SimpleValueElement, ListValueElement, MapValueElement, UnknownElement };
 
-    Element element(const QStringRef &r) const;
+    Element element(const StringView &r) const;
     static inline bool isValueElement(Element e)
         { return e == SimpleValueElement || e == ListValueElement || e == MapValueElement; }
     QVariant readSimpleValue(QXmlStreamReader &r, const QXmlStreamAttributes &attributes) const;
 
     bool handleStartElement(QXmlStreamReader &r);
-    bool handleEndElement(const QStringRef &name);
+    bool handleEndElement(const StringView &name);
 
     static QString formatWarning(const QXmlStreamReader &r, const QString &message);
 
@@ -195,9 +205,9 @@ private:
     QString m_currentVariableName;
 };
 
-QVariantMap ParseContext::parse(QFile &file)
+QVariantMap ParseContext::parse(const FilePath &file)
 {
-    QXmlStreamReader r(&file);
+    QXmlStreamReader r(file.fileContents());
 
     m_result.clear();
     m_currentVariableName.clear();
@@ -225,7 +235,7 @@ QVariantMap ParseContext::parse(QFile &file)
 
 bool ParseContext::handleStartElement(QXmlStreamReader &r)
 {
-    const QStringRef name = r.name();
+    const StringView name = r.name();
     const Element e = element(name);
     if (e == VariableElement) {
         m_currentVariableName = r.readElementText();
@@ -260,7 +270,7 @@ bool ParseContext::handleStartElement(QXmlStreamReader &r)
     return false;
 }
 
-bool ParseContext::handleEndElement(const QStringRef &name)
+bool ParseContext::handleEndElement(const StringView &name)
 {
     const Element e = element(name);
     if (ParseContext::isValueElement(e)) {
@@ -281,7 +291,7 @@ QString ParseContext::formatWarning(const QXmlStreamReader &r, const QString &me
 {
     QString result = QLatin1String("Warning reading ");
     if (const QIODevice *device = r.device())
-        if (const QFile *file = qobject_cast<const QFile *>(device))
+        if (const auto file = qobject_cast<const QFile *>(device))
             result += QDir::toNativeSeparators(file->fileName()) + QLatin1Char(':');
     result += QString::number(r.lineNumber());
     result += QLatin1String(": ");
@@ -289,7 +299,7 @@ QString ParseContext::formatWarning(const QXmlStreamReader &r, const QString &me
     return result;
 }
 
-ParseContext::Element ParseContext::element(const QStringRef &r) const
+ParseContext::Element ParseContext::element(const StringView &r) const
 {
     if (r == valueElement)
         return SimpleValueElement;
@@ -309,7 +319,7 @@ ParseContext::Element ParseContext::element(const QStringRef &r) const
 QVariant ParseContext::readSimpleValue(QXmlStreamReader &r, const QXmlStreamAttributes &attributes) const
 {
     // Simple value
-    const QStringRef type = attributes.value(typeAttribute);
+    const StringView type = attributes.value(typeAttribute);
     const QString text = r.readElementText();
     if (type == QLatin1String("QChar")) { // Workaround: QTBUG-12345
         QTC_ASSERT(text.size() == 1, return QVariant());
@@ -321,15 +331,13 @@ QVariant ParseContext::readSimpleValue(QXmlStreamReader &r, const QXmlStreamAttr
     }
     QVariant value;
     value.setValue(text);
-    value.convert(QMetaType::type(type.toLatin1().data()));
+    value.convert(QMetaType::type(type.toLatin1().constData()));
     return value;
 }
 
 // =================================== PersistentSettingsReader
 
-PersistentSettingsReader::PersistentSettingsReader()
-{
-}
+PersistentSettingsReader::PersistentSettingsReader() = default;
 
 QVariant PersistentSettingsReader::restoreValue(const QString &variable, const QVariant &defaultValue) const
 {
@@ -343,16 +351,15 @@ QVariantMap PersistentSettingsReader::restoreValues() const
     return m_valueMap;
 }
 
-bool PersistentSettingsReader::load(const FileName &fileName)
+bool PersistentSettingsReader::load(const FilePath &fileName)
 {
     m_valueMap.clear();
 
-    QFile file(fileName.toString());
-    if (!file.open(QIODevice::ReadOnly|QIODevice::Text))
+    if (fileName.fileSize() == 0) // skip empty files
         return false;
+
     ParseContext ctx;
-    m_valueMap = ctx.parse(file);
-    file.close();
+    m_valueMap = ctx.parse(fileName);
     return true;
 }
 
@@ -369,15 +376,17 @@ static void writeVariantValue(QXmlStreamWriter &w, const Context &ctx,
 {
     switch (static_cast<int>(variant.type())) {
     case static_cast<int>(QVariant::StringList):
-    case static_cast<int>(QVariant::List):
+    case static_cast<int>(QVariant::List): {
         w.writeStartElement(ctx.valueListElement);
         w.writeAttribute(ctx.typeAttribute, QLatin1String(QVariant::typeToName(QVariant::List)));
         if (!key.isEmpty())
             w.writeAttribute(ctx.keyAttribute, key);
-        foreach (const QVariant &var, variant.toList())
+        const QList<QVariant> list = variant.toList();
+        for (const QVariant &var : list)
             writeVariantValue(w, ctx, var);
         w.writeEndElement();
         break;
+    }
     case static_cast<int>(QVariant::Map): {
         w.writeStartElement(ctx.valueMapElement);
         w.writeAttribute(ctx.typeAttribute, QLatin1String(QVariant::typeToName(QVariant::Map)));
@@ -411,15 +420,10 @@ static void writeVariantValue(QXmlStreamWriter &w, const Context &ctx,
     }
 }
 
-PersistentSettingsWriter::PersistentSettingsWriter(const FileName &fileName, const QString &docType) :
+PersistentSettingsWriter::PersistentSettingsWriter(const FilePath &fileName, const QString &docType) :
     m_fileName(fileName), m_docType(docType)
 { }
 
-PersistentSettingsWriter::~PersistentSettingsWriter()
-{
-    write(m_savedData, 0);
-}
-
 bool PersistentSettingsWriter::save(const QVariantMap &data, QString *errorString) const
 {
     if (data == m_savedData)
@@ -440,7 +444,7 @@ bool PersistentSettingsWriter::save(const QVariantMap &data, QWidget *parent) co
 }
 #endif // QT_GUI_LIB
 
-FileName PersistentSettingsWriter::fileName() const
+FilePath PersistentSettingsWriter::fileName() const
 { return m_fileName; }
 
 //** * @brief Set contents of file (e.g. from data read from it). */
@@ -451,9 +455,8 @@ void PersistentSettingsWriter::setContents(const QVariantMap &data)
 
 bool PersistentSettingsWriter::write(const QVariantMap &data, QString *errorString) const
 {
-    QDir tmp;
-    tmp.mkpath(m_fileName.toFileInfo().path());
-    FileSaver saver(m_fileName.toString(), QIODevice::Text);
+    m_fileName.parentDir().ensureWritableDir();
+    FileSaver saver(m_fileName, QIODevice::Text);
     if (!saver.hasError()) {
         const Context ctx;
         QXmlStreamWriter w(saver.file());
