@@ -25,16 +25,17 @@
 
 #include "androidmanager.h"
 
-#include "androidbuildapkstep.h"
-#include "androidconstants.h"
-#include "androidconfigurations.h"
-#include "androidrunconfiguration.h"
-#include "androidglobal.h"
-#include "androidtoolchain.h"
-#include "androiddeployqtstep.h"
-#include "androidqtversion.h"
 #include "androidavdmanager.h"
+#include "androidbuildapkstep.h"
+#include "androidconfigurations.h"
+#include "androidconstants.h"
+#include "androiddeployqtstep.h"
+#include "androiddevice.h"
+#include "androidglobal.h"
+#include "androidqtversion.h"
+#include "androidrunconfiguration.h"
 #include "androidsdkmanager.h"
+#include "androidtoolchain.h"
 
 #include <coreplugin/documentmanager.h>
 #include <coreplugin/messagemanager.h>
@@ -44,15 +45,21 @@
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectnodes.h>
 #include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/session.h>
 #include <projectexplorer/target.h>
+#include <projectexplorer/buildsystem.h>
 
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtsupportconstants.h>
+#include <qtsupport/baseqtversion.h>
+
+#include <cmakeprojectmanager/cmakeprojectconstants.h>
 
 #include <utils/algorithm.h>
 #include <utils/qtcassert.h>
-#include <utils/synchronousprocess.h>
+#include <utils/qtcprocess.h>
+#include <utils/stringutils.h>
 
 #include <QApplication>
 #include <QDir>
@@ -64,67 +71,45 @@
 #include <QLoggingCategory>
 #include <QMessageBox>
 #include <QProcess>
-#include <QRegExp>
 #include <QRegularExpression>
 #include <QVersionNumber>
 
-namespace {
-    const QLatin1String AndroidManifestName("AndroidManifest.xml");
-    const QLatin1String AndroidDefaultPropertiesName("project.properties");
-    const QLatin1String AndroidDeviceSn("AndroidDeviceSerialNumber");
-    const QLatin1String AndroidDeviceAbis("AndroidDeviceAbis");
-    const QLatin1String ApiLevelKey("AndroidVersion.ApiLevel");
-    const QString packageNameRegEx("(?<token>package: )(.*?)(name=)'(?<target>.*?)'");
-    const QString activityRegEx("(?<token>launchable-activity: )(.*?)(name=)'(?<target>.*?)'");
-    const QString apkVersionRegEx("(?<token>package: )(.*?)(versionCode=)'(?<target>.*?)'");
-    const QString versionCodeRegEx("(?<token>versionCode=)(?<version>\\d*)");
-    const QString qtcSignature("This file is generated by QtCreator to be read by androiddeployqt and should not be modified by hand.");
-
-    static Q_LOGGING_CATEGORY(androidManagerLog, "qtc.android.androidManager", QtWarningMsg)
-
-    QString parseAaptOutput(const QString &output, const QString &regEx) {
-        const QRegularExpression regRx(regEx,
-                                       QRegularExpression::CaseInsensitiveOption |
-                                       QRegularExpression::MultilineOption);
-        QRegularExpressionMatch match = regRx.match(output);
-        if (match.hasMatch())
-            return match.captured("target");
-        return QString();
-    };
-} // anonymous namespace
-
-
 using namespace ProjectExplorer;
 using namespace Utils;
 
+using namespace Android::Internal;
+
 namespace Android {
 
-using namespace Internal;
+const char AndroidManifestName[] = "AndroidManifest.xml";
+const char AndroidDeviceSn[] = "AndroidDeviceSerialNumber";
+const char AndroidDeviceAbis[] = "AndroidDeviceAbis";
+const char ApiLevelKey[] = "AndroidVersion.ApiLevel";
+const char qtcSignature[] = "This file is generated by QtCreator to be read by "
+                            "androiddeployqt and should not be modified by hand.";
+
+static Q_LOGGING_CATEGORY(androidManagerLog, "qtc.android.androidManager", QtWarningMsg)
 
 class Library
 {
 public:
-    Library()
-    { level = -1; }
-    int level;
+    int level = -1;
     QStringList dependencies;
     QString name;
 };
 
 using LibrariesMap = QMap<QString, Library>;
 
-static bool openXmlFile(QDomDocument &doc, const Utils::FilePath &fileName);
-static bool openManifest(ProjectExplorer::Target *target, QDomDocument &doc);
+static bool openXmlFile(QDomDocument &doc, const FilePath &fileName);
+static bool openManifest(const Target *target, QDomDocument &doc);
 static int parseMinSdk(const QDomElement &manifestElem);
 
-static const ProjectNode *currentProjectNode(Target *target)
+static const ProjectNode *currentProjectNode(const Target *target)
 {
-    if (RunConfiguration *rc = target->activeRunConfiguration())
-        return target->project()->findNodeForBuildKey(rc->buildKey());
-    return nullptr;
+    return target->project()->findNodeForBuildKey(target->activeBuildKey());
 }
 
-QString AndroidManager::packageName(ProjectExplorer::Target *target)
+QString AndroidManager::packageName(const Target *target)
 {
     QDomDocument doc;
     if (!openManifest(target, doc))
@@ -133,7 +118,7 @@ QString AndroidManager::packageName(ProjectExplorer::Target *target)
     return manifestElem.attribute(QLatin1String("package"));
 }
 
-QString AndroidManager::packageName(const Utils::FilePath &manifestFile)
+QString AndroidManager::packageName(const FilePath &manifestFile)
 {
     QDomDocument doc;
     if (!openXmlFile(doc, manifestFile))
@@ -142,80 +127,13 @@ QString AndroidManager::packageName(const Utils::FilePath &manifestFile)
     return manifestElem.attribute(QLatin1String("package"));
 }
 
-bool AndroidManager::packageInstalled(const QString &deviceSerial,
-                                      const QString &packageName)
-{
-    if (deviceSerial.isEmpty() || packageName.isEmpty())
-        return false;
-    QStringList args = AndroidDeviceInfo::adbSelector(deviceSerial);
-    args << "shell" << "pm" << "list" << "packages";
-    QStringList lines = runAdbCommand(args).stdOut().split(QRegularExpression("[\\n\\r]"),
-                                                           QString::SkipEmptyParts);
-    for (const QString &line : lines) {
-        // Don't want to confuse com.abc.xyz with com.abc.xyz.def so check with
-        // endsWith
-        if (line.endsWith(packageName))
-            return true;
-    }
-    return false;
-}
-
-int AndroidManager::packageVersionCode(const QString &deviceSerial,
-                                              const QString &packageName)
-{
-    if (deviceSerial.isEmpty() || packageName.isEmpty())
-        return -1;
-
-    QStringList args = AndroidDeviceInfo::adbSelector(deviceSerial);
-    args << "shell" << "dumpsys" << "package" << packageName;
-    const QRegularExpression regRx(versionCodeRegEx,
-                                   QRegularExpression::CaseInsensitiveOption |
-                                   QRegularExpression::MultilineOption);
-    QRegularExpressionMatch match = regRx.match(runAdbCommand(args).stdOut());
-    if (match.hasMatch())
-        return match.captured("version").toInt();
-
-    return -1;
-}
-
-void AndroidManager::apkInfo(const Utils::FilePath &apkPath,
-                                    QString *packageName,
-                                    int *version,
-                                    QString *activityPath)
-{
-    SdkToolResult result;
-    result = runAaptCommand({"dump", "badging", apkPath.toString()});
-
-    QString packageStr;
-    if (activityPath) {
-        packageStr = parseAaptOutput(result.stdOut(), packageNameRegEx);
-        QString path = parseAaptOutput(result.stdOut(), activityRegEx);
-        if (!packageStr.isEmpty() && !path.isEmpty())
-            *activityPath = packageStr + '/' + path;
-    }
-
-    if (packageName) {
-        *packageName = activityPath ? packageStr :
-                                      parseAaptOutput(result.stdOut(), packageNameRegEx);
-    }
-
-    if (version) {
-        QString versionStr = parseAaptOutput(result.stdOut(), apkVersionRegEx);
-        *version = versionStr.toInt();
-    }
-}
-
-QString AndroidManager::intentName(ProjectExplorer::Target *target)
-{
-    return packageName(target) + QLatin1Char('/') + activityName(target);
-}
-
-QString AndroidManager::activityName(ProjectExplorer::Target *target)
+QString AndroidManager::activityName(const Target *target)
 {
     QDomDocument doc;
     if (!openManifest(target, doc))
         return QString();
-    QDomElement activityElem = doc.documentElement().firstChildElement(QLatin1String("application")).firstChildElement(QLatin1String("activity"));
+    QDomElement activityElem = doc.documentElement().firstChildElement(
+                QLatin1String("application")).firstChildElement(QLatin1String("activity"));
     return activityElem.attribute(QLatin1String("android:name"));
 }
 
@@ -224,34 +142,39 @@ QString AndroidManager::activityName(ProjectExplorer::Target *target)
     of the kit is returned if the manifest file of the APK cannot be found
     or parsed.
 */
-int AndroidManager::minimumSDK(ProjectExplorer::Target *target)
+int AndroidManager::minimumSDK(const Target *target)
 {
     QDomDocument doc;
     if (!openXmlFile(doc, AndroidManager::manifestSourcePath(target)))
         return minimumSDK(target->kit());
-    return parseMinSdk(doc.documentElement());
+    const int minSdkVersion = parseMinSdk(doc.documentElement());
+    if (minSdkVersion == 0)
+        return AndroidManager::defaultMinimumSDK(QtSupport::QtKitAspect::qtVersion(target->kit()));
+    return minSdkVersion;
 }
 
 /*!
     Returns the minimum Android API level required by the kit to compile. -1 is
     returned if the kit does not support Android.
 */
-int AndroidManager::minimumSDK(const ProjectExplorer::Kit *kit)
+int AndroidManager::minimumSDK(const Kit *kit)
 {
-    int minSDKVersion = -1;
-    QtSupport::BaseQtVersion *version = QtSupport::QtKitAspect::qtVersion(kit);
+    int minSdkVersion = -1;
+    QtSupport::QtVersion *version = QtSupport::QtKitAspect::qtVersion(kit);
     if (version && version->targetDeviceTypes().contains(Constants::ANDROID_DEVICE_TYPE)) {
-        Utils::FilePath stockManifestFilePath = Utils::FilePath::fromUserInput(
+        FilePath stockManifestFilePath = FilePath::fromUserInput(
             version->prefix().toString() + "/src/android/templates/AndroidManifest.xml");
         QDomDocument doc;
         if (openXmlFile(doc, stockManifestFilePath)) {
-            minSDKVersion = parseMinSdk(doc.documentElement());
+            minSdkVersion = parseMinSdk(doc.documentElement());
         }
     }
-    return minSDKVersion;
+    if (minSdkVersion == 0)
+        return AndroidManager::defaultMinimumSDK(version);
+    return minSdkVersion;
 }
 
-QString AndroidManager::buildTargetSDK(ProjectExplorer::Target *target)
+QString AndroidManager::buildTargetSDK(const Target *target)
 {
     if (auto bc = target->activeBuildConfiguration()) {
         if (auto androidBuildApkStep = bc->buildSteps()->firstOfType<AndroidBuildApkStep>())
@@ -265,17 +188,17 @@ QString AndroidManager::buildTargetSDK(ProjectExplorer::Target *target)
 
 QStringList AndroidManager::applicationAbis(const Target *target)
 {
-    auto qt = static_cast<AndroidQtVersion *>(QtSupport::QtKitAspect::qtVersion(target->kit()));
-    return qt->androidAbis();
+    auto qt = dynamic_cast<AndroidQtVersion *>(QtSupport::QtKitAspect::qtVersion(target->kit()));
+    return qt ? qt->androidAbis() : QStringList();
 }
 
 QString AndroidManager::archTriplet(const QString &abi)
 {
-    if (abi == "x86") {
+    if (abi == ProjectExplorer::Constants::ANDROID_ABI_X86) {
         return {"i686-linux-android"};
-    } else if (abi == "x86_64") {
+    } else if (abi == ProjectExplorer::Constants::ANDROID_ABI_X86_64) {
         return {"x86_64-linux-android"};
-    } else if (abi == "arm64-v8a") {
+    } else if (abi == ProjectExplorer::Constants::ANDROID_ABI_ARM64_V8A) {
         return {"aarch64-linux-android"};
     }
     return {"arm-linux-androideabi"};
@@ -283,11 +206,11 @@ QString AndroidManager::archTriplet(const QString &abi)
 
 QJsonObject AndroidManager::deploymentSettings(const Target *target)
 {
-    QtSupport::BaseQtVersion *qt = QtSupport::QtKitAspect::qtVersion(target->kit());
+    QtSupport::QtVersion *qt = QtSupport::QtKitAspect::qtVersion(target->kit());
     if (!qt)
         return {};
 
-    auto tc = ProjectExplorer::ToolChainKitAspect::toolChain(target->kit(), ProjectExplorer::Constants::CXX_LANGUAGE_ID);
+    auto tc = ToolChainKitAspect::cxxToolChain(target->kit());
     if (!tc || tc->typeId() != Constants::ANDROID_TOOLCHAIN_TYPEID)
         return {};
     QJsonObject settings;
@@ -295,17 +218,17 @@ QJsonObject AndroidManager::deploymentSettings(const Target *target)
     settings["qt"] = qt->prefix().toString();
     settings["ndk"] = AndroidConfigurations::currentConfig().ndkLocation(qt).toString();
     settings["sdk"] = AndroidConfigurations::currentConfig().sdkLocation().toString();
-    if (qt->qtVersion() < QtSupport::QtVersionNumber(5, 14, 0)) {
+    if (!qt->supportsMultipleQtAbis()) {
         const QStringList abis = applicationAbis(target);
         QTC_ASSERT(abis.size() == 1, return {});
-        settings["stdcpp-path"] = AndroidConfigurations::currentConfig().toolchainPath(qt)
-                                      .pathAppended("sysroot/usr/lib/")
-                                      .pathAppended(archTriplet(abis.first()))
-                                      .pathAppended("libc++_shared.so").toString();
+        settings["stdcpp-path"] = (AndroidConfigurations::currentConfig().toolchainPath(qt)
+                                      / "sysroot/usr/lib"
+                                      / archTriplet(abis.first())
+                                      / "libc++_shared.so").toString();
     } else {
         settings["stdcpp-path"] = AndroidConfigurations::currentConfig()
                                       .toolchainPath(qt)
-                                      .pathAppended("sysroot/usr/lib/")
+                                      .pathAppended("sysroot/usr/lib")
                                       .toString();
     }
     settings["toolchain-prefix"] =  "llvm";
@@ -323,23 +246,60 @@ bool AndroidManager::isQtCreatorGenerated(const FilePath &deploymentFile)
     return QJsonDocument::fromJson(f.readAll()).object()["_description"].toString() == qtcSignature;
 }
 
-Utils::FilePath AndroidManager::dirPath(const ProjectExplorer::Target *target)
+FilePath AndroidManager::androidBuildDirectory(const Target *target)
 {
-    if (auto *bc = target->activeBuildConfiguration())
-        return bc->buildDirectory().pathAppended(Constants::ANDROID_BUILDDIRECTORY);
-    return Utils::FilePath();
+    return buildDirectory(target) / Constants::ANDROID_BUILD_DIRECTORY;
 }
 
-Utils::FilePath AndroidManager::apkPath(const ProjectExplorer::Target *target)
+bool AndroidManager::isQt5CmakeProject(const ProjectExplorer::Target *target)
 {
-    QTC_ASSERT(target, return Utils::FilePath());
+    const QtSupport::QtVersion *qt = QtSupport::QtKitAspect::qtVersion(target->kit());
+    const bool isQt5 = qt && qt->qtVersion() < QtSupport::QtVersionNumber{6, 0, 0};
+    const Core::Context cmakeCtx = Core::Context(CMakeProjectManager::Constants::CMAKE_PROJECT_ID);
+    const bool isCmakeProject = (target->project()->projectContext() == cmakeCtx);
+    return isQt5 && isCmakeProject;
+}
+
+FilePath AndroidManager::buildDirectory(const Target *target)
+{
+    if (const BuildSystem *bs = target->buildSystem()) {
+        const QString buildKey = target->activeBuildKey();
+
+        // Get the target build dir based on the settings file path
+        FilePath buildDir;
+        const ProjectNode *node = target->project()->findNodeForBuildKey(buildKey);
+        if (node) {
+            const QString settingsFile = node->data(Constants::AndroidDeploySettingsFile).toString();
+            buildDir = FilePath::fromUserInput(settingsFile).parentDir();
+        }
+
+        if (!buildDir.isEmpty())
+            return buildDir;
+
+        // Otherwise fallback to target working dir
+        buildDir = bs->buildTarget(target->activeBuildKey()).workingDirectory;
+        if (isQt5CmakeProject(target)) {
+            // Return the main build dir and not the android libs dir
+            const QString libsDir = QString(Constants::ANDROID_BUILD_DIRECTORY) + "/libs";
+            Utils::FilePath parentDuildDir = buildDir.parentDir();
+            if (parentDuildDir.endsWith(libsDir) || libsDir.endsWith(libsDir + "/"))
+                return parentDuildDir.parentDir().parentDir();
+        }
+        return buildDir;
+    }
+    return {};
+}
+
+FilePath AndroidManager::apkPath(const Target *target)
+{
+    QTC_ASSERT(target, return {});
 
     auto bc = target->activeBuildConfiguration();
     if (!bc)
         return {};
     auto buildApkStep = bc->buildSteps()->firstOfType<AndroidBuildApkStep>();
     if (!buildApkStep)
-        return Utils::FilePath();
+        return {};
 
     QString apkPath("build/outputs/apk/android-build-");
     if (buildApkStep->signPackage())
@@ -347,7 +307,7 @@ Utils::FilePath AndroidManager::apkPath(const ProjectExplorer::Target *target)
     else
         apkPath += QLatin1String("debug.apk");
 
-    return dirPath(target).pathAppended(apkPath);
+    return androidBuildDirectory(target) / apkPath;
 }
 
 bool AndroidManager::matchedAbis(const QStringList &deviceAbis, const QStringList &appAbis)
@@ -370,25 +330,25 @@ QString AndroidManager::devicePreferredAbi(const QStringList &deviceAbis, const
 
 Abi AndroidManager::androidAbi2Abi(const QString &androidAbi)
 {
-    if (androidAbi == "arm64-v8a") {
+    if (androidAbi == ProjectExplorer::Constants::ANDROID_ABI_ARM64_V8A) {
         return Abi{Abi::Architecture::ArmArchitecture,
                    Abi::OS::LinuxOS,
                    Abi::OSFlavor::AndroidLinuxFlavor,
                    Abi::BinaryFormat::ElfFormat,
                    64, androidAbi};
-    } else if (androidAbi == "armeabi-v7a") {
+    } else if (androidAbi == ProjectExplorer::Constants::ANDROID_ABI_ARMEABI_V7A) {
         return Abi{Abi::Architecture::ArmArchitecture,
                    Abi::OS::LinuxOS,
                    Abi::OSFlavor::AndroidLinuxFlavor,
                    Abi::BinaryFormat::ElfFormat,
                    32, androidAbi};
-    } else if (androidAbi == "x86_64") {
+    } else if (androidAbi == ProjectExplorer::Constants::ANDROID_ABI_X86_64) {
         return Abi{Abi::Architecture::X86Architecture,
                    Abi::OS::LinuxOS,
                    Abi::OSFlavor::AndroidLinuxFlavor,
                    Abi::BinaryFormat::ElfFormat,
                    64, androidAbi};
-    } else if (androidAbi == "x86") {
+    } else if (androidAbi == ProjectExplorer::Constants::ANDROID_ABI_X86) {
         return Abi{Abi::Architecture::X86Architecture,
                    Abi::OS::LinuxOS,
                    Abi::OSFlavor::AndroidLinuxFlavor,
@@ -403,7 +363,7 @@ Abi AndroidManager::androidAbi2Abi(const QString &androidAbi)
     }
 }
 
-Utils::FilePath AndroidManager::manifestSourcePath(ProjectExplorer::Target *target)
+FilePath AndroidManager::manifestSourcePath(const Target *target)
 {
     if (const ProjectNode *node = currentProjectNode(target)) {
         const QString packageSource
@@ -417,12 +377,12 @@ Utils::FilePath AndroidManager::manifestSourcePath(ProjectExplorer::Target *targ
     return manifestPath(target);
 }
 
-Utils::FilePath AndroidManager::manifestPath(ProjectExplorer::Target *target)
+FilePath AndroidManager::manifestPath(const Target *target)
 {
     QVariant manifest = target->namedSettings(AndroidManifestName);
     if (manifest.isValid())
         return manifest.value<FilePath>();
-    return dirPath(target).pathAppended(AndroidManifestName);
+    return androidBuildDirectory(target).pathAppended(AndroidManifestName);
 }
 
 void AndroidManager::setManifestPath(Target *target, const FilePath &path)
@@ -430,24 +390,19 @@ void AndroidManager::setManifestPath(Target *target, const FilePath &path)
      target->setNamedSettings(AndroidManifestName, QVariant::fromValue(path));
 }
 
-Utils::FilePath AndroidManager::defaultPropertiesPath(ProjectExplorer::Target *target)
-{
-    return dirPath(target).pathAppended(AndroidDefaultPropertiesName);
-}
-
-QString AndroidManager::deviceSerialNumber(ProjectExplorer::Target *target)
+QString AndroidManager::deviceSerialNumber(const Target *target)
 {
     return target->namedSettings(AndroidDeviceSn).toString();
 }
 
-void AndroidManager::setDeviceSerialNumber(ProjectExplorer::Target *target, const QString &deviceSerialNumber)
+void AndroidManager::setDeviceSerialNumber(Target *target, const QString &deviceSerialNumber)
 {
-    qCDebug(androidManagerLog) << "Device serial for the target changed"
+    qCDebug(androidManagerLog) << "Target device serial changed:"
                                << target->displayName() << deviceSerialNumber;
     target->setNamedSettings(AndroidDeviceSn, deviceSerialNumber);
 }
 
-static QString preferredAbi(const QStringList &appAbis, Target *target)
+static QString preferredAbi(const QStringList &appAbis, const Target *target)
 {
     const auto deviceAbis = target->namedSettings(AndroidDeviceAbis).toStringList();
     for (const auto &abi : deviceAbis) {
@@ -457,97 +412,122 @@ static QString preferredAbi(const QStringList &appAbis, Target *target)
     return {};
 }
 
-QString AndroidManager::apkDevicePreferredAbi(Target *target)
+QString AndroidManager::apkDevicePreferredAbi(const Target *target)
 {
-    auto libsPath = dirPath(target).pathAppended("libs");
+    const FilePath libsPath = androidBuildDirectory(target).pathAppended("libs");
+    if (!libsPath.exists()) {
+        if (const ProjectNode *node = currentProjectNode(target)) {
+            const QString abi = preferredAbi(
+                        node->data(Android::Constants::AndroidAbis).toStringList(), target);
+            if (abi.isEmpty())
+                return node->data(Android::Constants::AndroidAbi).toString();
+        }
+    }
     QStringList apkAbis;
-    for (const auto &abi : QDir{libsPath.toString()}.entryList(QDir::Dirs | QDir::NoDotAndDotDot))
-        if (!QDir{libsPath.pathAppended(abi).toString()}.entryList(QStringList("*.so"), QDir::Files | QDir::NoDotAndDotDot).isEmpty())
-            apkAbis << abi;
+    const FilePaths libsPaths = libsPath.dirEntries(QDir::Dirs | QDir::NoDotAndDotDot);
+    for (const FilePath &abiDir : libsPaths) {
+        if (!abiDir.dirEntries({{"*.so"}, QDir::Files | QDir::NoDotAndDotDot}).isEmpty())
+            apkAbis << abiDir.fileName();
+    }
     return preferredAbi(apkAbis, target);
 }
 
-void AndroidManager::setDeviceAbis(ProjectExplorer::Target *target, const QStringList &deviceAbis)
+void AndroidManager::setDeviceAbis(Target *target, const QStringList &deviceAbis)
 {
     target->setNamedSettings(AndroidDeviceAbis, deviceAbis);
 }
 
-int AndroidManager::deviceApiLevel(ProjectExplorer::Target *target)
+int AndroidManager::deviceApiLevel(const Target *target)
 {
     return target->namedSettings(ApiLevelKey).toInt();
 }
 
-void AndroidManager::setDeviceApiLevel(ProjectExplorer::Target *target, int level)
+void AndroidManager::setDeviceApiLevel(Target *target, int level)
 {
-    qCDebug(androidManagerLog) << "Device API level for the target changed"
+    qCDebug(androidManagerLog) << "Target device API level changed:"
                                << target->displayName() << level;
     target->setNamedSettings(ApiLevelKey, level);
 }
 
-QPair<int, int> AndroidManager::apiLevelRange()
+int AndroidManager::defaultMinimumSDK(const QtSupport::QtVersion *qtVersion)
 {
-    return qMakePair(16, 29);
+    if (qtVersion && qtVersion->qtVersion() >= QtSupport::QtVersionNumber{6, 0})
+        return 23;
+    else if (qtVersion && qtVersion->qtVersion() >= QtSupport::QtVersionNumber{5, 13})
+        return 21;
+    else
+        return 16;
 }
 
 QString AndroidManager::androidNameForApiLevel(int x)
 {
     switch (x) {
+    case 2:
+        return QLatin1String("Android 1.1");
+    case 3:
+        return QLatin1String("Android 1.5 (Cupcake)");
     case 4:
-        return QLatin1String("Android 1.6");
+        return QLatin1String("Android 1.6 (Donut)");
     case 5:
-        return QLatin1String("Android 2.0");
+        return QLatin1String("Android 2.0 (Eclair)");
     case 6:
-        return QLatin1String("Android 2.0.1");
+        return QLatin1String("Android 2.0.1 (Eclair)");
     case 7:
-        return QLatin1String("Android 2.1.x");
+        return QLatin1String("Android 2.1 (Eclair)");
     case 8:
-        return QLatin1String("Android 2.2.x");
+        return QLatin1String("Android 2.2 (Froyo)");
     case 9:
-        return QLatin1String("Android 2.3, 2.3.1, 2.3.2");
+        return QLatin1String("Android 2.3 (Gingerbread)");
     case 10:
-        return QLatin1String("Android 2.3.3, 2.3.4");
+        return QLatin1String("Android 2.3.3 (Gingerbread)");
     case 11:
-        return QLatin1String("Android 3.0.x");
+        return QLatin1String("Android 3.0 (Honeycomb)");
     case 12:
-        return QLatin1String("Android 3.1.x");
+        return QLatin1String("Android 3.1 (Honeycomb)");
     case 13:
-        return QLatin1String("Android 3.2");
+        return QLatin1String("Android 3.2 (Honeycomb)");
     case 14:
-        return QLatin1String("Android 4.0, 4.0.1, 4.0.2");
+        return QLatin1String("Android 4.0 (IceCreamSandwich)");
     case 15:
-        return QLatin1String("Android 4.0.3, 4.0.4");
+        return QLatin1String("Android 4.0.3 (IceCreamSandwich)");
     case 16:
-        return QLatin1String("Android 4.1, 4.1.1");
+        return QLatin1String("Android 4.1 (Jelly Bean)");
     case 17:
-        return QLatin1String("Android 4.2, 4.2.2");
+        return QLatin1String("Android 4.2 (Jelly Bean)");
     case 18:
-        return QLatin1String("Android 4.3");
+        return QLatin1String("Android 4.3 (Jelly Bean)");
     case 19:
-        return QLatin1String("Android 4.4");
+        return QLatin1String("Android 4.4 (KitKat)");
     case 20:
-        return QLatin1String("Android 4.4W");
+        return QLatin1String("Android 4.4W (KitKat Wear)");
     case 21:
-        return QLatin1String("Android 5.0");
+        return QLatin1String("Android 5.0 (Lollipop)");
     case 22:
-        return QLatin1String("Android 5.1");
+        return QLatin1String("Android 5.1 (Lollipop)");
     case 23:
-        return QLatin1String("Android 6.0");
+        return QLatin1String("Android 6.0 (Marshmallow)");
     case 24:
-        return QLatin1String("Android 7.0");
+        return QLatin1String("Android 7.0 (Nougat)");
     case 25:
-        return QLatin1String("Android 7.1");
+        return QLatin1String("Android 7.1.1 (Nougat)");
     case 26:
-        return QLatin1String("Android 8.0");
+        return QLatin1String("Android 8.0 (Oreo)");
     case 27:
-        return QLatin1String("Android 8.1");
+        return QLatin1String("Android 8.1 (Oreo)");
     case 28:
-        return QLatin1String("Android 9");
+        return QLatin1String("Android 9.0 (Pie)");
     case 29:
-        return QLatin1String("Android 10");
+        return QLatin1String("Android 10.0 (Q)");
     case 30:
-        return QLatin1String("Android 11");
+        return QLatin1String("Android 11.0 (R)");
+    case 31:
+        return QLatin1String("Android 12.0 (S)");
+    case 32:
+        return QLatin1String("Android 12L (API 32)");
+    case 33:
+        return QLatin1String("Android Tiramisu");
     default:
-        return tr("Unknown Android version. API Level: %1").arg(QString::number(x));
+        return tr("Unknown Android version. API Level: %1").arg(x);
     }
 }
 
@@ -556,7 +536,7 @@ static void raiseError(const QString &reason)
     QMessageBox::critical(nullptr, AndroidManager::tr("Error creating Android templates."), reason);
 }
 
-static bool openXmlFile(QDomDocument &doc, const Utils::FilePath &fileName)
+static bool openXmlFile(QDomDocument &doc, const FilePath &fileName)
 {
     QFile f(fileName.toString());
     if (!f.open(QIODevice::ReadOnly))
@@ -569,7 +549,7 @@ static bool openXmlFile(QDomDocument &doc, const Utils::FilePath &fileName)
     return true;
 }
 
-static bool openManifest(ProjectExplorer::Target *target, QDomDocument &doc)
+static bool openManifest(const Target *target, QDomDocument &doc)
 {
     return openXmlFile(doc, AndroidManager::manifestPath(target));
 }
@@ -588,28 +568,29 @@ static int parseMinSdk(const QDomElement &manifestElem)
     return 0;
 }
 
-void AndroidManager::installQASIPackage(ProjectExplorer::Target *target, const QString &packagePath)
+void AndroidManager::installQASIPackage(Target *target, const FilePath &packagePath)
 {
     const QStringList appAbis = AndroidManager::applicationAbis(target);
     if (appAbis.isEmpty())
         return;
-    const int deviceAPILevel = AndroidManager::minimumSDK(target);
-    AndroidDeviceInfo info = AndroidConfigurations::showDeviceDialog(target->project(), deviceAPILevel, appAbis);
+    const IDevice::ConstPtr device = DeviceKitAspect::device(target->kit());
+    AndroidDeviceInfo info = AndroidDevice::androidDeviceInfoFromIDevice(device.data());
     if (!info.isValid()) // aborted
         return;
 
     QString deviceSerialNumber = info.serialNumber;
-    if (info.type == AndroidDeviceInfo::Emulator) {
-        deviceSerialNumber = AndroidAvdManager().startAvd(info.avdname);
+    if (info.type == IDevice::Emulator) {
+        deviceSerialNumber = AndroidAvdManager().startAvd(info.avdName);
         if (deviceSerialNumber.isEmpty())
-            Core::MessageManager::write(tr("Starting Android virtual device failed."));
+            Core::MessageManager::writeDisrupting(tr("Starting Android virtual device failed."));
     }
 
     QStringList arguments = AndroidDeviceInfo::adbSelector(deviceSerialNumber);
-    arguments << "install" << "-r " << packagePath;
+    arguments << "install" << "-r " << packagePath.path();
     QString error;
     if (!runAdbCommandDetached(arguments, &error, true))
-        Core::MessageManager::write(tr("Android package installation failed.\n%1").arg(error));
+        Core::MessageManager::writeDisrupting(
+            tr("Android package installation failed.\n%1").arg(error));
 }
 
 bool AndroidManager::checkKeystorePassword(const QString &keystorePath, const QString &keystorePasswd)
@@ -618,10 +599,11 @@ bool AndroidManager::checkKeystorePassword(const QString &keystorePath, const QS
         return false;
     const CommandLine cmd(AndroidConfigurations::currentConfig().keytoolPath(),
                           {"-list", "-keystore", keystorePath, "--storepass", keystorePasswd});
-    SynchronousProcess proc;
+    QtcProcess proc;
     proc.setTimeoutS(10);
-    SynchronousProcessResponse response = proc.run(cmd);
-    return (response.result == Utils::SynchronousProcessResponse::Finished && response.exitCode == 0);
+    proc.setCommand(cmd);
+    proc.runBlocking(EventLoopMode::On);
+    return proc.result() == ProcessResult::FinishedWithSuccess;
 }
 
 bool AndroidManager::checkCertificatePassword(const QString &keystorePath, const QString &keystorePasswd, const QString &alias, const QString &certificatePasswd)
@@ -634,11 +616,11 @@ bool AndroidManager::checkCertificatePassword(const QString &keystorePath, const
     else
         arguments << certificatePasswd;
 
-    SynchronousProcess proc;
+    QtcProcess proc;
     proc.setTimeoutS(10);
-    SynchronousProcessResponse response
-            = proc.run({AndroidConfigurations::currentConfig().keytoolPath(), arguments});
-    return response.result == SynchronousProcessResponse::Finished && response.exitCode == 0;
+    proc.setCommand({AndroidConfigurations::currentConfig().keytoolPath(), arguments});
+    proc.runBlocking(EventLoopMode::On);
+    return proc.result() == ProcessResult::FinishedWithSuccess;
 }
 
 bool AndroidManager::checkCertificateExists(const QString &keystorePath,
@@ -648,11 +630,11 @@ bool AndroidManager::checkCertificateExists(const QString &keystorePath,
     QStringList arguments = { "-list", "-keystore", keystorePath,
                               "--storepass", keystorePasswd, "-alias", alias };
 
-    SynchronousProcess proc;
+    QtcProcess proc;
     proc.setTimeoutS(10);
-    SynchronousProcessResponse response
-            = proc.run({AndroidConfigurations::currentConfig().keytoolPath(), arguments});
-    return response.result == SynchronousProcessResponse::Finished && response.exitCode == 0;
+    proc.setCommand({AndroidConfigurations::currentConfig().keytoolPath(), arguments});
+    proc.runBlocking(EventLoopMode::On);
+    return proc.result() == ProcessResult::FinishedWithSuccess;
 }
 
 using GradleProperties = QMap<QByteArray, QByteArray>;
@@ -664,7 +646,8 @@ static GradleProperties readGradleProperties(const QString &path)
     if (!file.open(QIODevice::ReadOnly))
         return properties;
 
-    foreach (const QByteArray &line, file.readAll().split('\n')) {
+    const QList<QByteArray> lines = file.readAll().split('\n');
+    for (const QByteArray &line : lines) {
         if (line.trimmed().startsWith('#'))
             continue;
 
@@ -716,10 +699,9 @@ static bool mergeGradleProperties(const QString &path, GradleProperties properti
     return true;
 }
 
-
-bool AndroidManager::updateGradleProperties(ProjectExplorer::Target *target, const QString &buildKey)
+bool AndroidManager::updateGradleProperties(Target *target, const QString &buildKey)
 {
-    QtSupport::BaseQtVersion *version = QtSupport::QtKitAspect::qtVersion(target->kit());
+    QtSupport::QtVersion *version = QtSupport::QtKitAspect::qtVersion(target->kit());
     if (!version)
         return false;
 
@@ -727,8 +709,7 @@ bool AndroidManager::updateGradleProperties(ProjectExplorer::Target *target, con
     if (key.isEmpty()) {
         // FIXME: This case is triggered from AndroidBuildApkWidget::createApplicationGroup
         // and should be avoided.
-        if (RunConfiguration *rc = target->activeRunConfiguration())
-            key = rc->buildKey();
+        key = target->activeBuildKey();
     }
 
     QTC_ASSERT(!key.isEmpty(), return false);
@@ -743,7 +724,7 @@ bool AndroidManager::updateGradleProperties(ProjectExplorer::Target *target, con
     if (!packageSourceDir.exists())
         return false;
 
-    const FilePath wrapperProps = packageSourceDir.pathAppended("gradle/wrapper/gradle-wrapper.properties");
+    const FilePath wrapperProps = packageSourceDir / "gradle/wrapper/gradle-wrapper.properties";
     if (wrapperProps.exists()) {
         GradleProperties wrapperProperties = readGradleProperties(wrapperProps.toString());
         QString distributionUrl = QString::fromLocal8Bit(wrapperProperties["distributionUrl"]);
@@ -756,7 +737,7 @@ bool AndroidManager::updateGradleProperties(ProjectExplorer::Target *target, con
 
     GradleProperties localProperties;
     localProperties["sdk.dir"] = AndroidConfigurations::currentConfig().sdkLocation().toString().toLocal8Bit();
-    const FilePath localPropertiesFile = packageSourceDir.pathAppended("local.properties");
+    const FilePath localPropertiesFile = packageSourceDir / "local.properties";
     if (!mergeGradleProperties(localPropertiesFile.toString(), localProperties))
         return false;
 
@@ -775,27 +756,13 @@ bool AndroidManager::updateGradleProperties(ProjectExplorer::Target *target, con
     return mergeGradleProperties(gradlePropertiesPath, gradleProperties);
 }
 
-int AndroidManager::findApiLevel(const Utils::FilePath &platformPath)
-{
-    int apiLevel = -1;
-    const Utils::FilePath propertiesPath = platformPath.pathAppended("/source.properties");
-    if (propertiesPath.exists()) {
-        QSettings sdkProperties(propertiesPath.toString(), QSettings::IniFormat);
-        bool validInt = false;
-        apiLevel = sdkProperties.value(ApiLevelKey).toInt(&validInt);
-        if (!validInt)
-            apiLevel = -1;
-    }
-    return apiLevel;
-}
-
 QProcess *AndroidManager::runAdbCommandDetached(const QStringList &args, QString *err,
                                                 bool deleteOnFinish)
 {
     std::unique_ptr<QProcess> p(new QProcess);
-    const QString adb = AndroidConfigurations::currentConfig().adbToolPath().toString();
+    const FilePath adb = AndroidConfigurations::currentConfig().adbToolPath();
     qCDebug(androidManagerLog) << "Running command (async):" << CommandLine(adb, args).toUserOutput();
-    p->start(adb, args);
+    p->start(adb.toString(), args);
     if (p->waitForStarted(500) && p->state() == QProcess::Running) {
         if (deleteOnFinish) {
             connect(p.get(), QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
@@ -817,18 +784,20 @@ SdkToolResult AndroidManager::runCommand(const CommandLine &command,
                                          const QByteArray &writeData, int timeoutS)
 {
     Android::SdkToolResult cmdResult;
-    Utils::SynchronousProcess cmdProc;
+    QtcProcess cmdProc;
     cmdProc.setTimeoutS(timeoutS);
+    cmdProc.setWriteData(writeData);
     qCDebug(androidManagerLog) << "Running command (sync):" << command.toUserOutput();
-    SynchronousProcessResponse response = cmdProc.run(command, writeData);
-    cmdResult.m_stdOut = response.stdOut().trimmed();
-    cmdResult.m_stdErr = response.stdErr().trimmed();
-    cmdResult.m_success = response.result == Utils::SynchronousProcessResponse::Finished;
-    qCDebug(androidManagerLog) << "Running command (sync) finshed:" << command.toUserOutput()
+    cmdProc.setCommand(command);
+    cmdProc.runBlocking(EventLoopMode::On);
+    cmdResult.m_stdOut = cmdProc.cleanedStdOut().trimmed();
+    cmdResult.m_stdErr = cmdProc.cleanedStdErr().trimmed();
+    cmdResult.m_success = cmdProc.result() == ProcessResult::FinishedWithSuccess;
+    qCDebug(androidManagerLog) << "Command finshed (sync):" << command.toUserOutput()
                                << "Success:" << cmdResult.m_success
-                               << "Output:" << response.allRawOutput();
+                               << "Output:" << cmdProc.allRawOutput();
     if (!cmdResult.success())
-        cmdResult.m_exitMessage = response.exitMessage(command.executable().toString(), timeoutS);
+        cmdResult.m_exitMessage = cmdProc.exitMessage();
     return cmdResult;
 }
 
@@ -838,10 +807,4 @@ SdkToolResult AndroidManager::runAdbCommand(const QStringList &args,
     return runCommand({AndroidConfigurations::currentConfig().adbToolPath(), args},
                       writeData, timeoutS);
 }
-
-SdkToolResult AndroidManager::runAaptCommand(const QStringList &args, int timeoutS)
-{
-    return runCommand({AndroidConfigurations::currentConfig().aaptToolPath(), args}, {},
-                      timeoutS);
-}
 } // namespace Android
