@@ -40,7 +40,6 @@
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/idocument.h>
-#include <cppeditor/cppeditorconstants.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectnodes.h>
 #include <projectexplorer/projecttree.h>
@@ -50,28 +49,26 @@
 
 #include <QAction>
 #include <QMenu>
+#include <QVersionNumber>
 
 using namespace TextEditor;
 
 namespace Beautifier {
 namespace Internal {
-namespace Uncrustify {
 
 Uncrustify::Uncrustify()
 {
-    Core::ActionContainer *menu = Core::ActionManager::createMenu(Constants::Uncrustify::MENU_ID);
+    Core::ActionContainer *menu = Core::ActionManager::createMenu("Uncrustify.Menu");
     menu->menu()->setTitle(tr("&Uncrustify"));
 
     m_formatFile = new QAction(BeautifierPlugin::msgFormatCurrentFile(), this);
     Core::Command *cmd
-            = Core::ActionManager::registerAction(m_formatFile,
-                                                  Constants::Uncrustify::ACTION_FORMATFILE);
+            = Core::ActionManager::registerAction(m_formatFile, "Uncrustify.FormatFile");
     menu->addAction(cmd);
     connect(m_formatFile, &QAction::triggered, this, &Uncrustify::formatFile);
 
     m_formatRange = new QAction(BeautifierPlugin::msgFormatSelectedText(), this);
-    cmd = Core::ActionManager::registerAction(m_formatRange,
-                                              Constants::Uncrustify::ACTION_FORMATSELECTED);
+    cmd = Core::ActionManager::registerAction(m_formatRange, "Uncrustify.FormatSelectedText");
     menu->addAction(cmd);
     connect(m_formatRange, &QAction::triggered, this, &Uncrustify::formatSelectedText);
 
@@ -83,7 +80,7 @@ Uncrustify::Uncrustify()
 
 QString Uncrustify::id() const
 {
-    return QLatin1String(Constants::Uncrustify::DISPLAY_NAME);
+    return QLatin1String(Constants::UNCRUSTIFY_DISPLAY_NAME);
 }
 
 void Uncrustify::updateActions(Core::IEditor *editor)
@@ -98,7 +95,7 @@ void Uncrustify::formatFile()
     const QString cfgFileName = configurationFile();
     if (cfgFileName.isEmpty()) {
         BeautifierPlugin::showError(BeautifierPlugin::msgCannotGetConfigurationFile(
-                                        tr(Constants::Uncrustify::DISPLAY_NAME)));
+                                        tr(Constants::UNCRUSTIFY_DISPLAY_NAME)));
     } else {
         formatCurrentFile(command(cfgFileName));
     }
@@ -109,7 +106,7 @@ void Uncrustify::formatSelectedText()
     const QString cfgFileName = configurationFile();
     if (cfgFileName.isEmpty()) {
         BeautifierPlugin::showError(BeautifierPlugin::msgCannotGetConfigurationFile(
-                                        tr(Constants::Uncrustify::DISPLAY_NAME)));
+                                        tr(Constants::UNCRUSTIFY_DISPLAY_NAME)));
         return;
     }
 
@@ -184,7 +181,7 @@ Command Uncrustify::command(const QString &cfgFile, bool fragment) const
     Command command;
     command.setExecutable(m_settings.command().toString());
     command.setProcessing(Command::PipeProcessing);
-    if (m_settings.version() >= 62) {
+    if (m_settings.version() >= QVersionNumber(0, 62)) {
         command.addOption("--assume");
         command.addOption("%file");
     } else {
@@ -200,6 +197,5 @@ Command Uncrustify::command(const QString &cfgFile, bool fragment) const
     return command;
 }
 
-} // namespace Uncrustify
 } // namespace Internal
 } // namespace Beautifier
