@@ -60,6 +60,12 @@
 #include <QWheelEvent>
 
 using namespace Core;
+using namespace Utils;
+
+namespace BinEditor {
+namespace Internal {
+
+const QChar MidpointChar(u'\u00B7');
 
 static QByteArray calculateHexPattern(const QByteArray &pattern)
 {
@@ -78,9 +84,6 @@ static QByteArray calculateHexPattern(const QByteArray &pattern)
     return result;
 }
 
-namespace BinEditor {
-namespace Internal {
-
 class BinEditorWidgetPrivate : public EditorService
 {
 public:
@@ -400,24 +403,25 @@ bool BinEditorWidget::isReadOnly() const
     return m_readOnly;
 }
 
-bool BinEditorWidget::save(QString *errorString, const QString &oldFileName, const QString &newFileName)
+bool BinEditorWidget::save(QString *errorString, const FilePath &oldFilePath, const FilePath &newFilePath)
 {
-    if (oldFileName != newFileName) {
-        QString tmpName;
+    if (oldFilePath != newFilePath) {
+        FilePath tmpName;
         {
-            QTemporaryFile tmp(newFileName + QLatin1String("_XXXXXX.new"));
+            QTemporaryFile tmp(newFilePath.toString() + QLatin1String("_XXXXXX.new"));
             if (!tmp.open())
                 return false;
-            tmpName = tmp.fileName();
+            tmpName = FilePath::fromString(tmp.fileName());
         }
-        if (!QFile::copy(oldFileName, tmpName))
+        if (!oldFilePath.copyFile(tmpName))
             return false;
-        if (QFile::exists(newFileName) && !QFile::remove(newFileName))
+        if (newFilePath.exists() && !newFilePath.removeFile())
             return false;
-        if (!QFile::rename(tmpName, newFileName))
+        if (!tmpName.renameFile(newFilePath))
             return false;
     }
-    Utils::FileSaver saver(newFileName, QIODevice::ReadWrite); // QtBug: WriteOnly truncates.
+
+    FileSaver saver(newFilePath, QIODevice::ReadWrite); // QtBug: WriteOnly truncates.
     if (!saver.hasError()) {
         QFile *output = saver.file();
         const qint64 size = output->size();
@@ -567,7 +571,7 @@ Utils::optional<qint64> BinEditorWidget::posAt(const QPoint &pos, bool includeEm
                 break;
             QChar qc(QLatin1Char(dataAt(dataPos)));
             if (!qc.isPrint())
-                qc = 0xB7;
+                qc = MidpointChar;
             x -= fontMetrics().horizontalAdvance(qc);
             if (x <= 0)
                 break;
@@ -863,7 +867,7 @@ void BinEditorWidget::paintEvent(QPaintEvent *e)
                     break;
                 QChar qc(QLatin1Char(dataAt(pos, isOld)));
                 if (qc.unicode() >= 127 || !qc.isPrint())
-                    qc = 0xB7;
+                    qc = MidpointChar;
                 printable += qc;
             }
         } else {
@@ -901,7 +905,7 @@ void BinEditorWidget::paintEvent(QPaintEvent *e)
                 int item_x = -xoffset +  m_margin + c * m_columnWidth + m_labelWidth;
 
                 QColor color;
-                foreach (const Markup &m, m_markup) {
+                for (const Markup &m : qAsConst(m_markup)) {
                     if (m.covers(lineAddress + c)) {
                         color = m.color;
                         break;
@@ -1232,7 +1236,7 @@ QString BinEditorWidget::toolTip(const QHelpEvent *helpEvent) const
     str << "<html><head/><body><p align=\"center\"><b>"
         << tr("Memory at 0x%1").arg(address, 0, 16) << "</b></p>";
 
-    foreach (const Markup &m, m_markup) {
+    for (const Markup &m : qAsConst(m_markup)) {
         if (m.covers(address) && !m.toolTip.isEmpty()) {
             str << "<p>" <<  m.toolTip << "</p><br>";
             break;
@@ -1467,7 +1471,7 @@ void BinEditorWidget::zoomF(float delta)
     else if (step < 0 && step > -1)
         step = -1;
 
-    const int newZoom = TextEditor::TextEditorSettings::instance()->increaseFontZoom(int(step));
+    const int newZoom = TextEditor::TextEditorSettings::increaseFontZoom(int(step));
     showZoomIndicator(this, newZoom);
 }
 
