@@ -25,12 +25,9 @@
 
 #include "clangtoolrunner.h"
 
-#include "clangtoolsconstants.h"
-
 #include <utils/environment.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
-#include <utils/synchronousprocess.h>
 #include <utils/temporaryfile.h>
 
 #include <QDebug>
@@ -40,6 +37,8 @@
 
 static Q_LOGGING_CATEGORY(LOG, "qtc.clangtools.runner", QtWarningMsg)
 
+using namespace Utils;
+
 namespace ClangTools {
 namespace Internal {
 
@@ -58,35 +57,52 @@ static QString finishedWithBadExitCode(const QString &name, int exitCode)
     return ClangToolRunner::tr("%1 finished with exit code: %2.").arg(name).arg(exitCode);
 }
 
-void ClangToolRunner::init(const QString &outputDirPath,
-                           const Utils::Environment &environment)
+ClangToolRunner::ClangToolRunner(QObject *parent)
+    : QObject(parent)
+{}
+
+void ClangToolRunner::init(const FilePath &outputDirPath, const Environment &environment)
 {
     m_outputDirPath = outputDirPath;
     QTC_CHECK(!m_outputDirPath.isEmpty());
 
-    m_process.setProcessChannelMode(QProcess::MergedChannels);
-    m_process.setProcessEnvironment(environment.toProcessEnvironment());
+    m_process.setEnvironment(environment);
+    m_process.setUseCtrlCStub(true);
     m_process.setWorkingDirectory(m_outputDirPath); // Current clang-cl puts log file into working dir.
-    connect(&m_process, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
-            this, &ClangToolRunner::onProcessFinished);
-    connect(&m_process, &QProcess::errorOccurred, this, &ClangToolRunner::onProcessError);
-    connect(&m_process, &QProcess::readyRead, this, &ClangToolRunner::onProcessOutput);
+    connect(&m_process, &QtcProcess::done, this, &ClangToolRunner::onProcessDone);
 }
 
-ClangToolRunner::~ClangToolRunner()
+QStringList ClangToolRunner::mainToolArguments() const
 {
-    Utils::SynchronousProcess::stopProcess(m_process);
+    QStringList result;
+    result << "-export-fixes=" + m_outputFilePath;
+    if (!m_overlayFilePath.isEmpty() && supportsVFSOverlay())
+        result << "--vfsoverlay=" + m_overlayFilePath;
+    result << QDir::toNativeSeparators(m_fileToAnalyze);
+    return result;
 }
 
-static QString createOutputFilePath(const QString &dirPath, const QString &fileToAnalyze)
+bool ClangToolRunner::supportsVFSOverlay() const
+{
+    static QMap<FilePath, bool> vfsCapabilities;
+    auto it = vfsCapabilities.find(m_executable);
+    if (it == vfsCapabilities.end()) {
+        QtcProcess p;
+        p.setCommand({m_executable, {"--help"}});
+        p.runBlocking();
+        it = vfsCapabilities.insert(m_executable, p.allOutput().contains("vfsoverlay"));
+    }
+    return it.value();
+}
+
+static QString createOutputFilePath(const FilePath &dirPath, const QString &fileToAnalyze)
 {
     const QString fileName = QFileInfo(fileToAnalyze).fileName();
-    const QString fileTemplate = dirPath
-            + QLatin1String("/report-") + fileName + QLatin1String("-XXXXXX");
+    const FilePath fileTemplate = dirPath.pathAppended("report-" + fileName + "-XXXXXX");
 
-    Utils::TemporaryFile temporaryFile("clangtools");
+    TemporaryFile temporaryFile("clangtools");
     temporaryFile.setAutoRemove(false);
-    temporaryFile.setFileTemplate(fileTemplate);
+    temporaryFile.setFileTemplate(fileTemplate.path());
     if (temporaryFile.open()) {
         temporaryFile.close();
         return temporaryFile.fileName();
@@ -101,55 +117,40 @@ bool ClangToolRunner::run(const QString &fileToAnalyze, const QStringList &compi
     QTC_CHECK(!compilerOptions.contains(fileToAnalyze));
 
     m_fileToAnalyze = fileToAnalyze;
-    m_processOutput.clear();
 
     m_outputFilePath = createOutputFilePath(m_outputDirPath, fileToAnalyze);
     QTC_ASSERT(!m_outputFilePath.isEmpty(), return false);
-    const QStringList arguments = m_argsCreator(compilerOptions);
-    m_commandLine = Utils::QtcProcess::joinArgs(QStringList(m_executable) + arguments);
+    m_commandLine = {m_executable, m_argsCreator(compilerOptions)};
 
-    qCDebug(LOG).noquote() << "Starting" << m_commandLine;
-    m_process.start(m_executable, arguments);
+    qCDebug(LOG).noquote() << "Starting" << m_commandLine.toUserOutput();
+    m_process.setCommand(m_commandLine);
+    m_process.start();
     return true;
 }
 
-void ClangToolRunner::onProcessFinished(int exitCode, QProcess::ExitStatus exitStatus)
+void ClangToolRunner::onProcessDone()
 {
-    if (exitStatus == QProcess::NormalExit) {
-        if (exitCode == 0) {
-            qCDebug(LOG).noquote() << "Output:\n" << Utils::SynchronousProcess::normalizeNewlines(
-                                                        QString::fromLocal8Bit(m_processOutput));
-            emit finishedWithSuccess(m_fileToAnalyze);
-        } else {
-            emit finishedWithFailure(finishedWithBadExitCode(m_name, exitCode),
-                                     commandlineAndOutput());
-        }
+    if (m_process.result() == ProcessResult::StartFailed) {
+        emit finishedWithFailure(generalProcessError(m_name), commandlineAndOutput());
+    } else if (m_process.result() == ProcessResult::FinishedWithSuccess) {
+        qCDebug(LOG).noquote() << "Output:\n" << m_process.cleanedStdOut();
+        emit finishedWithSuccess(m_fileToAnalyze);
+    } else if (m_process.result() == ProcessResult::FinishedWithError) {
+        emit finishedWithFailure(finishedWithBadExitCode(m_name, m_process.exitCode()),
+                                 commandlineAndOutput());
     } else { // == QProcess::CrashExit
         emit finishedWithFailure(finishedDueToCrash(m_name), commandlineAndOutput());
     }
 }
 
-void ClangToolRunner::onProcessError(QProcess::ProcessError error)
-{
-    if (error == QProcess::Crashed)
-        return; // handled by slot of finished()
-
-    emit finishedWithFailure(generalProcessError(m_name), commandlineAndOutput());
-}
-
-void ClangToolRunner::onProcessOutput()
-{
-    m_processOutput.append(m_process.readAll());
-}
-
 QString ClangToolRunner::commandlineAndOutput() const
 {
     return tr("Command line: %1\n"
               "Process Error: %2\n"
               "Output:\n%3")
-        .arg(m_commandLine,
-             QString::number(m_process.error()),
-             Utils::SynchronousProcess::normalizeNewlines(QString::fromLocal8Bit(m_processOutput)));
+        .arg(m_commandLine.toUserOutput())
+        .arg(m_process.error())
+        .arg(m_process.cleanedStdOut());
 }
 
 } // namespace Internal
