@@ -28,16 +28,18 @@
 #include "clangtool.h"
 #include "clangtoolsdiagnostic.h"
 #include "clangtoolssettings.h"
-#include "clangtoolsutils.h"
 
 #include <coreplugin/icore.h>
-#include <cpptools/clangdiagnosticconfig.h>
-#include <cpptools/cppmodelmanager.h>
-#include <cpptools/cpptoolsreuse.h>
-#include <cpptools/cpptoolstestcase.h>
+
+#include <cppeditor/clangdiagnosticconfig.h>
+#include <cppeditor/cppmodelmanager.h>
+#include <cppeditor/cpptoolsreuse.h>
+#include <cppeditor/cpptoolstestcase.h>
+
 #include <projectexplorer/kitinformation.h>
 #include <projectexplorer/kitmanager.h>
 #include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/toolchain.h>
 
 #include <qtsupport/qtkitinformation.h>
@@ -50,11 +52,11 @@
 #include <QTimer>
 #include <QtTest>
 
-using namespace CppTools;
+using namespace CppEditor;
 using namespace ProjectExplorer;
 using namespace Utils;
 
-Q_DECLARE_METATYPE(CppTools::ClangDiagnosticConfig)
+Q_DECLARE_METATYPE(CppEditor::ClangDiagnosticConfig)
 
 namespace ClangTools {
 namespace Internal {
@@ -115,9 +117,7 @@ void ClangToolsUnitTests::testProject()
 
     // Open project
     Tests::ProjectOpenerAndCloser projectManager;
-    const ProjectInfo projectInfo = projectManager.open(projectFilePath, true, m_kit);
-    const bool isProjectOpen = projectInfo.isValid();
-    QVERIFY(isProjectOpen);
+    QVERIFY(projectManager.open(projectFilePath, true, m_kit));
 
     // Run tool
     ClangTool *tool = ClangTool::instance();
@@ -125,10 +125,10 @@ void ClangToolsUnitTests::testProject()
                     ClangToolsSettings::instance()->runSettings(),
                     diagnosticConfig);
     QSignalSpy waitForFinishedTool(tool, &ClangTool::finished);
-    QVERIFY(waitForFinishedTool.wait(30000));
+    QVERIFY(waitForFinishedTool.wait(m_timeout));
 
     // Check for errors
-    const QString errorText = waitForFinishedTool.takeFirst().first().toString();
+    const QString errorText = waitForFinishedTool.takeFirst().constFirst().toString();
     const bool finishedSuccessfully = errorText.isEmpty();
     if (!finishedSuccessfully)
         qWarning("Error: %s", qPrintable(errorText));
@@ -186,5 +186,11 @@ void ClangToolsUnitTests::addTestRow(const QByteArray &relativeFilePath,
         << absoluteFilePath << expectedDiagCount << diagnosticConfig;
 }
 
+int ClangToolsUnitTests::getTimeout()
+{
+    const int t = qEnvironmentVariableIntValue("QTC_CLANGTOOLS_TEST_TIMEOUT");
+    return t > 0 ? t : 480000;
+}
+
 } // namespace Internal
 } // namespace ClangTools
