@@ -26,6 +26,7 @@
 #include "builddirparameters.h"
 
 #include "cmakebuildconfiguration.h"
+#include "cmakebuildsystem.h"
 #include "cmakekitinformation.h"
 #include "cmakeprojectplugin.h"
 #include "cmakespecificsettings.h"
@@ -36,6 +37,7 @@
 #include <projectexplorer/target.h>
 #include <projectexplorer/toolchain.h>
 
+#include <utils/algorithm.h>
 #include <utils/qtcassert.h>
 
 using namespace ProjectExplorer;
@@ -45,30 +47,41 @@ namespace Internal {
 
 BuildDirParameters::BuildDirParameters() = default;
 
-BuildDirParameters::BuildDirParameters(CMakeBuildConfiguration *bc)
+BuildDirParameters::BuildDirParameters(CMakeBuildSystem *buildSystem)
 {
-    QTC_ASSERT(bc, return );
+    QTC_ASSERT(buildSystem, return);
+    auto bc = buildSystem->cmakeBuildConfiguration();
+    QTC_ASSERT(bc, return);
 
     const Utils::MacroExpander *expander = bc->macroExpander();
 
-    initialCMakeArguments = Utils::transform(bc->initialCMakeArguments(),
-                                             [expander](const QString &s) {
-                                                 return expander->expand(s);
-                                             });
-    extraCMakeArguments = Utils::transform(bc->extraCMakeArguments(),
-                                             [expander](const QString &s) {
-                                                 return expander->expand(s);
-                                             });
-
+    const QStringList expandedArguments = Utils::transform(buildSystem->initialCMakeArguments(),
+                                                           [expander](const QString &s) {
+                                                               return expander->expand(s);
+                                                           });
+    initialCMakeArguments = Utils::filtered(expandedArguments,
+                                            [](const QString &s) { return !s.isEmpty(); });
+    configurationChangesArguments = Utils::transform(buildSystem->configurationChangesArguments(),
+                                                     [expander](const QString &s) {
+                                                         return expander->expand(s);
+                                                     });
+    additionalCMakeArguments = Utils::transform(buildSystem->additionalCMakeArguments(),
+                                                [expander](const QString &s) {
+                                                    return expander->expand(s);
+                                                });
     const Target *t = bc->target();
     const Kit *k = t->kit();
     const Project *p = t->project();
 
     projectName = p->displayName();
 
-    sourceDirectory = p->projectDirectory();
+    sourceDirectory = bc->sourceDirectory();
+    if (sourceDirectory.isEmpty())
+        sourceDirectory = p->projectDirectory();
     buildDirectory = bc->buildDirectory();
 
+    cmakeBuildType = buildSystem->cmakeBuildType();
+
     environment = bc->environment();
     // Disable distributed building for configuration runs. CMake does not do those in parallel,
     // so there is no win in sending data over the network.
@@ -76,13 +89,6 @@ BuildDirParameters::BuildDirParameters(CMakeBuildConfiguration *bc)
     if (Utils::HostOsInfo::isAnyUnixHost())
         environment.set("ICECC", "no");
 
-    CMakeSpecificSettings *settings = CMakeProjectPlugin::projectTypeSpecificSettings();
-    if (!settings->ninjaPath().isEmpty()) {
-        const Utils::FilePath setting = settings->ninjaPath();
-        const Utils::FilePath path = setting.toFileInfo().isFile() ? setting.parentDir() : setting;
-        environment.appendOrSetPath(path.toString());
-    }
-
     cmakeToolId = CMakeKitAspect::cmakeToolId(k);
 }
 
@@ -96,8 +102,5 @@ CMakeTool *BuildDirParameters::cmakeTool() const
     return CMakeToolManager::findById(cmakeToolId);
 }
 
-BuildDirParameters::BuildDirParameters(const BuildDirParameters &) = default;
-BuildDirParameters &BuildDirParameters::operator=(const BuildDirParameters &) = default;
-
 } // namespace Internal
 } // namespace CMakeProjectManager
