@@ -26,13 +26,13 @@
 #include "coreplugin.h"
 #include "designmode.h"
 #include "editmode.h"
+#include "foldernavigationwidget.h"
 #include "helpmanager.h"
 #include "icore.h"
 #include "idocument.h"
 #include "iwizardfactory.h"
 #include "mainwindow.h"
 #include "modemanager.h"
-#include "reaper_p.h"
 #include "themechooser.h"
 
 #include <coreplugin/actionmanager/actionmanager.h>
@@ -49,9 +49,11 @@
 #include <extensionsystem/pluginmanager.h>
 #include <extensionsystem/pluginspec.h>
 #include <utils/algorithm.h>
+#include <utils/checkablemessagebox.h>
+#include <utils/commandline.h>
 #include <utils/infobar.h>
 #include <utils/macroexpander.h>
-#include <utils/mimetypes/mimedatabase.h>
+#include <utils/mimeutils.h>
 #include <utils/pathchooser.h>
 #include <utils/savefile.h>
 #include <utils/stringutils.h>
@@ -62,6 +64,7 @@
 #include <QDebug>
 #include <QDir>
 #include <QJsonObject>
+#include <QLabel>
 #include <QMenu>
 #include <QMessageBox>
 #include <QSettings>
@@ -75,13 +78,26 @@ using namespace Utils;
 
 static CorePlugin *m_instance = nullptr;
 
+const char kWarnCrashReportingSetting[] = "WarnCrashReporting";
+const char kEnvironmentChanges[] = "Core/EnvironmentChanges";
+
+void CorePlugin::setupSystemEnvironment()
+{
+    m_instance->m_startupSystemEnvironment = Environment::systemEnvironment();
+    const EnvironmentItems changes = EnvironmentItem::fromStringList(
+        ICore::settings()->value(kEnvironmentChanges).toStringList());
+    setEnvironmentChanges(changes);
+}
+
 CorePlugin::CorePlugin()
 {
     qRegisterMetaType<Id>();
     qRegisterMetaType<Core::Search::TextPosition>();
     qRegisterMetaType<Utils::CommandLine>();
     qRegisterMetaType<Utils::FilePath>();
+    qRegisterMetaType<Utils::Environment>();
     m_instance = this;
+    setupSystemEnvironment();
 }
 
 CorePlugin::~CorePlugin()
@@ -90,6 +106,7 @@ CorePlugin::~CorePlugin()
     Find::destroy();
 
     delete m_locator;
+    delete m_folderNavigationWidgetFactory;
     delete m_editMode;
 
     DesignMode::destroyModeIfRequired();
@@ -146,11 +163,12 @@ bool CorePlugin::initialize(const QStringList &arguments, QString *errorMessage)
         return false;
     }
     const CoreArguments args = parseArguments(arguments);
-    Theme::initialPalette(); // Initialize palette before setting it
     Theme *themeFromArg = ThemeEntry::createTheme(args.themeId);
-    setCreatorTheme(themeFromArg ? themeFromArg
-                                 : ThemeEntry::createTheme(ThemeEntry::themeSetting()));
-    InfoBar::initialize(ICore::settings(), creatorTheme());
+    Theme *theme = themeFromArg ? themeFromArg
+                                : ThemeEntry::createTheme(ThemeEntry::themeSetting());
+    Theme::setInitialPalette(theme); // Initialize palette before setting it
+    setCreatorTheme(theme);
+    InfoBar::initialize(ICore::settings());
     new ActionManager(this);
     ActionManager::setPresentationModeEnabled(args.presentationMode);
     m_mainWindow = new MainWindow;
@@ -162,6 +180,8 @@ bool CorePlugin::initialize(const QStringList &arguments, QString *errorMessage)
     m_editMode = new EditMode;
     ModeManager::activateMode(m_editMode->id());
 
+    m_folderNavigationWidgetFactory = new FolderNavigationWidgetFactory;
+
     IWizardFactory::initialize();
 
     // Make sure we respect the process's umask when creating new files
@@ -188,7 +208,7 @@ bool CorePlugin::initialize(const QStringList &arguments, QString *errorMessage)
     expander->registerVariable("Config:DefaultProjectDirectory", tr("The configured default directory for projects."),
                                []() { return DocumentManager::projectsDirectory().toString(); });
     expander->registerVariable("Config:LastFileDialogDirectory", tr("The directory last visited in a file dialog."),
-                               []() { return DocumentManager::fileDialogLastVisitedDirectory(); });
+                               []() { return DocumentManager::fileDialogLastVisitedDirectory().toString(); });
     expander->registerVariable("HostOs:isWindows",
                                tr("Is %1 running on Windows?").arg(Constants::IDE_DISPLAY_NAME),
                                []() { return QVariant(Utils::HostOsInfo::isWindowsHost()).toString(); });
@@ -211,7 +231,7 @@ bool CorePlugin::initialize(const QStringList &arguments, QString *errorMessage)
     expander->registerVariable("IDE:ResourcePath",
                                tr("The directory where %1 finds its pre-installed resources.")
                                    .arg(Constants::IDE_DISPLAY_NAME),
-                               []() { return ICore::resourcePath(); });
+                               []() { return ICore::resourcePath().toString(); });
     expander->registerPrefix("CurrentDate:", tr("The current date (QDate formatstring)."),
                              [](const QString &fmt) { return QDate::currentDate().toString(fmt); });
     expander->registerPrefix("CurrentTime:", tr("The current time (QTime formatstring)."),
@@ -223,6 +243,11 @@ bool CorePlugin::initialize(const QStringList &arguments, QString *errorMessage)
 
     Utils::PathChooser::setAboutToShowContextMenuHandler(&CorePlugin::addToPathChooserContextMenu);
 
+#ifdef ENABLE_CRASHPAD
+    connect(ICore::instance(), &ICore::coreOpened, this, &CorePlugin::warnAboutCrashReporing,
+            Qt::QueuedConnection);
+#endif
+
     return true;
 }
 
@@ -259,13 +284,39 @@ QObject *CorePlugin::remoteCommand(const QStringList & /* options */,
         });
         return nullptr;
     }
-    IDocument *res = m_mainWindow->openFiles(
-                args, ICore::OpenFilesFlags(ICore::SwitchMode | ICore::CanContainLineAndColumnNumbers | ICore::SwitchSplitIfAlreadyVisible),
+    const FilePaths filePaths = Utils::transform(args, FilePath::fromUserInput);
+    IDocument *res = MainWindow::openFiles(
+                filePaths,
+                ICore::OpenFilesFlags(ICore::SwitchMode | ICore::CanContainLineAndColumnNumbers | ICore::SwitchSplitIfAlreadyVisible),
                 workingDirectory);
     m_mainWindow->raiseWindow();
     return res;
 }
 
+Environment CorePlugin::startupSystemEnvironment()
+{
+    return m_instance->m_startupSystemEnvironment;
+}
+
+EnvironmentItems CorePlugin::environmentChanges()
+{
+    return m_instance->m_environmentChanges;
+}
+
+void CorePlugin::setEnvironmentChanges(const EnvironmentItems &changes)
+{
+    if (m_instance->m_environmentChanges == changes)
+        return;
+    m_instance->m_environmentChanges = changes;
+    Environment systemEnv = m_instance->m_startupSystemEnvironment;
+    systemEnv.modify(changes);
+    Environment::setSystemEnvironment(systemEnv);
+    ICore::settings()->setValueWithDefault(kEnvironmentChanges,
+                                           EnvironmentItem::toStringList(changes));
+    if (ICore::instance())
+        emit ICore::instance()->systemEnvironmentChanged();
+}
+
 void CorePlugin::fileOpenRequest(const QString &f)
 {
     remoteCommand(QStringList(), QString(), QStringList(f));
@@ -279,13 +330,16 @@ void CorePlugin::addToPathChooserContextMenu(Utils::PathChooser *pathChooser, QM
     if (QDir().exists(pathChooser->filePath().toString())) {
         auto *showInGraphicalShell = new QAction(Core::FileUtils::msgGraphicalShellAction(), menu);
         connect(showInGraphicalShell, &QAction::triggered, pathChooser, [pathChooser]() {
-            Core::FileUtils::showInGraphicalShell(pathChooser, pathChooser->filePath().toString());
+            Core::FileUtils::showInGraphicalShell(pathChooser, pathChooser->filePath());
         });
         menu->insertAction(firstAction, showInGraphicalShell);
 
         auto *showInTerminal = new QAction(Core::FileUtils::msgTerminalHereAction(), menu);
         connect(showInTerminal, &QAction::triggered, pathChooser, [pathChooser]() {
-            Core::FileUtils::openTerminal(pathChooser->filePath().toString());
+            if (pathChooser->openTerminalHandler())
+                pathChooser->openTerminalHandler()();
+            else
+                FileUtils::openTerminal(pathChooser->filePath());
         });
         menu->insertAction(firstAction, showInTerminal);
 
@@ -341,9 +395,64 @@ void CorePlugin::checkSettings()
     showMsgBox(errorMsg, QMessageBox::Critical);
 }
 
+void CorePlugin::warnAboutCrashReporing()
+{
+    if (!ICore::infoBar()->canInfoBeAdded(kWarnCrashReportingSetting))
+        return;
+
+    QString warnStr = ICore::settings()->value("CrashReportingEnabled", false).toBool()
+            ? tr("%1 collects crash reports for the sole purpose of fixing bugs. "
+                 "To disable this feature go to %2.")
+            : tr("%1 can collect crash reports for the sole purpose of fixing bugs. "
+                 "To enable this feature go to %2.");
+
+    if (Utils::HostOsInfo::isMacHost()) {
+        warnStr = warnStr.arg(QLatin1String(Core::Constants::IDE_DISPLAY_NAME),
+                              Core::Constants::IDE_DISPLAY_NAME + tr(" > Preferences > Environment > System"));
+    } else {
+        warnStr = warnStr.arg(QLatin1String(Core::Constants::IDE_DISPLAY_NAME),
+                              tr("Edit > Preferences > Environment > System"));
+    }
+
+    Utils::InfoBarEntry info(kWarnCrashReportingSetting, warnStr,
+                             Utils::InfoBarEntry::GlobalSuppression::Enabled);
+    info.addCustomButton(tr("Configure..."), [] {
+        ICore::infoBar()->removeInfo(kWarnCrashReportingSetting);
+        ICore::infoBar()->globallySuppressInfo(kWarnCrashReportingSetting);
+        ICore::showOptionsDialog(Core::Constants::SETTINGS_ID_SYSTEM);
+    });
+
+    info.setDetailsWidgetCreator([]() -> QWidget * {
+        auto label = new QLabel;
+        label->setWordWrap(true);
+        label->setOpenExternalLinks(true);
+        label->setText(msgCrashpadInformation());
+        label->setContentsMargins(0, 0, 0, 8);
+        return label;
+    });
+    ICore::infoBar()->addInfo(info);
+}
+
+// static
+QString CorePlugin::msgCrashpadInformation()
+{
+    return tr("%1 uses Google Crashpad for collecting crashes and sending them to our backend "
+              "for processing. Crashpad may capture arbitrary contents from crashed process’ "
+              "memory, including user sensitive information, URLs, and whatever other content "
+              "users have trusted %1 with. The collected crash reports are however only used "
+              "for the sole purpose of fixing bugs.").arg(Core::Constants::IDE_DISPLAY_NAME)
+            + "<br><br>" + tr("More information:")
+            + "<br><a href='https://chromium.googlesource.com/crashpad/crashpad/+/master/doc/"
+                           "overview_design.md'>" + tr("Crashpad Overview") + "</a>"
+              "<br><a href='https://sentry.io/security/'>" + tr("%1 security policy").arg("Sentry.io")
+            + "</a>";
+}
+
 ExtensionSystem::IPlugin::ShutdownFlag CorePlugin::aboutToShutdown()
 {
     Find::aboutToShutdown();
+    ExtensionSystem::IPlugin::ShutdownFlag shutdownFlag = m_locator->aboutToShutdown(
+        [this] { emit asynchronousShutdownFinished(); });
     m_mainWindow->aboutToShutdown();
-    return SynchronousShutdown;
+    return shutdownFlag;
 }
