@@ -81,9 +81,16 @@ bool CurrentDocumentFind::supportsReplace() const
     return m_currentFind->supportsReplace();
 }
 
+bool CurrentDocumentFind::supportsSelectAll() const
+{
+    if (!m_currentFind)
+        return false;
+    return m_currentFind->supportsSelectAll();
+}
+
 FindFlags CurrentDocumentFind::supportedFindFlags() const
 {
-    QTC_ASSERT(m_currentFind, return nullptr);
+    QTC_ASSERT(m_currentFind, return {});
     return m_currentFind->supportedFindFlags();
 }
 
@@ -119,6 +126,12 @@ IFindSupport::Result CurrentDocumentFind::findStep(const QString &txt, FindFlags
     return m_currentFind->findStep(txt, findFlags);
 }
 
+void CurrentDocumentFind::selectAll(const QString &txt, FindFlags findFlags)
+{
+    QTC_ASSERT(m_currentFind && m_currentFind->supportsSelectAll(), return);
+    m_currentFind->selectAll(txt, findFlags);
+}
+
 void CurrentDocumentFind::replace(const QString &before, const QString &after, FindFlags findFlags)
 {
     QTC_ASSERT(m_currentFind, return);
@@ -165,8 +178,12 @@ void CurrentDocumentFind::updateCandidateFindFilter(QWidget *old, QWidget *now)
         if (!impl)
             candidate = candidate->parentWidget();
     }
-    if (candidate == m_candidateWidget && impl == m_candidateFind)
+    if (candidate == m_candidateWidget && impl == m_candidateFind) {
+        // trigger update of action state since a changed focus can still require disabling the
+        // Find/Replace action
+        emit changed();
         return;
+    }
     if (m_candidateWidget)
         disconnect(Aggregation::Aggregate::parentAggregate(m_candidateWidget), &Aggregation::Aggregate::changed,
                    this, &CurrentDocumentFind::candidateAggregationChanged);
