@@ -31,11 +31,23 @@
 #include <QScrollBar>
 #include <QStyle>
 #include <QStyleOptionSlider>
-#include <QTimer>
 
 using namespace Utils;
+
 namespace Core {
 
+/*!
+    \class Core::Highlight
+    \inmodule QtCreator
+    \internal
+*/
+
+/*!
+    \class Core::HighlightScrollBarController
+    \inmodule QtCreator
+    \internal
+*/
+
 class HighlightScrollBarOverlay : public QWidget
 {
 public:
@@ -92,7 +104,7 @@ void HighlightScrollBarOverlay::scheduleUpdate()
         return;
 
     m_isCacheUpdateScheduled = true;
-    QTimer::singleShot(0, this, QOverload<>::of(&QWidget::update));
+    QMetaObject::invokeMethod(this, QOverload<>::of(&QWidget::update), Qt::QueuedConnection);
 }
 
 void HighlightScrollBarOverlay::paintEvent(QPaintEvent *paintEvent)
@@ -110,8 +122,8 @@ void HighlightScrollBarOverlay::paintEvent(QPaintEvent *paintEvent)
     const QRect &gRect = overlayRect();
     const QRect &hRect = handleRect();
 
-    const int marginX = 3;
-    const int marginH = -2 * marginX + 1;
+    constexpr int marginX = 3;
+    constexpr int marginH = -2 * marginX + 1;
     const QRect aboveHandleRect = QRect(gRect.x() + marginX,
                                         gRect.y(),
                                         gRect.width() + marginH,
@@ -254,7 +266,7 @@ static void insertPosition(QMap<int, int> *map, int position)
 
     bool gluedWithPrev = false;
     if (itNext != map->begin()) {
-        auto itPrev = itNext - 1;
+        auto itPrev = std::prev(itNext);
         const int keyStart = itPrev.key();
         const int keyEnd = itPrev.value();
         if (position >= keyStart && position <= keyEnd)
@@ -272,7 +284,7 @@ static void insertPosition(QMap<int, int> *map, int position)
         itNext = map->erase(itNext);
         if (gluedWithPrev) {
             // glue with prev and next
-            auto itPrev = itNext - 1;
+            auto itPrev = std::prev(itNext);
             *itPrev = keyEnd;
         } else {
             // glue with next
