@@ -28,17 +28,17 @@
 #include "icore.h"
 #include "documentmanager.h"
 #include "idocument.h"
-#include "infobar.h"
 
 #include <coreplugin/dialogs/addtovcsdialog.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
 
-#include <vcsbase/vcsbaseconstants.h>
 #include <extensionsystem/pluginmanager.h>
 #include <utils/algorithm.h>
+#include <utils/infobar.h>
 #include <utils/optional.h>
 #include <utils/qtcassert.h>
+#include <vcsbase/vcsbaseconstants.h>
 
 #include <QDir>
 #include <QString>
@@ -48,6 +48,8 @@
 #include <QFileInfo>
 #include <QMessageBox>
 
+using namespace Utils;
+
 namespace Core {
 
 #if defined(WITH_TESTS)
@@ -77,7 +79,7 @@ public:
         QString topLevel;
     };
 
-    Utils::optional<VcsInfo> findInCache(const QString &dir)
+    Utils::optional<VcsInfo> findInCache(const QString &dir) const
     {
         QTC_ASSERT(QDir(dir).isAbsolute(), return Utils::nullopt);
         QTC_ASSERT(!dir.endsWith(QLatin1Char('/')), return Utils::nullopt);
@@ -99,7 +101,8 @@ public:
         QTC_ASSERT(QDir::fromNativeSeparators(dir) == dir, return);
 
         const QString dirSlash = dir + QLatin1Char('/');
-        foreach (const QString &key, m_cachedMatches.keys()) {
+        const QList<QString> keys = m_cachedMatches.keys();
+        for (const QString &key : keys) {
             if (key == dir || key.startsWith(dirSlash))
                 m_cachedMatches.remove(key);
         }
@@ -134,7 +137,7 @@ public:
     QMap<QString, VcsInfo> m_cachedMatches;
     IVersionControl *m_unconfiguredVcs = nullptr;
 
-    QStringList m_cachedAdditionalToolsPaths;
+    FilePaths m_cachedAdditionalToolsPaths;
     bool m_cachedAdditionalToolsPathsDirty = true;
 };
 
@@ -170,9 +173,13 @@ VcsManager *VcsManager::instance()
 void VcsManager::extensionsInitialized()
 {
     // Change signal connections
-    foreach (IVersionControl *versionControl, versionControls()) {
-        connect(versionControl, &IVersionControl::filesChanged,
-                DocumentManager::instance(), &DocumentManager::filesChangedInternally);
+    const QList<IVersionControl *> versionControlList = versionControls();
+    for (const IVersionControl *versionControl : versionControlList) {
+        connect(versionControl, &IVersionControl::filesChanged, DocumentManager::instance(),
+                [](const QStringList fileNames) {
+                    DocumentManager::notifyFilesChangedInternally(
+                        Utils::transform(fileNames, &Utils::FilePath::fromString));
+                });
         connect(versionControl, &IVersionControl::repositoryChanged,
                 m_instance, &VcsManager::repositoryChanged);
         connect(versionControl, &IVersionControl::configurationChanged,
@@ -198,21 +205,21 @@ static QString absoluteWithNoTrailingSlash(const QString &directory)
     return res;
 }
 
-void VcsManager::resetVersionControlForDirectory(const QString &inputDirectory)
+void VcsManager::resetVersionControlForDirectory(const FilePath &inputDirectory)
 {
     if (inputDirectory.isEmpty())
         return;
 
-    const QString directory = absoluteWithNoTrailingSlash(inputDirectory);
+    const QString directory = absoluteWithNoTrailingSlash(inputDirectory.toString());
     d->resetCache(directory);
-    emit m_instance->repositoryChanged(directory);
+    emit m_instance->repositoryChanged(FilePath::fromString(directory));
 }
 
-IVersionControl* VcsManager::findVersionControlForDirectory(const QString &inputDirectory,
+IVersionControl* VcsManager::findVersionControlForDirectory(const FilePath &inputDirectory,
                                                             QString *topLevelDirectory)
 {
-    typedef QPair<QString, IVersionControl *> StringVersionControlPair;
-    typedef QList<StringVersionControlPair> StringVersionControlPairs;
+    using StringVersionControlPair = QPair<QString, IVersionControl *>;
+    using StringVersionControlPairs = QList<StringVersionControlPair>;
     if (inputDirectory.isEmpty()) {
         if (topLevelDirectory)
             topLevelDirectory->clear();
@@ -220,7 +227,7 @@ IVersionControl* VcsManager::findVersionControlForDirectory(const QString &input
     }
 
     // Make sure we an absolute path:
-    QString directory = absoluteWithNoTrailingSlash(inputDirectory);
+    QString directory = absoluteWithNoTrailingSlash(inputDirectory.toString());
 #ifdef WITH_TESTS
     if (directory[0].isLetter() && directory.indexOf(QLatin1Char(':') + QLatin1String(TEST_PREFIX)) == 1)
         directory = directory.mid(2);
@@ -235,10 +242,11 @@ IVersionControl* VcsManager::findVersionControlForDirectory(const QString &input
     // Nothing: ask the IVersionControls directly.
     StringVersionControlPairs allThatCanManage;
 
-    foreach (IVersionControl * versionControl, versionControls()) {
-        QString topLevel;
-        if (versionControl->managesDirectory(directory, &topLevel))
-            allThatCanManage.push_back(StringVersionControlPair(topLevel, versionControl));
+    const QList<IVersionControl *> versionControlList = versionControls();
+    for (IVersionControl *versionControl : versionControlList) {
+        FilePath topLevel;
+        if (versionControl->managesDirectory(FilePath::fromString(directory), &topLevel))
+            allThatCanManage.push_back(StringVersionControlPair(topLevel.toString(), versionControl));
     }
 
     // To properly find a nested repository (say, git checkout inside SVN),
@@ -296,14 +304,14 @@ IVersionControl* VcsManager::findVersionControlForDirectory(const QString &input
             }
             return versionControl;
         } else {
-            InfoBar *infoBar = curDocument ? curDocument->infoBar() : nullptr;
+            Utils::InfoBar *infoBar = curDocument ? curDocument->infoBar() : nullptr;
             if (infoBar && infoBar->canInfoBeAdded(vcsWarning)) {
-                InfoBarEntry info(vcsWarning,
-                                  tr("%1 repository was detected but %1 is not configured.")
-                                  .arg(versionControl->displayName()),
-                                  InfoBarEntry::GlobalSuppressionEnabled);
+                Utils::InfoBarEntry info(vcsWarning,
+                                         tr("%1 repository was detected but %1 is not configured.")
+                                             .arg(versionControl->displayName()),
+                                         Utils::InfoBarEntry::GlobalSuppression::Enabled);
                 d->m_unconfiguredVcs = versionControl;
-                info.setCustomButtonInfo(ICore::msgShowOptionsDialog(), []() {
+                info.addCustomButton(ICore::msgShowOptionsDialog(), []() {
                     QTC_ASSERT(d->m_unconfiguredVcs, return);
                     ICore::showOptionsDialog(d->m_unconfiguredVcs->id());
                  });
@@ -316,11 +324,11 @@ IVersionControl* VcsManager::findVersionControlForDirectory(const QString &input
     return versionControl;
 }
 
-QString VcsManager::findTopLevelForDirectory(const QString &directory)
+FilePath VcsManager::findTopLevelForDirectory(const FilePath &directory)
 {
     QString result;
     findVersionControlForDirectory(directory, &result);
-    return result;
+    return FilePath::fromString(result);
 }
 
 QStringList VcsManager::repositories(const IVersionControl *vc)
@@ -333,26 +341,58 @@ QStringList VcsManager::repositories(const IVersionControl *vc)
     return result;
 }
 
-bool VcsManager::promptToDelete(const QString &fileName)
+bool VcsManager::promptToDelete(IVersionControl *versionControl, const QString &fileName)
 {
-    if (IVersionControl *vc = findVersionControlForDirectory(QFileInfo(fileName).absolutePath()))
-        return promptToDelete(vc, fileName);
-    return true;
+    return promptToDelete(versionControl, {Utils::FilePath::fromString(fileName)}).isEmpty();
 }
 
-bool VcsManager::promptToDelete(IVersionControl *vc, const QString &fileName)
+FilePaths VcsManager::promptToDelete(const FilePaths &filePaths)
 {
-    QTC_ASSERT(vc, return true);
+    // Categorize files by their parent directory, so we won't call
+    // findVersionControlForDirectory() more often than necessary.
+    QMap<FilePath, FilePaths> filesByParentDir;
+    for (const FilePath &fp : filePaths)
+        filesByParentDir[fp.absolutePath()].append(fp);
+
+    // Categorize by version control system.
+    QHash<IVersionControl *, FilePaths> filesByVersionControl;
+    for (auto it = filesByParentDir.cbegin(); it != filesByParentDir.cend(); ++it) {
+        IVersionControl * const vc = findVersionControlForDirectory(it.key());
+        if (vc)
+            filesByVersionControl[vc] << it.value();
+    }
+
+    // Remove the files.
+    FilePaths failedFiles;
+    for (auto it = filesByVersionControl.cbegin(); it != filesByVersionControl.cend(); ++it)
+        failedFiles << promptToDelete(it.key(), it.value());
+
+    return failedFiles;
+}
+
+FilePaths VcsManager::promptToDelete(IVersionControl *vc, const FilePaths &filePaths)
+{
+    QTC_ASSERT(vc, return {});
     if (!vc->supportsOperation(IVersionControl::DeleteOperation))
-        return true;
+        return {};
+
+    const QString fileListForUi = "<ul><li>" + transform(filePaths, [](const FilePath &fp) {
+        return fp.toUserOutput();
+    }).join("</li><li>") + "</li></ul>";
     const QString title = tr("Version Control");
-    const QString msg = tr("Would you like to remove this file from the version control system (%1)?\n"
-                           "Note: This might remove the local file.").arg(vc->displayName());
+    const QString msg = tr("Remove the following files from the version control system (%2)?"
+                           "%1Note: This might remove the local file.").arg(fileListForUi, vc->displayName());
     const QMessageBox::StandardButton button =
         QMessageBox::question(ICore::dialogParent(), title, msg, QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes);
     if (button != QMessageBox::Yes)
-        return true;
-    return vc->vcsDelete(fileName);
+        return {};
+
+    FilePaths failedFiles;
+    for (const FilePath &fp : filePaths) {
+        if (!vc->vcsDelete(fp))
+            failedFiles << fp;
+    }
+    return failedFiles;
 }
 
 QString VcsManager::msgAddToVcsTitle()
@@ -383,65 +423,61 @@ QString VcsManager::msgToAddToVcsFailed(const QStringList &files, const IVersion
               .arg(vc->displayName(), files.join(QString(QLatin1Char('\n'))));
 }
 
-QStringList VcsManager::additionalToolsPath()
+FilePaths VcsManager::additionalToolsPath()
 {
     if (d->m_cachedAdditionalToolsPathsDirty) {
         d->m_cachedAdditionalToolsPaths.clear();
-        foreach (IVersionControl *vc, versionControls())
+        for (IVersionControl *vc : versionControls())
             d->m_cachedAdditionalToolsPaths.append(vc->additionalToolsPath());
         d->m_cachedAdditionalToolsPathsDirty = false;
     }
     return d->m_cachedAdditionalToolsPaths;
 }
 
-void VcsManager::promptToAdd(const QString &directory, const QStringList &fileNames)
+void VcsManager::promptToAdd(const FilePath &directory, const FilePaths &filePaths)
 {
     IVersionControl *vc = findVersionControlForDirectory(directory);
     if (!vc || !vc->supportsOperation(IVersionControl::AddOperation))
         return;
 
-    QStringList unmanagedFiles;
-    QDir dir(directory);
-    foreach (const QString &fileName, fileNames) {
-        if (!vc->managesFile(directory, dir.relativeFilePath(fileName)))
-            unmanagedFiles << fileName;
-    }
+    const FilePaths unmanagedFiles = vc->unmanagedFiles(filePaths);
     if (unmanagedFiles.isEmpty())
         return;
 
-    Internal::AddToVcsDialog dlg(ICore::mainWindow(), VcsManager::msgAddToVcsTitle(),
+    Internal::AddToVcsDialog dlg(ICore::dialogParent(), VcsManager::msgAddToVcsTitle(),
                                  unmanagedFiles, vc->displayName());
     if (dlg.exec() == QDialog::Accepted) {
         QStringList notAddedToVc;
-        foreach (const QString &file, unmanagedFiles) {
-            if (!vc->vcsAdd(file))
-                notAddedToVc << file;
+        for (const FilePath &file : unmanagedFiles) {
+            if (!vc->vcsAdd(directory.resolvePath(file)))
+                notAddedToVc << file.toUserOutput();
         }
 
         if (!notAddedToVc.isEmpty()) {
-            QMessageBox::warning(ICore::mainWindow(), VcsManager::msgAddToVcsFailedTitle(),
+            QMessageBox::warning(ICore::dialogParent(),
+                                 VcsManager::msgAddToVcsFailedTitle(),
                                  VcsManager::msgToAddToVcsFailed(notAddedToVc, vc));
         }
     }
 }
 
-void VcsManager::emitRepositoryChanged(const QString &repository)
+void VcsManager::emitRepositoryChanged(const FilePath &repository)
 {
     emit m_instance->repositoryChanged(repository);
 }
 
 void VcsManager::clearVersionControlCache()
 {
-    QStringList repoList = d->m_cachedMatches.keys();
+    const QStringList repoList = d->m_cachedMatches.keys();
     d->clearCache();
-    foreach (const QString &repo, repoList)
-        emit m_instance->repositoryChanged(repo);
+    for (const QString &repo : repoList)
+        emit m_instance->repositoryChanged(FilePath::fromString(repo));
 }
 
 void VcsManager::handleConfigurationChanges()
 {
     d->m_cachedAdditionalToolsPathsDirty = true;
-    IVersionControl *vcs = qobject_cast<IVersionControl *>(sender());
+    auto vcs = qobject_cast<IVersionControl *>(sender());
     if (vcs)
         emit configurationChanged(vcs);
 }
@@ -462,16 +498,16 @@ namespace Internal {
 const char ID_VCS_A[] = "A";
 const char ID_VCS_B[] = "B";
 
-typedef QHash<QString, QString> FileHash;
+using FileHash = QHash<FilePath, FilePath>;
 
 static FileHash makeHash(const QStringList &list)
 {
     FileHash result;
-    foreach (const QString &i, list) {
+    for (const QString &i : list) {
         QStringList parts = i.split(QLatin1Char(':'));
         QTC_ASSERT(parts.count() == 2, continue);
-        result.insert(QString::fromLatin1(TEST_PREFIX) + parts.at(0),
-                      QString::fromLatin1(TEST_PREFIX) + parts.at(1));
+        result.insert(FilePath::fromString(QString::fromLatin1(TEST_PREFIX) + parts.at(0)),
+                      FilePath::fromString(QString::fromLatin1(TEST_PREFIX) + parts.at(1)));
     }
     return result;
 }
@@ -556,7 +592,7 @@ void CorePlugin::testVcsManager()
 
     // From VCSes:
     int expectedCount = 0;
-    foreach (const QString &result, results) {
+    for (const QString &result : qAsConst(results)) {
         // qDebug() << "Expecting:" << result;
 
         QStringList split = result.split(QLatin1Char(':'));
@@ -573,7 +609,8 @@ void CorePlugin::testVcsManager()
             ++expectedCount;
 
         IVersionControl *vcs;
-        vcs = VcsManager::findVersionControlForDirectory(makeString(directory), &realTopLevel);
+        vcs = VcsManager::findVersionControlForDirectory(
+            FilePath::fromString(makeString(directory)), &realTopLevel);
         QCOMPARE(realTopLevel, makeString(topLevel));
         if (vcs)
             QCOMPARE(vcs->id().toString(), vcsId);
