@@ -25,21 +25,22 @@
 
 #include "debuggerdialogs.h"
 
+#include "cdb/cdbengine.h"
 #include "debuggerkitinformation.h"
 #include "debuggerruncontrol.h"
-#include "cdb/cdbengine.h"
+
+#include <app/app_version.h>
 
 #include <coreplugin/icore.h>
+
+#include <projectexplorer/devicesupport/sshparameters.h>
 #include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/toolchain.h>
 
-#include <app/app_version.h>
-#include <utils/pathchooser.h>
 #include <utils/fancylineedit.h>
+#include <utils/pathchooser.h>
 #include <utils/qtcassert.h>
 
-#include <ssh/sshconnection.h>
-
 #include <QButtonGroup>
 #include <QCheckBox>
 #include <QComboBox>
@@ -49,9 +50,10 @@
 #include <QFormLayout>
 #include <QGroupBox>
 #include <QLabel>
+#include <QPlainTextEdit>
 #include <QPushButton>
 #include <QRadioButton>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QScrollArea>
 #include <QSpinBox>
 
@@ -82,9 +84,14 @@ public:
     PathChooser *workingDirectory;
     QCheckBox *breakAtMainCheckBox;
     QCheckBox *runInTerminalCheckBox;
+    QCheckBox *useTargetExtendedRemoteCheckBox;
     PathChooser *debuginfoPathChooser;
-    QLabel *serverStartScriptLabel;
-    PathChooser *serverStartScriptPathChooser;
+    QLabel *sysRootLabel;
+    PathChooser *sysRootPathChooser;
+    QLabel *serverInitCommandsLabel;
+    QPlainTextEdit *serverInitCommandsTextEdit;
+    QLabel *serverResetCommandsLabel;
+    QPlainTextEdit *serverResetCommandsTextEdit;
     QComboBox *historyComboBox;
     QDialogButtonBox *buttonBox;
 };
@@ -97,36 +104,6 @@ Q_DECLARE_METATYPE(Debugger::Internal::StartApplicationParameters)
 namespace Debugger {
 namespace Internal {
 
-///////////////////////////////////////////////////////////////////////
-//
-// DebuggerKitChooser
-//
-///////////////////////////////////////////////////////////////////////
-
-DebuggerKitChooser::DebuggerKitChooser(Mode mode, QWidget *parent)
-    : KitChooser(parent)
-    , m_hostAbi(Abi::hostAbi())
-    , m_mode(mode)
-{
-    setKitPredicate([this](const Kit *k) {
-        // Match valid debuggers and restrict local debugging to compatible toolchains.
-        auto errors = DebuggerKitAspect::configurationErrors(k);
-        // we do not care for mismatched ABI if we want *any* debugging
-        if (m_mode == AnyDebugging && errors == DebuggerKitAspect::DebuggerDoesNotMatch)
-            errors = DebuggerKitAspect::NoConfigurationError;
-        if (errors)
-            return false;
-        if (m_mode == LocalDebugging)
-            return ToolChainKitAspect::targetAbi(k).os() == m_hostAbi.os();
-        return true;
-    });
-}
-
-QString DebuggerKitChooser::kitToolTip(Kit *k) const
-{
-    return DebuggerKitAspect::displayString(k);
-}
-
 ///////////////////////////////////////////////////////////////////////
 //
 // StartApplicationParameters
@@ -150,19 +127,23 @@ public:
     Runnable runnable;
     bool breakAtMain = false;
     bool runInTerminal = false;
-    FilePath serverStartScript;
-    QString debugInfoLocation;
+    bool useTargetExtendedRemote = false;
+    FilePath sysRoot;
+    QString serverInitCommands;
+    QString serverResetCommands;
+    FilePath debugInfoLocation;
 };
 
 bool StartApplicationParameters::equals(const StartApplicationParameters &rhs) const
 {
-    return runnable.executable == rhs.runnable.executable
+    return runnable.command == rhs.runnable.command
         && serverPort == rhs.serverPort
-        && runnable.commandLineArguments == rhs.runnable.commandLineArguments
         && runnable.workingDirectory == rhs.runnable.workingDirectory
         && breakAtMain == rhs.breakAtMain
         && runInTerminal == rhs.runInTerminal
-        && serverStartScript == rhs.serverStartScript
+        && sysRoot == rhs.sysRoot
+        && serverInitCommands == rhs.serverInitCommands
+        && serverResetCommands == rhs.serverResetCommands
         && kitId == rhs.kitId
         && debugInfoLocation == rhs.debugInfoLocation
         && serverAddress == rhs.serverAddress;
@@ -172,8 +153,8 @@ QString StartApplicationParameters::displayName() const
 {
     const int maxLength = 60;
 
-    QString name = FilePath::fromString(runnable.executable).fileName()
-            + ' ' + runnable.commandLineArguments;
+    QString name = runnable.command.executable().fileName()
+            + ' ' + runnable.command.arguments();
     if (name.size() > 60) {
         int index = name.lastIndexOf(' ', maxLength);
         if (index == -1)
@@ -193,13 +174,16 @@ void StartApplicationParameters::toSettings(QSettings *settings) const
     settings->setValue("LastKitId", kitId.toSetting());
     settings->setValue("LastServerPort", serverPort);
     settings->setValue("LastServerAddress", serverAddress);
-    settings->setValue("LastExternalExecutable", runnable.executable);
-    settings->setValue("LastExternalExecutableArguments", runnable.commandLineArguments);
-    settings->setValue("LastExternalWorkingDirectory", runnable.workingDirectory);
+    settings->setValue("LastExternalExecutable", runnable.command.executable().toVariant());
+    settings->setValue("LastExternalExecutableArguments", runnable.command.arguments());
+    settings->setValue("LastExternalWorkingDirectory", runnable.workingDirectory.toVariant());
     settings->setValue("LastExternalBreakAtMain", breakAtMain);
     settings->setValue("LastExternalRunInTerminal", runInTerminal);
-    settings->setValue("LastServerStartScript", serverStartScript.toVariant());
-    settings->setValue("LastDebugInfoLocation", debugInfoLocation);
+    settings->setValue("LastExternalUseTargetExtended", useTargetExtendedRemote);
+    settings->setValue("LastServerInitCommands", serverInitCommands);
+    settings->setValue("LastServerResetCommands", serverResetCommands);
+    settings->setValue("LastDebugInfoLocation", debugInfoLocation.toVariant());
+    settings->setValue("LastSysRoot", sysRoot.toVariant());
 }
 
 void StartApplicationParameters::fromSettings(const QSettings *settings)
@@ -207,13 +191,16 @@ void StartApplicationParameters::fromSettings(const QSettings *settings)
     kitId = Id::fromSetting(settings->value("LastKitId"));
     serverPort = settings->value("LastServerPort").toUInt();
     serverAddress = settings->value("LastServerAddress").toString();
-    runnable.executable = settings->value("LastExternalExecutable").toString();
-    runnable.commandLineArguments = settings->value("LastExternalExecutableArguments").toString();
-    runnable.workingDirectory = settings->value("LastExternalWorkingDirectory").toString();
+    runnable.command.setExecutable(FilePath::fromVariant(settings->value("LastExternalExecutable")));
+    runnable.command.setArguments(settings->value("LastExternalExecutableArguments").toString());
+    runnable.workingDirectory = FilePath::fromVariant(settings->value("LastExternalWorkingDirectory"));
     breakAtMain = settings->value("LastExternalBreakAtMain").toBool();
     runInTerminal = settings->value("LastExternalRunInTerminal").toBool();
-    serverStartScript = FilePath::fromVariant(settings->value("LastServerStartScript"));
-    debugInfoLocation = settings->value("LastDebugInfoLocation").toString();
+    useTargetExtendedRemote = settings->value("LastExternalUseTargetExtended").toBool();
+    serverInitCommands = settings->value("LastServerInitCommands").toString();
+    serverResetCommands = settings->value("LastServerResetCommands").toString();
+    debugInfoLocation = FilePath::fromVariant(settings->value("LastDebugInfoLocation"));
+    sysRoot = FilePath::fromVariant(settings->value("LastSysRoot"));
 }
 
 ///////////////////////////////////////////////////////////////////////
@@ -225,7 +212,6 @@ void StartApplicationParameters::fromSettings(const QSettings *settings)
 StartApplicationDialog::StartApplicationDialog(QWidget *parent)
   : QDialog(parent), d(new StartApplicationDialogPrivate)
 {
-    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
     setWindowTitle(tr("Start Debugger"));
 
     d->kitChooser = new KitChooser(this);
@@ -266,16 +252,33 @@ StartApplicationDialog::StartApplicationDialog(QWidget *parent)
     d->breakAtMainCheckBox = new QCheckBox(this);
     d->breakAtMainCheckBox->setText(QString());
 
-    d->serverStartScriptPathChooser = new PathChooser(this);
-    d->serverStartScriptPathChooser->setExpectedKind(PathChooser::File);
-    d->serverStartScriptPathChooser->setPromptDialogTitle(tr("Select Server Start Script"));
-    d->serverStartScriptPathChooser->setToolTip(tr(
-        "This option can be used to point to a script that will be used "
-        "to start a debug server. If the field is empty, "
-        "default methods to set up debug servers will be used."));
-    d->serverStartScriptLabel = new QLabel(tr("&Server start script:"), this);
-    d->serverStartScriptLabel->setBuddy(d->serverStartScriptPathChooser);
-    d->serverStartScriptLabel->setToolTip(d->serverStartScriptPathChooser->toolTip());
+    d->useTargetExtendedRemoteCheckBox = new QCheckBox(this);
+
+    d->sysRootPathChooser = new PathChooser(this);
+    d->sysRootPathChooser->setExpectedKind(PathChooser::Directory);
+    d->sysRootPathChooser->setHistoryCompleter("Debugger.SysRoot.History");
+    d->sysRootPathChooser->setPromptDialogTitle(tr("Select SysRoot Directory"));
+    d->sysRootPathChooser->setToolTip(tr(
+        "This option can be used to override the kit's SysRoot setting."));
+    d->sysRootLabel = new QLabel(tr("Override S&ysRoot:"), this);
+    d->sysRootLabel->setBuddy(d->sysRootPathChooser);
+    d->sysRootLabel->setToolTip(d->sysRootPathChooser->toolTip());
+
+    d->serverInitCommandsTextEdit = new QPlainTextEdit(this);
+    d->serverInitCommandsTextEdit->setToolTip(tr(
+        "This option can be used to send the target init commands."));
+
+    d->serverInitCommandsLabel = new QLabel(tr("&Init commands:"), this);
+    d->serverInitCommandsLabel->setBuddy(d->serverInitCommandsTextEdit);
+    d->serverInitCommandsLabel->setToolTip(d->serverInitCommandsTextEdit->toolTip());
+
+    d->serverResetCommandsTextEdit = new QPlainTextEdit(this);
+    d->serverResetCommandsTextEdit->setToolTip(tr(
+        "This option can be used to send the target reset commands."));
+
+    d->serverResetCommandsLabel = new QLabel(tr("&Reset commands:"), this);
+    d->serverResetCommandsLabel->setBuddy(d->serverResetCommandsTextEdit);
+    d->serverResetCommandsLabel->setToolTip(d->serverResetCommandsTextEdit->toolTip());
 
     d->debuginfoPathChooser = new PathChooser(this);
     d->debuginfoPathChooser->setPromptDialogTitle(tr("Select Location of Debugging Information"));
@@ -307,7 +310,10 @@ StartApplicationDialog::StartApplicationDialog(QWidget *parent)
     formLayout->addRow(tr("&Working directory:"), d->workingDirectory);
     formLayout->addRow(tr("Run in &terminal:"), d->runInTerminalCheckBox);
     formLayout->addRow(tr("Break at \"&main\":"), d->breakAtMainCheckBox);
-    formLayout->addRow(d->serverStartScriptLabel, d->serverStartScriptPathChooser);
+    formLayout->addRow(tr("Use target extended-remote to connect:"), d->useTargetExtendedRemoteCheckBox);
+    formLayout->addRow(d->sysRootLabel, d->sysRootPathChooser);
+    formLayout->addRow(d->serverInitCommandsLabel, d->serverInitCommandsTextEdit);
+    formLayout->addRow(d->serverResetCommandsLabel, d->serverResetCommandsTextEdit);
     formLayout->addRow(tr("Debug &information:"), d->debuginfoPathChooser);
     formLayout->addRow(d->channelOverrideHintLabel);
     formLayout->addRow(d->channelOverrideLabel, d->channelOverrideEdit);
@@ -343,7 +349,7 @@ void StartApplicationDialog::setHistory(const QList<StartApplicationParameters>
     d->historyComboBox->clear();
     for (int i = l.size(); --i >= 0; ) {
         const StartApplicationParameters &p = l.at(i);
-        if (!p.runnable.executable.isEmpty())
+        if (!p.runnable.command.isEmpty())
             d->historyComboBox->addItem(p.displayName(), QVariant::fromValue(p));
     }
 }
@@ -394,8 +400,10 @@ void StartApplicationDialog::run(bool attachRemote)
     dialog.setHistory(history);
     dialog.setParameters(history.back());
     if (!attachRemote) {
-        dialog.d->serverStartScriptPathChooser->setVisible(false);
-        dialog.d->serverStartScriptLabel->setVisible(false);
+        dialog.d->serverInitCommandsTextEdit->setVisible(false);
+        dialog.d->serverInitCommandsLabel->setVisible(false);
+        dialog.d->serverResetCommandsTextEdit->setVisible(false);
+        dialog.d->serverResetCommandsLabel->setVisible(false);
         dialog.d->serverPortSpinBox->setVisible(false);
         dialog.d->serverPortLabel->setVisible(false);
         dialog.d->channelOverrideHintLabel->setVisible(false);
@@ -437,15 +445,17 @@ void StartApplicationDialog::run(bool attachRemote)
     debugger->setBreakOnMain(newParameters.breakAtMain);
     debugger->setDebugInfoLocation(newParameters.debugInfoLocation);
     debugger->setInferior(inferior);
-    debugger->setServerStartScript(newParameters.serverStartScript); // Note: This requires inferior.
+    debugger->setCommandsAfterConnect(newParameters.serverInitCommands);
+    debugger->setCommandsForReset(newParameters.serverResetCommands);
     debugger->setUseTerminal(newParameters.runInTerminal);
+    debugger->setUseExtendedRemote(newParameters.useTargetExtendedRemote);
+    if (!newParameters.sysRoot.isEmpty())
+        debugger->setSysRoot(newParameters.sysRoot);
 
     bool isLocal = !dev || (dev->type() == ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE);
-    if (isLocal) {
-        Environment inferiorEnvironment = Environment::systemEnvironment();
-        k->addToEnvironment(inferiorEnvironment);
-        debugger->setInferiorEnvironment(inferiorEnvironment);
-    }
+    if (isLocal) // FIXME: Restriction needed?
+        debugger->setInferiorEnvironment(k->runEnvironment());
+
     if (!attachRemote)
         debugger->setStartMode(isLocal ? StartExternal : StartRemoteProcess);
 
@@ -453,6 +463,7 @@ void StartApplicationDialog::run(bool attachRemote)
         debugger->setStartMode(AttachToRemoteServer);
         debugger->setCloseMode(KillAtClose);
         debugger->setUseContinueInsteadOfRun(true);
+        debugger->setRunControlName(tr("Attach to %1").arg(debugger->remoteChannel()));
     }
     debugger->startRunControl();
 }
@@ -472,14 +483,17 @@ StartApplicationParameters StartApplicationDialog::parameters() const
     StartApplicationParameters result;
     result.serverPort = d->serverPortSpinBox->value();
     result.serverAddress = d->channelOverrideEdit->text();
-    result.runnable.executable = d->localExecutablePathChooser->path();
-    result.serverStartScript = d->serverStartScriptPathChooser->fileName();
+    result.runnable.command.setExecutable(d->localExecutablePathChooser->filePath());
+    result.sysRoot = d->sysRootPathChooser->filePath();
+    result.serverInitCommands = d->serverInitCommandsTextEdit->toPlainText();
+    result.serverResetCommands = d->serverResetCommandsTextEdit->toPlainText();
     result.kitId = d->kitChooser->currentKitId();
-    result.debugInfoLocation = d->debuginfoPathChooser->path();
-    result.runnable.commandLineArguments = d->arguments->text();
-    result.runnable.workingDirectory = d->workingDirectory->path();
+    result.debugInfoLocation = d->debuginfoPathChooser->filePath();
+    result.runnable.command.setArguments(d->arguments->text());
+    result.runnable.workingDirectory = d->workingDirectory->filePath();
     result.breakAtMain = d->breakAtMainCheckBox->isChecked();
     result.runInTerminal = d->runInTerminalCheckBox->isChecked();
+    result.useTargetExtendedRemote = d->useTargetExtendedRemoteCheckBox->isChecked();
     return result;
 }
 
@@ -488,13 +502,16 @@ void StartApplicationDialog::setParameters(const StartApplicationParameters &p)
     d->kitChooser->setCurrentKitId(p.kitId);
     d->serverPortSpinBox->setValue(p.serverPort);
     d->channelOverrideEdit->setText(p.serverAddress);
-    d->localExecutablePathChooser->setPath(p.runnable.executable);
-    d->serverStartScriptPathChooser->setFileName(p.serverStartScript);
-    d->debuginfoPathChooser->setPath(p.debugInfoLocation);
-    d->arguments->setText(p.runnable.commandLineArguments);
-    d->workingDirectory->setPath(p.runnable.workingDirectory);
-    d->runInTerminalCheckBox->setChecked(p.runInTerminal);
+    d->localExecutablePathChooser->setFilePath(p.runnable.command.executable());
+    d->sysRootPathChooser->setFilePath(p.sysRoot);
+    d->serverInitCommandsTextEdit->setPlainText(p.serverInitCommands);
+    d->serverResetCommandsTextEdit->setPlainText(p.serverResetCommands);
+    d->debuginfoPathChooser->setFilePath(p.debugInfoLocation);
+    d->arguments->setText(p.runnable.command.arguments());
+    d->workingDirectory->setFilePath(p.runnable.workingDirectory);
     d->breakAtMainCheckBox->setChecked(p.breakAtMain);
+    d->runInTerminalCheckBox->setChecked(p.runInTerminal);
+    d->useTargetExtendedRemoteCheckBox->setChecked(p.useTargetExtendedRemote);
     updateState();
 }
 
@@ -515,10 +532,10 @@ AttachToQmlPortDialog::AttachToQmlPortDialog(QWidget *parent)
   : QDialog(parent),
     d(new AttachToQmlPortDialogPrivate)
 {
-    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
     setWindowTitle(tr("Start Debugger"));
 
-    d->kitChooser = new DebuggerKitChooser(DebuggerKitChooser::AnyDebugging, this);
+    d->kitChooser = new KitChooser(this);
+    d->kitChooser->setShowIcons(true);
     d->kitChooser->populate();
 
     d->portSpinBox = new QSpinBox(this);
@@ -569,7 +586,7 @@ void AttachToQmlPortDialog::setKitId(Id id)
 // --------- StartRemoteCdbDialog
 static QString cdbRemoteHelp()
 {
-    const char *cdbConnectionSyntax =
+    const char cdbConnectionSyntax[] =
             "Server:Port<br>"
             "tcp:server=Server,port=Port[,password=Password][,ipversion=6]\n"
             "tcp:clicon=Server,port=Port[,password=Password][,ipversion=6]\n"
@@ -588,17 +605,16 @@ static QString cdbRemoteHelp()
                 "Launch the remote CDB as <code>%5 &lt;executable&gt;</code> "
                 "to use TCP/IP as communication protocol.</p><p>Enter the connection parameters as:</p>"
                 "<pre>%6</pre></body></html>")
-            .arg(Core::Constants::IDE_DISPLAY_NAME,
-                 ext32, ext64, "_NT_DEBUGGER_EXTENSION_PATH",
-                 "cdb.exe -server tcp:port=1234",
-                 QLatin1String(cdbConnectionSyntax));
+            .arg(QString(Core::Constants::IDE_DISPLAY_NAME),
+                 ext32, ext64, QString("_NT_DEBUGGER_EXTENSION_PATH"),
+                 QString("cdb.exe -server tcp:port=1234"),
+                 QString(cdbConnectionSyntax));
 }
 
 StartRemoteCdbDialog::StartRemoteCdbDialog(QWidget *parent) :
     QDialog(parent), m_lineEdit(new QLineEdit)
 {
     setWindowTitle(tr("Start a CDB Remote Session"));
-    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
     auto groupBox = new QGroupBox;
 
@@ -651,10 +667,11 @@ QString StartRemoteCdbDialog::connection() const
 {
     const QString rc = m_lineEdit->text();
     // Transform an IP:POrt ('localhost:1234') specification into full spec
-    QRegExp ipRegexp("([\\w\\.\\-_]+):([0-9]{1,4})");
+    QRegularExpression ipRegexp("([\\w\\.\\-_]+):([0-9]{1,4})");
     QTC_ASSERT(ipRegexp.isValid(), return QString());
-    if (ipRegexp.exactMatch(rc))
-        return QString::fromLatin1("tcp:server=%1,port=%2").arg(ipRegexp.cap(1), ipRegexp.cap(2));
+    const QRegularExpressionMatch match = ipRegexp.match(rc);
+    if (match.hasMatch())
+        return QString::fromLatin1("tcp:server=%1,port=%2").arg(match.captured(1), match.captured(2));
     return rc;
 }
 
@@ -670,7 +687,6 @@ AddressDialog::AddressDialog(QWidget *parent) :
         m_box(new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel))
 {
     setWindowTitle(tr("Select Start Address"));
-    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
     auto hLayout = new QHBoxLayout;
     hLayout->addWidget(new QLabel(tr("Enter an address:") + ' '));
@@ -748,7 +764,6 @@ public:
 StartRemoteEngineDialog::StartRemoteEngineDialog(QWidget *parent)
     : QDialog(parent), d(new StartRemoteEngineDialogPrivate)
 {
-    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
     setWindowTitle(tr("Start Remote Engine"));
 
     d->host = new FancyLineEdit(this);
@@ -902,7 +917,6 @@ TypeFormatsDialog::TypeFormatsDialog(QWidget *parent)
    : QDialog(parent), m_ui(new TypeFormatsDialogUi(this))
 {
     setWindowTitle(tr("Type Formats"));
-    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
     m_ui->addPage(tr("Qt Types"));
     m_ui->addPage(tr("Standard Types"));
     m_ui->addPage(tr("Misc Types"));
