@@ -25,23 +25,21 @@
 
 #include "loadcoredialog.h"
 
-#include "debuggerdialogs.h"
 #include "debuggerkitinformation.h"
 #include "gdb/gdbengine.h"
 
-#include <projectexplorer/kitinformation.h>
+#include <projectexplorer/devicesupport/devicefilesystemmodel.h>
+#include <projectexplorer/devicesupport/filetransfer.h>
+#include <projectexplorer/devicesupport/idevice.h>
+#include <projectexplorer/kitchooser.h>
 #include <projectexplorer/projectexplorerconstants.h>
-#include <ssh/sftpfilesystemmodel.h>
-#include <ssh/sshconnection.h>
+
 #include <utils/pathchooser.h>
+#include <utils/processinterface.h>
 #include <utils/qtcassert.h>
 #include <utils/temporaryfile.h>
 
 #include <QCheckBox>
-#include <QDebug>
-#include <QDir>
-#include <QRegExp>
-
 #include <QDialogButtonBox>
 #include <QFormLayout>
 #include <QHeaderView>
@@ -54,7 +52,6 @@
 
 using namespace Core;
 using namespace ProjectExplorer;
-using namespace QSsh;
 using namespace Utils;
 
 namespace Debugger {
@@ -74,24 +71,20 @@ public:
     explicit SelectRemoteFileDialog(QWidget *parent);
 
     void attachToDevice(Kit *k);
-    QString localFile() const { return m_localFile; }
-    QString remoteFile() const { return m_remoteFile; }
+    FilePath localFile() const { return m_localFile; }
+    FilePath remoteFile() const { return m_remoteFile; }
 
 private:
-    void handleSftpOperationFinished(SftpJobId, const QString &error);
-    void handleSftpOperationFailed(const QString &errorMessage);
-    void handleConnectionError(const QString &errorMessage);
-    void handleRemoteError(const QString &errorMessage);
     void selectFile();
 
     QSortFilterProxyModel m_model;
-    SftpFileSystemModel m_fileSystemModel;
+    DeviceFileSystemModel m_fileSystemModel;
     QTreeView *m_fileSystemView;
     QTextBrowser *m_textBrowser;
     QDialogButtonBox *m_buttonBox;
-    QString m_localFile;
-    QString m_remoteFile;
-    SftpJobId m_sftpJobId;
+    FilePath m_localFile;
+    FilePath m_remoteFile;
+    FileTransfer m_fileTransfer;
 };
 
 SelectRemoteFileDialog::SelectRemoteFileDialog(QWidget *parent)
@@ -110,7 +103,7 @@ SelectRemoteFileDialog::SelectRemoteFileDialog(QWidget *parent)
     m_fileSystemView->header()->setStretchLastSection(true);
 
     m_textBrowser = new QTextBrowser(this);
-    m_textBrowser->setEnabled(false);
+    m_textBrowser->setReadOnly(true);
 
     m_buttonBox = new QDialogButtonBox(this);
     m_buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
@@ -122,15 +115,23 @@ SelectRemoteFileDialog::SelectRemoteFileDialog(QWidget *parent)
     layout->addWidget(m_textBrowser);
     layout->addWidget(m_buttonBox);
 
-    connect(m_buttonBox, &QDialogButtonBox::rejected,
-            this, &QDialog::reject);
-    connect(m_buttonBox, &QDialogButtonBox::accepted,
-            this, &SelectRemoteFileDialog::selectFile);
+    connect(m_buttonBox, &QDialogButtonBox::rejected, this, &QDialog::reject);
+    connect(m_buttonBox, &QDialogButtonBox::accepted, this, &SelectRemoteFileDialog::selectFile);
 
-    connect(&m_fileSystemModel, &SftpFileSystemModel::sftpOperationFailed,
-            this, &SelectRemoteFileDialog::handleSftpOperationFailed);
-    connect(&m_fileSystemModel, &SftpFileSystemModel::connectionError,
-            this, &SelectRemoteFileDialog::handleConnectionError);
+    connect(&m_fileTransfer, &FileTransfer::done, this, [this](const ProcessResultData &result) {
+        const bool success = result.m_error == QProcess::UnknownError
+                          && result.m_exitStatus == QProcess::NormalExit
+                          && result.m_exitCode == 0;
+        if (success) {
+            m_textBrowser->append(tr("Download of remote file succeeded."));
+            accept();
+        } else {
+            m_textBrowser->append(tr("Download of remote file failed: %1")
+                                  .arg(result.m_errorString));
+            m_buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
+            m_fileSystemView->setEnabled(true);
+        }
+    });
 }
 
 void SelectRemoteFileDialog::attachToDevice(Kit *k)
@@ -139,36 +140,7 @@ void SelectRemoteFileDialog::attachToDevice(Kit *k)
     QTC_ASSERT(k, return);
     IDevice::ConstPtr device = DeviceKitAspect::device(k);
     QTC_ASSERT(device, return);
-    SshConnectionParameters sshParams = device->sshParameters();
-    m_fileSystemModel.setSshConnection(sshParams);
-}
-
-void SelectRemoteFileDialog::handleSftpOperationFailed(const QString &errorMessage)
-{
-    m_textBrowser->append(errorMessage);
-    //reject();
-}
-
-void SelectRemoteFileDialog::handleConnectionError(const QString &errorMessage)
-{
-    m_textBrowser->append(errorMessage);
-    //reject();
-}
-
-void SelectRemoteFileDialog::handleSftpOperationFinished(SftpJobId, const QString &error)
-{
-    if (error.isEmpty()) {
-        m_textBrowser->append(tr("Download of remote file succeeded."));
-        accept();
-    } else {
-        m_textBrowser->append(error);
-        //reject();
-    }
-}
-
-void SelectRemoteFileDialog::handleRemoteError(const QString &errorMessage)
-{
-    m_textBrowser->append(errorMessage);
+    m_fileSystemModel.setDevice(device);
 }
 
 void SelectRemoteFileDialog::selectFile()
@@ -180,18 +152,17 @@ void SelectRemoteFileDialog::selectFile()
     m_buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
     m_fileSystemView->setEnabled(false);
 
-    connect(&m_fileSystemModel, &SftpFileSystemModel::sftpOperationFinished,
-            this, &SelectRemoteFileDialog::handleSftpOperationFinished);
-
     {
-        Utils::TemporaryFile localFile("remotecore-XXXXXX");
+        TemporaryFile localFile("remotecore-XXXXXX");
         localFile.open();
-        m_localFile = localFile.fileName();
+        m_localFile = FilePath::fromString(localFile.fileName());
     }
 
     idx = idx.sibling(idx.row(), 1);
-    m_remoteFile = m_fileSystemModel.data(idx, SftpFileSystemModel::PathRole).toString();
-    m_sftpJobId = m_fileSystemModel.downloadFile(idx, m_localFile);
+    m_remoteFile = FilePath::fromVariant(m_fileSystemModel.data(idx, DeviceFileSystemModel::PathRole));
+
+    m_fileTransfer.setFilesToTransfer({{m_remoteFile, m_localFile}});
+    m_fileTransfer.start();
 }
 
 ///////////////////////////////////////////////////////////////////////
@@ -213,6 +184,7 @@ public:
     QPushButton *selectRemoteCoreButton;
 
     PathChooser *overrideStartScriptFileName;
+    PathChooser *sysRootDirectory;
 
     QDialogButtonBox *buttonBox;
 
@@ -251,7 +223,6 @@ AttachCoreDialog::AttachCoreDialog(QWidget *parent)
     : QDialog(parent), d(new AttachCoreDialogPrivate)
 {
     setWindowTitle(tr("Load Core File"));
-    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
     d->buttonBox = new QDialogButtonBox(this);
     d->buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
@@ -289,6 +260,13 @@ AttachCoreDialog::AttachCoreDialog(QWidget *parent)
     d->overrideStartScriptFileName->setExpectedKind(PathChooser::File);
     d->overrideStartScriptFileName->setPromptDialogTitle(tr("Select Startup Script"));
 
+    d->sysRootDirectory = new PathChooser(this);
+    d->sysRootDirectory->setHistoryCompleter("Debugger.SysRoot.History");
+    d->sysRootDirectory->setExpectedKind(PathChooser::Directory);
+    d->sysRootDirectory->setPromptDialogTitle(tr("Select SysRoot Directory"));
+    d->sysRootDirectory->setToolTip(tr(
+        "This option can be used to override the kit's SysRoot setting"));
+
     auto coreLayout = new QHBoxLayout;
     coreLayout->addWidget(d->localCoreFileName);
     coreLayout->addWidget(d->remoteCoreFileName);
@@ -303,6 +281,7 @@ AttachCoreDialog::AttachCoreDialog(QWidget *parent)
     formLayout->addRow(tr("Core file:"), coreLayout);
     formLayout->addRow(tr("&Executable or symbol file:"), d->symbolFileName);
     formLayout->addRow(tr("Override &start script:"), d->overrideStartScriptFileName);
+    formLayout->addRow(tr("Override S&ysRoot:"), d->sysRootDirectory);
 
     auto line = new QFrame(this);
     line->setFrameShape(QFrame::HLine);
@@ -365,15 +344,16 @@ bool AttachCoreDialog::useLocalCoreFile() const
 
 void AttachCoreDialog::coreFileChanged(const QString &core)
 {
-    if (!HostOsInfo::isWindowsHost() && QFile::exists(core)) {
+    const FilePath coreFile = FilePath::fromUserInput(core);
+    if (coreFile.osType() != OsType::OsTypeWindows && coreFile.exists()) {
         Kit *k = d->kitChooser->currentKit();
         QTC_ASSERT(k, return);
         Runnable debugger = DebuggerKitAspect::runnable(k);
-        CoreInfo cinfo = CoreInfo::readExecutableNameFromCore(debugger, core);
+        CoreInfo cinfo = CoreInfo::readExecutableNameFromCore(debugger, coreFile);
         if (!cinfo.foundExecutableName.isEmpty())
-            d->symbolFileName->setFileName(FilePath::fromString(cinfo.foundExecutableName));
+            d->symbolFileName->setFilePath(cinfo.foundExecutableName);
         else if (!d->symbolFileName->isValid() && !cinfo.rawStringFromCore.isEmpty())
-            d->symbolFileName->setFileName(FilePath::fromString(cinfo.rawStringFromCore));
+            d->symbolFileName->setFilePath(FilePath::fromString(cinfo.rawStringFromCore));
     }
     changed();
 }
@@ -406,39 +386,39 @@ void AttachCoreDialog::selectRemoteCoreFile()
     dlg.attachToDevice(d->kitChooser->currentKit());
     if (dlg.exec() == QDialog::Rejected)
         return;
-    d->localCoreFileName->setPath(dlg.localFile());
-    d->remoteCoreFileName->setText(dlg.remoteFile());
+    d->localCoreFileName->setFilePath(dlg.localFile());
+    d->remoteCoreFileName->setText(dlg.remoteFile().toUserOutput());
     changed();
 }
 
-QString AttachCoreDialog::localCoreFile() const
+FilePath AttachCoreDialog::localCoreFile() const
 {
-    return d->localCoreFileName->path();
+    return d->localCoreFileName->filePath();
 }
 
 FilePath AttachCoreDialog::symbolFile() const
 {
-    return d->symbolFileName->fileName();
+    return d->symbolFileName->filePath();
 }
 
-void AttachCoreDialog::setSymbolFile(const QString &symbolFileName)
+void AttachCoreDialog::setSymbolFile(const FilePath &symbolFilePath)
 {
-    d->symbolFileName->setPath(symbolFileName);
+    d->symbolFileName->setFilePath(symbolFilePath);
 }
 
-void AttachCoreDialog::setLocalCoreFile(const QString &fileName)
+void AttachCoreDialog::setLocalCoreFile(const FilePath &coreFilePath)
 {
-    d->localCoreFileName->setPath(fileName);
+    d->localCoreFileName->setFilePath(coreFilePath);
 }
 
-void AttachCoreDialog::setRemoteCoreFile(const QString &fileName)
+void AttachCoreDialog::setRemoteCoreFile(const FilePath &coreFilePath)
 {
-    d->remoteCoreFileName->setText(fileName);
+    d->remoteCoreFileName->setText(coreFilePath.toUserOutput());
 }
 
-QString AttachCoreDialog::remoteCoreFile() const
+FilePath AttachCoreDialog::remoteCoreFile() const
 {
-    return d->remoteCoreFileName->text();
+    return FilePath::fromUserInput(d->remoteCoreFileName->text());
 }
 
 void AttachCoreDialog::setKitId(Id id)
@@ -461,14 +441,24 @@ Kit *AttachCoreDialog::kit() const
     return d->kitChooser->currentKit();
 }
 
-QString AttachCoreDialog::overrideStartScript() const
+FilePath AttachCoreDialog::overrideStartScript() const
 {
-    return d->overrideStartScriptFileName->path();
+    return d->overrideStartScriptFileName->filePath();
 }
 
-void AttachCoreDialog::setOverrideStartScript(const QString &scriptName)
+void AttachCoreDialog::setOverrideStartScript(const FilePath &scriptName)
 {
-    d->overrideStartScriptFileName->setPath(scriptName);
+    d->overrideStartScriptFileName->setFilePath(scriptName);
+}
+
+FilePath AttachCoreDialog::sysRoot() const
+{
+    return d->sysRootDirectory->filePath();
+}
+
+void AttachCoreDialog::setSysRoot(const FilePath &sysRoot)
+{
+    d->sysRootDirectory->setFilePath(sysRoot);
 }
 
 } // namespace Internal
