@@ -32,7 +32,6 @@
 
 #include "../gitplugin.h"
 #include "../gitclient.h"
-#include "../gitversioncontrol.h"
 #include "../gitconstants.h"
 #include <vcsbase/vcsbaseconstants.h>
 #include <vcsbase/vcsbaseeditor.h>
@@ -48,15 +47,16 @@
 #include <coreplugin/actionmanager/command.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/locator/commandlocator.h>
+#include <coreplugin/messagebox.h>
 
 #include <vcsbase/vcsoutputwindow.h>
 
-#include <utils/synchronousprocess.h>
-#include <coreplugin/messagebox.h>
+#include <utils/environment.h>
+#include <utils/qtcprocess.h>
 
 #include <QDebug>
 #include <QProcess>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QAction>
 #include <QFileDialog>
 #include <QMessageBox>
@@ -65,6 +65,8 @@
 #include <QFutureWatcher>
 
 using namespace Core;
+using namespace Utils;
+
 using namespace Git::Internal;
 
 enum { debug = 0 };
@@ -91,10 +93,10 @@ class FetchContext : public QObject
      Q_OBJECT
 public:
     FetchContext(const QSharedPointer<GerritChange> &change,
-                 const QString &repository, const Utils::FileName &git,
+                 const FilePath &repository, const FilePath &git,
                  const GerritServer &server,
                  FetchMode fm, QObject *parent = nullptr);
-    ~FetchContext();
+    ~FetchContext() override;
     void start();
 
 private:
@@ -105,8 +107,7 @@ private:
         ErrorState
     };
 
-    void processError(QProcess::ProcessError);
-    void processFinished(int exitCode, QProcess::ExitStatus);
+    void processDone();
     void processReadyReadStandardError();
     void processReadyReadStandardOutput();
 
@@ -117,18 +118,18 @@ private:
     void terminate();
 
     const QSharedPointer<GerritChange> m_change;
-    const QString m_repository;
+    const FilePath m_repository;
     const FetchMode m_fetchMode;
-    const Utils::FileName m_git;
+    const Utils::FilePath m_git;
     const GerritServer m_server;
     State m_state;
-    QProcess m_process;
+    QtcProcess m_process;
     QFutureInterface<void> m_progress;
     QFutureWatcher<void> m_watcher;
 };
 
 FetchContext::FetchContext(const QSharedPointer<GerritChange> &change,
-                           const QString &repository, const Utils::FileName &git,
+                           const FilePath &repository, const FilePath &git,
                            const GerritServer &server,
                            FetchMode fm, QObject *parent)
     : QObject(parent)
@@ -139,18 +140,15 @@ FetchContext::FetchContext(const QSharedPointer<GerritChange> &change,
     , m_server(server)
     , m_state(FetchState)
 {
-    connect(&m_process, &QProcess::errorOccurred, this, &FetchContext::processError);
-    connect(&m_process, static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished),
-            this, &FetchContext::processFinished);
-    connect(&m_process, &QProcess::readyReadStandardError,
+    connect(&m_process, &QtcProcess::done, this, &FetchContext::processDone);
+    connect(&m_process, &QtcProcess::readyReadStandardError,
             this, &FetchContext::processReadyReadStandardError);
-    connect(&m_process, &QProcess::readyReadStandardOutput,
+    connect(&m_process, &QtcProcess::readyReadStandardOutput,
             this, &FetchContext::processReadyReadStandardOutput);
     connect(&m_watcher, &QFutureWatcher<void>::canceled, this, &FetchContext::terminate);
     m_watcher.setFuture(m_progress.future());
     m_process.setWorkingDirectory(repository);
-    m_process.setProcessEnvironment(GitPlugin::client()->processEnvironment());
-    m_process.closeWriteChannel();
+    m_process.setEnvironment(GitClient::instance()->processEnvironment());
 }
 
 FetchContext::~FetchContext()
@@ -170,34 +168,31 @@ void FetchContext::start()
     m_progress.reportStarted();
     // Order: initialize future before starting the process in case error handling is invoked.
     const QStringList args = m_change->gitFetchArguments(m_server);
-    VcsBase::VcsOutputWindow::appendCommand(m_repository, m_git, args);
-    m_process.start(m_git.toString(), args);
-    m_process.closeWriteChannel();
+    VcsBase::VcsOutputWindow::appendCommand(m_repository, {m_git, args});
+    m_process.setCommand({m_git, args});
+    m_process.start();
 }
 
-void FetchContext::processFinished(int exitCode, QProcess::ExitStatus es)
+void FetchContext::processDone()
 {
-    if (es != QProcess::NormalExit) {
-        handleError(tr("%1 crashed.").arg(m_git.toUserOutput()));
+    if (m_process.result() != ProcessResult::FinishedWithSuccess) {
+        handleError(m_process.exitMessage());
         return;
     }
-    if (exitCode) {
-        handleError(tr("%1 returned %2.").arg(m_git.toUserOutput()).arg(exitCode));
+    if (m_state != FetchState)
         return;
-    }
-    if (m_state == FetchState) {
-        m_progress.setProgressValue(m_progress.progressValue() + 1);
-        if (m_fetchMode == FetchDisplay)
-            show();
-        else if (m_fetchMode == FetchCherryPick)
-            cherryPick();
-        else if (m_fetchMode == FetchCheckout)
-            checkout();
 
-        m_progress.reportFinished();
-        m_state = DoneState;
-        deleteLater();
-    }
+    m_progress.setProgressValue(m_progress.progressValue() + 1);
+    if (m_fetchMode == FetchDisplay)
+        show();
+    else if (m_fetchMode == FetchCherryPick)
+        cherryPick();
+    else if (m_fetchMode == FetchCheckout)
+        checkout();
+
+    m_progress.reportFinished();
+    m_state = DoneState;
+    deleteLater();
 }
 
 void FetchContext::processReadyReadStandardError()
@@ -226,22 +221,11 @@ void FetchContext::handleError(const QString &e)
     deleteLater();
 }
 
-void FetchContext::processError(QProcess::ProcessError e)
-{
-    if (m_progress.isCanceled())
-        return;
-    const QString msg = tr("Error running %1: %2").arg(m_git.toUserOutput(), m_process.errorString());
-    if (e == QProcess::FailedToStart)
-        handleError(msg);
-    else
-        VcsBase::VcsOutputWindow::appendError(msg);
-}
-
 void FetchContext::show()
 {
     const QString title = QString::number(m_change->number) + '/'
             + QString::number(m_change->currentPatchSet.patchSetNumber);
-    GitPlugin::client()->show(m_repository, "FETCH_HEAD", title);
+    GitClient::instance()->show(m_repository.toString(), "FETCH_HEAD", title);
 }
 
 void FetchContext::cherryPick()
@@ -249,20 +233,20 @@ void FetchContext::cherryPick()
     // Point user to errors.
     VcsBase::VcsOutputWindow::instance()->popup(IOutputPane::ModeSwitch
                                                   | IOutputPane::WithFocus);
-    GitPlugin::client()->synchronousCherryPick(m_repository, "FETCH_HEAD");
+    GitClient::instance()->synchronousCherryPick(m_repository, "FETCH_HEAD");
 }
 
 void FetchContext::checkout()
 {
-    GitPlugin::client()->stashAndCheckout(m_repository, "FETCH_HEAD");
+    GitClient::instance()->checkout(m_repository, "FETCH_HEAD");
 }
 
 void FetchContext::terminate()
 {
-    Utils::SynchronousProcess::stopProcess(m_process);
+    m_process.stop();
+    m_process.waitForFinished();
 }
 
-
 GerritPlugin::GerritPlugin(QObject *parent)
     : QObject(parent)
     , m_parameters(new GerritParameters)
@@ -270,11 +254,9 @@ GerritPlugin::GerritPlugin(QObject *parent)
 {
 }
 
-GerritPlugin::~GerritPlugin()
-{
-}
+GerritPlugin::~GerritPlugin() = default;
 
-bool GerritPlugin::initialize(ActionContainer *ac)
+void GerritPlugin::initialize(ActionContainer *ac)
 {
     m_parameters->fromSettings(ICore::settings());
 
@@ -298,7 +280,6 @@ bool GerritPlugin::initialize(ActionContainer *ac)
         if (m_dialog)
             m_dialog->scheduleUpdateRemotes();
     });
-    return true;
 }
 
 void GerritPlugin::updateActions(const VcsBase::VcsBasePluginState &state)
@@ -316,14 +297,14 @@ void GerritPlugin::addToLocator(CommandLocator *locator)
     locator->appendCommand(m_pushToGerritCommand);
 }
 
-void GerritPlugin::push(const QString &topLevel)
+void GerritPlugin::push(const FilePath &topLevel)
 {
     // QScopedPointer is required to delete the dialog when leaving the function
-    GerritPushDialog dialog(topLevel, m_reviewers, m_parameters, ICore::mainWindow());
+    GerritPushDialog dialog(topLevel, m_reviewers, m_parameters, ICore::dialogParent());
 
     const QString initErrorMessage = dialog.initErrorMessage();
     if (!initErrorMessage.isEmpty()) {
-        QMessageBox::warning(ICore::mainWindow(), tr("Initialization Failed"), initErrorMessage);
+        QMessageBox::warning(ICore::dialogParent(), tr("Initialization Failed"), initErrorMessage);
         return;
     }
 
@@ -332,12 +313,12 @@ void GerritPlugin::push(const QString &topLevel)
 
     dialog.storeTopic();
     m_reviewers = dialog.reviewers();
-    GitPlugin::client()->push(topLevel, {dialog.selectedRemoteName(), dialog.pushTarget()});
+    GitClient::instance()->push(topLevel, {dialog.selectedRemoteName(), dialog.pushTarget()});
 }
 
-static QString currentRepository()
+static FilePath currentRepository()
 {
-    return GitPlugin::instance()->currentState().topLevel();
+    return GitPlugin::currentState().topLevel();
 }
 
 // Open or raise the Gerrit dialog window.
@@ -350,8 +331,9 @@ void GerritPlugin::openView()
             if (!ICore::showOptionsDialog("Gerrit"))
                 return;
         }
-        GerritDialog *gd = new GerritDialog(m_parameters, m_server, currentRepository(), ICore::mainWindow());
+        GerritDialog *gd = new GerritDialog(m_parameters, m_server, currentRepository(), ICore::dialogParent());
         gd->setModal(false);
+        ICore::registerWindow(gd, Context("Git.Gerrit"));
         connect(gd, &GerritDialog::fetchDisplay, this,
                 [this](const QSharedPointer<GerritChange> &change) { fetch(change, FetchDisplay); });
         connect(gd, &GerritDialog::fetchCherryPick, this,
@@ -377,34 +359,34 @@ void GerritPlugin::push()
     push(currentRepository());
 }
 
-Utils::FileName GerritPlugin::gitBinDirectory()
+Utils::FilePath GerritPlugin::gitBinDirectory()
 {
-    return GitPlugin::client()->gitBinDirectory();
+    return GitClient::instance()->gitBinDirectory();
 }
 
 // Find the branch of a repository.
-QString GerritPlugin::branch(const QString &repository)
+QString GerritPlugin::branch(const FilePath &repository)
 {
-    return GitPlugin::client()->synchronousCurrentLocalBranch(repository);
+    return GitClient::instance()->synchronousCurrentLocalBranch(repository);
 }
 
 void GerritPlugin::fetch(const QSharedPointer<GerritChange> &change, int mode)
 {
     // Locate git.
-    const Utils::FileName git = GitPlugin::client()->vcsBinary();
+    const Utils::FilePath git = GitClient::instance()->vcsBinary();
     if (git.isEmpty()) {
         VcsBase::VcsOutputWindow::appendError(tr("Git is not available."));
         return;
     }
 
-    QString repository;
+    FilePath repository;
     bool verifiedRepository = false;
-    if (!m_dialog.isNull() && !m_parameters.isNull() && QFile::exists(m_dialog->repositoryPath()))
+    if (!m_dialog.isNull() && !m_parameters.isNull() && m_dialog->repositoryPath().exists())
         repository = m_dialog->repositoryPath();
 
     if (!repository.isEmpty()) {
         // Check if remote from a working dir is the same as remote from patch
-        QMap<QString, QString> remotesList = GitPlugin::client()->synchronousRemotesList(repository);
+        QMap<QString, QString> remotesList = GitClient::instance()->synchronousRemotesList(repository);
         if (!remotesList.isEmpty()) {
             const QStringList remotes = remotesList.values();
             for (QString remote : remotes) {
@@ -417,14 +399,14 @@ void GerritPlugin::fetch(const QSharedPointer<GerritChange> &change, int mode)
             }
 
             if (!verifiedRepository) {
-                const SubmoduleDataMap submodules = GitPlugin::client()->submoduleList(repository);
+                const SubmoduleDataMap submodules = GitClient::instance()->submoduleList(repository);
                 for (const SubmoduleData &submoduleData : submodules) {
                     QString remote = submoduleData.url;
                     if (remote.endsWith(".git"))
                         remote.chop(4);
                     if (remote.contains(m_server->host) && remote.endsWith(change->project)
-                            && QFile::exists(repository + '/' + submoduleData.dir)) {
-                        repository = QDir::cleanPath(repository + '/' + submoduleData.dir);
+                            && repository.pathAppended(submoduleData.dir).exists()) {
+                        repository = repository.pathAppended(submoduleData.dir).cleanPath();
                         verifiedRepository = true;
                         break;
                     }
@@ -433,12 +415,12 @@ void GerritPlugin::fetch(const QSharedPointer<GerritChange> &change, int mode)
 
             if (!verifiedRepository) {
                 QMessageBox::StandardButton answer = QMessageBox::question(
-                            ICore::mainWindow(), tr("Remote Not Verified"),
+                            ICore::dialogParent(), tr("Remote Not Verified"),
                             tr("Change host %1\nand project %2\n\nwere not verified among remotes"
                                " in %3. Select different folder?")
                             .arg(m_server->host,
                                  change->project,
-                                 QDir::toNativeSeparators(repository)),
+                                 repository.toUserOutput()),
                             QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel,
                             QMessageBox::Yes);
                 switch (answer) {
@@ -458,17 +440,15 @@ void GerritPlugin::fetch(const QSharedPointer<GerritChange> &change, int mode)
         // Ask the user for a repository to retrieve the change.
         const QString title =
                 tr("Enter Local Repository for \"%1\" (%2)").arg(change->project, change->branch);
-        const QString suggestedRespository =
-                findLocalRepository(change->project, change->branch);
-        repository = QFileDialog::getExistingDirectory(m_dialog.data(),
-                                                       title, suggestedRespository);
+        const FilePath suggestedRespository =
+                FilePath::fromString(findLocalRepository(change->project, change->branch));
+        repository = FileUtils::getExistingDirectory(m_dialog.data(), title, suggestedRespository);
     }
 
     if (repository.isEmpty())
         return;
 
-    FetchContext *fc = new FetchContext(change, repository, git,
-                                        *m_server, FetchMode(mode), this);
+    auto fc = new FetchContext(change, repository, git, *m_server, FetchMode(mode), this);
     connect(fc, &QObject::destroyed, this, &GerritPlugin::fetchFinished);
     emit fetchStarted(change);
     fc->start();
@@ -477,33 +457,33 @@ void GerritPlugin::fetch(const QSharedPointer<GerritChange> &change, int mode)
 // Try to find a matching repository for a project by asking the VcsManager.
 QString GerritPlugin::findLocalRepository(QString project, const QString &branch) const
 {
-    const QStringList gitRepositories = VcsManager::repositories(GitPlugin::instance()->gitVersionControl());
+    const QStringList gitRepositories = VcsManager::repositories(GitPlugin::versionControl());
     // Determine key (file name) to look for (qt/qtbase->'qtbase').
     const int slashPos = project.lastIndexOf('/');
     if (slashPos != -1)
         project.remove(0, slashPos + 1);
     // When looking at branch 1.7, try to check folders
     // "qtbase_17", 'qtbase1.7' with a semi-smart regular expression.
-    QScopedPointer<QRegExp> branchRegexp;
+    QScopedPointer<QRegularExpression> branchRegexp;
     if (!branch.isEmpty() && branch != "master") {
         QString branchPattern = branch;
         branchPattern.replace('.', "[\\.-_]?");
         const QString pattern = '^' + project
                                 + "[-_]?"
                                 + branchPattern + '$';
-        branchRegexp.reset(new QRegExp(pattern));
+        branchRegexp.reset(new QRegularExpression(pattern));
         if (!branchRegexp->isValid())
             branchRegexp.reset(); // Oops.
     }
     for (const QString &repository : gitRepositories) {
-        const QString fileName = Utils::FileName::fromString(repository).fileName();
-        if ((!branchRegexp.isNull() && branchRegexp->exactMatch(fileName))
+        const QString fileName = Utils::FilePath::fromString(repository).fileName();
+        if ((!branchRegexp.isNull() && branchRegexp->match(fileName).hasMatch())
             || fileName == project) {
             // Perform a check on the branch.
             if (branch.isEmpty())  {
                 return repository;
             } else {
-                const QString repositoryBranch = GerritPlugin::branch(repository);
+                const QString repositoryBranch = GerritPlugin::branch(FilePath::fromString(repository));
                 if (repositoryBranch.isEmpty() || repositoryBranch == branch)
                     return repository;
             } // !branch.isEmpty()
