@@ -34,19 +34,24 @@
 
 #include <QDateTime>
 #include <QDebug>
+#include <QDesktopServices>
 #include <QDir>
 #include <QFileInfo>
-#include <QFutureWatcher>
 #include <QStringList>
 #include <QUrl>
 
 #include <QHelpEngineCore>
 
 #include <QMutexLocker>
+
+#ifndef HELP_NEW_FILTER_ENGINE
 #include <QSqlDatabase>
 #include <QSqlDriver>
 #include <QSqlError>
 #include <QSqlQuery>
+#else
+#include <QtHelp/QHelpLink>
+#endif
 
 using namespace Core;
 
@@ -84,6 +89,8 @@ struct HelpManagerPrivate
 static HelpManager *m_instance = nullptr;
 static HelpManagerPrivate *d = nullptr;
 
+#ifndef HELP_NEW_FILTER_ENGINE
+
 // -- DbCleaner
 
 struct DbCleaner
@@ -93,6 +100,8 @@ struct DbCleaner
     QString name;
 };
 
+#endif
+
 // -- HelpManager
 
 HelpManager::HelpManager(QObject *parent) :
@@ -101,10 +110,12 @@ HelpManager::HelpManager(QObject *parent) :
     QTC_CHECK(!m_instance);
     m_instance = this;
     d = new HelpManagerPrivate;
+    QDesktopServices::setUrlHandler("qthelp", this, "showHelpUrl");
 }
 
 HelpManager::~HelpManager()
 {
+    QDesktopServices::unsetUrlHandler("qthelp");
     delete d;
     m_instance = nullptr;
 }
@@ -117,8 +128,7 @@ HelpManager *HelpManager::instance()
 
 QString HelpManager::collectionFilePath()
 {
-    return QDir::cleanPath(ICore::userResourcePath()
-        + QLatin1String("/helpcollection.qhc"));
+    return ICore::userResourcePath("helpcollection.qhc").toString();
 }
 
 void HelpManager::registerDocumentation(const QStringList &files)
@@ -136,8 +146,20 @@ void HelpManager::registerDocumentation(const QStringList &files)
             emit Core::HelpManager::Signals::instance()->documentationChanged();
         }
     });
-    ProgressManager::addTask(future, tr("Update Documentation"),
-                             kUpdateDocumentationTask);
+    ProgressManager::addTask(future, tr("Update Documentation"), kUpdateDocumentationTask);
+}
+
+void HelpManager::unregisterDocumentation(const QStringList &fileNames)
+{
+    if (fileNames.isEmpty())
+        return;
+    const auto getNamespaces = [](const QStringList &fileNames) {
+        QMutexLocker locker(&d->m_helpengineMutex);
+        return Utils::transform(fileNames, [](const QString &filePath) {
+            return QHelpEngineCore::namespaceName(filePath);
+        });
+    };
+    unregisterNamespaces(getNamespaces(fileNames));
 }
 
 void HelpManager::registerDocumentationNow(QFutureInterface<bool> &futureInterface,
@@ -149,6 +171,9 @@ void HelpManager::registerDocumentationNow(QFutureInterface<bool> &futureInterfa
     futureInterface.setProgressValue(0);
 
     QHelpEngineCore helpEngine(collectionFilePath());
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    helpEngine.setReadOnly(false);
+#endif
     helpEngine.setupData();
     bool docsChanged = false;
     QStringList nameSpaces = helpEngine.registeredDocumentations();
@@ -156,7 +181,7 @@ void HelpManager::registerDocumentationNow(QFutureInterface<bool> &futureInterfa
         if (futureInterface.isCanceled())
             break;
         futureInterface.setProgressValue(futureInterface.progressValue() + 1);
-        const QString &nameSpace = helpEngine.namespaceName(file);
+        const QString &nameSpace = QHelpEngineCore::namespaceName(file);
         if (nameSpace.isEmpty())
             continue;
         if (!nameSpaces.contains(nameSpace)) {
@@ -172,7 +197,7 @@ void HelpManager::registerDocumentationNow(QFutureInterface<bool> &futureInterfa
     futureInterface.reportResult(docsChanged);
 }
 
-void HelpManager::unregisterDocumentation(const QStringList &nameSpaces)
+void HelpManager::unregisterNamespaces(const QStringList &nameSpaces)
 {
     if (d->m_needsSetup) {
         for (const QString &name : nameSpaces)
@@ -184,6 +209,8 @@ void HelpManager::unregisterDocumentation(const QStringList &nameSpaces)
     bool docsChanged = false;
     for (const QString &nameSpace : nameSpaces) {
         const QString filePath = d->m_helpEngine->documentationFileName(nameSpace);
+        if (filePath.isEmpty()) // wasn't registered anyhow, ignore
+            continue;
         if (d->m_helpEngine->unregisterDocumentation(nameSpace)) {
             docsChanged = true;
             d->m_userRegisteredFiles.remove(filePath);
@@ -211,20 +238,36 @@ QSet<QString> HelpManager::userDocumentationPaths()
 }
 
 // This should go into Qt 4.8 once we start using it for Qt Creator
-QMap<QString, QUrl> HelpManager::linksForKeyword(const QString &key)
+QMultiMap<QString, QUrl> HelpManager::linksForKeyword(const QString &key)
 {
     QTC_ASSERT(!d->m_needsSetup, return {});
     if (key.isEmpty())
         return {};
+#ifndef HELP_NEW_FILTER_ENGINE
     return d->m_helpEngine->linksForKeyword(key);
+#else
+    QMultiMap<QString, QUrl> links;
+    const QList<QHelpLink> docs = d->m_helpEngine->documentsForKeyword(key, QString());
+    for (const auto &doc : docs)
+        links.insert(doc.title, doc.url);
+    return links;
+#endif
 }
 
-QMap<QString, QUrl> HelpManager::linksForIdentifier(const QString &id)
+QMultiMap<QString, QUrl> HelpManager::linksForIdentifier(const QString &id)
 {
     QTC_ASSERT(!d->m_needsSetup, return {});
     if (id.isEmpty())
         return {};
+#ifndef HELP_NEW_FILTER_ENGINE
     return d->m_helpEngine->linksForIdentifier(id);
+#else
+    QMultiMap<QString, QUrl> links;
+    const QList<QHelpLink> docs = d->m_helpEngine->documentsForIdentifier(id, QString());
+    for (const auto &doc : docs)
+        links.insert(doc.title, doc.url);
+    return links;
+#endif
 }
 
 QUrl HelpManager::findFile(const QUrl &url)
@@ -253,7 +296,7 @@ QStringList HelpManager::registeredNamespaces()
 QString HelpManager::namespaceFromFile(const QString &file)
 {
     QTC_ASSERT(!d->m_needsSetup, return {});
-    return d->m_helpEngine->namespaceName(file);
+    return QHelpEngineCore::namespaceName(file);
 }
 
 QString HelpManager::fileFromNamespace(const QString &nameSpace)
@@ -278,6 +321,8 @@ QVariant HelpManager::customValue(const QString &key, const QVariant &value)
     return d->m_helpEngine->customValue(key, value);
 }
 
+#ifndef HELP_NEW_FILTER_ENGINE
+
 HelpManager::Filters HelpManager::filters()
 {
     QTC_ASSERT(!d->m_needsSetup, return {});
@@ -344,6 +389,8 @@ void HelpManager::addUserDefinedFilter(const QString &filter, const QStringList
         emit m_instance->collectionFileChanged();
 }
 
+#endif
+
 void HelpManager::aboutToShutdown()
 {
     if (d && d->m_registerFuture.isRunning()) {
@@ -364,6 +411,12 @@ void HelpManager::setupHelpManager()
 
     // create the help engine
     d->m_helpEngine = new QHelpEngineCore(collectionFilePath(), m_instance);
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    d->m_helpEngine->setReadOnly(false);
+#endif
+#ifdef HELP_NEW_FILTER_ENGINE
+    d->m_helpEngine->setUsesFilterEngine(true);
+#endif
     d->m_helpEngine->setupData();
 
     for (const QString &filePath : d->documentationFromInstaller())
@@ -372,12 +425,12 @@ void HelpManager::setupHelpManager()
     d->cleanUpDocumentation();
 
     if (!d->m_nameSpacesToUnregister.isEmpty()) {
-        m_instance->unregisterDocumentation(d->m_nameSpacesToUnregister.toList());
+        m_instance->unregisterNamespaces(Utils::toList(d->m_nameSpacesToUnregister));
         d->m_nameSpacesToUnregister.clear();
     }
 
     if (!d->m_filesToRegister.isEmpty()) {
-        m_instance->registerDocumentation(d->m_filesToRegister.toList());
+        m_instance->registerDocumentation(Utils::toList(d->m_filesToRegister));
         d->m_filesToRegister.clear();
     }
 
@@ -432,14 +485,14 @@ const QStringList HelpManagerPrivate::documentationFromInstaller()
 
 void HelpManagerPrivate::readSettings()
 {
-    m_userRegisteredFiles = ICore::settings()->value(QLatin1String(kUserDocumentationKey))
-            .toStringList().toSet();
+    m_userRegisteredFiles = Utils::toSet(ICore::settings()->value(QLatin1String(kUserDocumentationKey))
+            .toStringList());
 }
 
 void HelpManagerPrivate::writeSettings()
 {
-    const QStringList list = m_userRegisteredFiles.toList();
-    ICore::settings()->setValue(QLatin1String(kUserDocumentationKey), list);
+    const QStringList list = Utils::toList(m_userRegisteredFiles);
+    ICore::settings()->setValueWithDefault(kUserDocumentationKey, list);
 }
 
 } // Internal
