@@ -114,7 +114,7 @@ ModelsManager::ModelsManager(QObject *parent)
             this, &ModelsManager::onOpenDefaultModel, Qt::QueuedConnection);
 #endif
 
-    Core::Context projecTreeContext(ProjectExplorer::Constants::C_PROJECT_TREE);
+    Core::Context projectTreeContext(ProjectExplorer::Constants::C_PROJECT_TREE);
     Core::ActionContainer *folderContainer = Core::ActionManager::actionContainer(
                 ProjectExplorer::Constants::M_FOLDERCONTEXT);
     folderContainer->insertGroup(ProjectExplorer::Constants::G_FOLDER_FILES,
@@ -122,7 +122,7 @@ ModelsManager::ModelsManager(QObject *parent)
     d->openDiagramContextMenuItem = new QAction(tr("Open Diagram"), this);
     Core::Command *cmd = Core::ActionManager::registerAction(
                 d->openDiagramContextMenuItem, Constants::ACTION_EXPLORER_OPEN_DIAGRAM,
-                projecTreeContext);
+                projectTreeContext);
     folderContainer->addAction(cmd, Constants::EXPLORER_GROUP_MODELING);
     connect(d->openDiagramContextMenuItem, &QAction::triggered,
             this, &ModelsManager::onOpenDiagramFromProjectExplorer);
@@ -140,10 +140,9 @@ ModelsManager::~ModelsManager()
 ExtDocumentController *ModelsManager::createModel(ModelDocument *modelDocument)
 {
     auto documentController = new ExtDocumentController(this);
-    QDir dir;
-    dir.setPath(Core::ICore::resourcePath() + QLatin1String("/modeleditor"));
     // TODO error output on reading definition files
-    documentController->configController()->readStereotypeDefinitions(dir.path());
+    documentController->configController()->readStereotypeDefinitions(
+        Core::ICore::resourcePath("modeleditor").toString());
 
     d->managedModels.append(ManagedModel(documentController, modelDocument));
     return documentController;
@@ -168,7 +167,7 @@ void ModelsManager::releaseModel(ExtDocumentController *documentController)
 
 void ModelsManager::openDiagram(const qmt::Uid &modelUid, const qmt::Uid &diagramUid)
 {
-    foreach (const ManagedModel &managedModel, d->managedModels) {
+    for (const ManagedModel &managedModel : qAsConst(d->managedModels)) {
         if (managedModel.m_documentController->projectController()->project()->uid() == modelUid) {
             qmt::MDiagram *diagram = managedModel.m_documentController->modelController()->findObject<qmt::MDiagram>(diagramUid);
             QMT_ASSERT(diagram, continue);
@@ -227,14 +226,11 @@ void ModelsManager::setDiagramClipboard(ExtDocumentController *documentControlle
     emit diagramClipboardChanged(isDiagramClipboardEmpty());
 }
 
-void ModelsManager::onAboutToShowContextMenu(ProjectExplorer::Project *project,
-                                             ProjectExplorer::Node *node)
+void ModelsManager::onAboutToShowContextMenu(ProjectExplorer::Node *node)
 {
-    Q_UNUSED(project);
-
     bool canOpenDiagram = false;
 
-    foreach (const ManagedModel &managedModel, d->managedModels) {
+    for (const ManagedModel &managedModel : qAsConst(d->managedModels)) {
         if (managedModel.m_documentController->pxNodeController()->hasDiagramForExplorerNode(node)) {
             canOpenDiagram = true;
             break;
@@ -250,9 +246,9 @@ void ModelsManager::onAboutToShowContextMenu(ProjectExplorer::Project *project,
 
 void ModelsManager::onOpenDiagramFromProjectExplorer()
 {
-    if (ProjectExplorer::ProjectTree::findCurrentNode() == d->contextMenuOwnerNode) {
+    if (ProjectExplorer::ProjectTree::currentNode() == d->contextMenuOwnerNode) {
         qmt::MDiagram *diagram = nullptr;
-        foreach (const ManagedModel &managedModel, d->managedModels) {
+        for (const ManagedModel &managedModel : qAsConst(d->managedModels)) {
             if ((diagram = managedModel.m_documentController->pxNodeController()->findDiagramForExplorerNode(d->contextMenuOwnerNode))) {
                 openDiagram(managedModel.m_documentController, diagram);
                 break;
@@ -263,7 +259,7 @@ void ModelsManager::onOpenDiagramFromProjectExplorer()
 
 void ModelsManager::onOpenDefaultModel(const qmt::Uid &modelUid)
 {
-    QString modelFile = d->modelIndexer->findModel(modelUid);
+    const auto modelFile = Utils::FilePath::fromString(d->modelIndexer->findModel(modelUid));
     if (!modelFile.isEmpty())
         Core::EditorManager::openEditor(modelFile);
 }
@@ -271,7 +267,7 @@ void ModelsManager::onOpenDefaultModel(const qmt::Uid &modelUid)
 void ModelsManager::openDiagram(ExtDocumentController *documentController,
                                  qmt::MDiagram *diagram)
 {
-    foreach (const ManagedModel &managedModel, d->managedModels) {
+    for (const ManagedModel &managedModel : qAsConst(d->managedModels)) {
         if (managedModel.m_documentController == documentController) {
             Core::IEditor *editor = Core::EditorManager::activateEditorForDocument(managedModel.m_modelDocument);
             if (auto modelEditor = qobject_cast<ModelEditor *>(editor)) {
