@@ -32,22 +32,36 @@
 #include "panelswidget.h"
 #include "project.h"
 #include "projectexplorer.h"
+#include "projectexplorerconstants.h"
 #include "projectimporter.h"
 #include "projectpanelfactory.h"
+#include "projectsettingswidget.h"
 #include "session.h"
 #include "target.h"
 #include "targetsettingspanel.h"
 
+#include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/actionmanager/commandbutton.h>
+#include <coreplugin/coreconstants.h>
 #include <coreplugin/coreicons.h>
+#include <coreplugin/find/optionspopup.h>
+#include <coreplugin/findplaceholder.h>
+#include <coreplugin/icontext.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
+#include <coreplugin/outputwindow.h>
 
 #include <utils/algorithm.h>
 #include <utils/basetreeview.h>
+#include <utils/hostosinfo.h>
 #include <utils/navigationtreeview.h>
 #include <utils/qtcassert.h>
 #include <utils/styledbar.h>
 #include <utils/treemodel.h>
+#include <utils/utilsicons.h>
+
+#include <texteditor/fontsettings.h>
+#include <texteditor/texteditorsettings.h>
 
 #include <QApplication>
 #include <QComboBox>
@@ -70,6 +84,145 @@ namespace Internal {
 
 class MiscSettingsGroupItem;
 
+const char kBuildSystemOutputContext[] = "ProjectsMode.BuildSystemOutput";
+const char kRegExpActionId[] = "OutputFilter.RegularExpressions.BuildSystemOutput";
+const char kCaseSensitiveActionId[] = "OutputFilter.CaseSensitive.BuildSystemOutput";
+const char kInvertActionId[] = "OutputFilter.Invert.BuildSystemOutput";
+
+class BuildSystemOutputWindow : public OutputWindow
+{
+public:
+    BuildSystemOutputWindow();
+
+    QWidget *toolBar();
+
+private:
+    void updateFilter();
+
+    QPointer<QWidget> m_toolBar;
+    QPointer<FancyLineEdit> m_filterOutputLineEdit;
+    QAction m_clear;
+    QAction m_filterActionRegexp;
+    QAction m_filterActionCaseSensitive;
+    QAction m_invertFilterAction;
+    QAction m_zoomIn;
+    QAction m_zoomOut;
+};
+
+BuildSystemOutputWindow::BuildSystemOutputWindow()
+    : OutputWindow(Context(kBuildSystemOutputContext), "ProjectsMode.BuildSystemOutput.Zoom")
+{
+    setReadOnly(true);
+
+    Command *clearCommand = ActionManager::command(Core::Constants::OUTPUTPANE_CLEAR);
+    m_clear.setIcon(Utils::Icons::CLEAN_TOOLBAR.icon());
+    m_clear.setText(clearCommand->action()->text());
+    ActionManager::registerAction(&m_clear,
+                                  Core::Constants::OUTPUTPANE_CLEAR,
+                                  Context(kBuildSystemOutputContext));
+    connect(&m_clear, &QAction::triggered, this, &OutputWindow::clear);
+
+    m_filterActionRegexp.setCheckable(true);
+    m_filterActionRegexp.setText(ProjectWindow::tr("Use Regular Expressions"));
+    connect(&m_filterActionRegexp, &QAction::toggled, this, &BuildSystemOutputWindow::updateFilter);
+    Core::ActionManager::registerAction(&m_filterActionRegexp,
+                                        kRegExpActionId,
+                                        Context(Constants::C_PROJECTEXPLORER));
+
+    m_filterActionCaseSensitive.setCheckable(true);
+    m_filterActionCaseSensitive.setText(ProjectWindow::tr("Case Sensitive"));
+    connect(&m_filterActionCaseSensitive,
+            &QAction::toggled,
+            this,
+            &BuildSystemOutputWindow::updateFilter);
+    Core::ActionManager::registerAction(&m_filterActionCaseSensitive,
+                                        kCaseSensitiveActionId,
+                                        Context(Constants::C_PROJECTEXPLORER));
+
+    m_invertFilterAction.setCheckable(true);
+    m_invertFilterAction.setText(ProjectWindow::tr("Show Non-matching Lines"));
+    connect(&m_invertFilterAction, &QAction::toggled, this, &BuildSystemOutputWindow::updateFilter);
+    Core::ActionManager::registerAction(&m_invertFilterAction,
+                                        kInvertActionId,
+                                        Context(Constants::C_PROJECTEXPLORER));
+
+    connect(TextEditor::TextEditorSettings::instance(),
+            &TextEditor::TextEditorSettings::fontSettingsChanged,
+            this,
+            [this] { setBaseFont(TextEditor::TextEditorSettings::fontSettings().font()); });
+    setBaseFont(TextEditor::TextEditorSettings::fontSettings().font());
+
+    m_zoomIn.setIcon(Utils::Icons::PLUS_TOOLBAR.icon());
+    connect(&m_zoomIn, &QAction::triggered, this, [this] { zoomIn(); });
+    ActionManager::registerAction(&m_zoomIn,
+                                  Core::Constants::ZOOM_IN,
+                                  Context(kBuildSystemOutputContext));
+
+    m_zoomOut.setIcon(Utils::Icons::MINUS.icon());
+    connect(&m_zoomOut, &QAction::triggered, this, [this] { zoomOut(); });
+    ActionManager::registerAction(&m_zoomOut,
+                                  Core::Constants::ZOOM_OUT,
+                                  Context(kBuildSystemOutputContext));
+}
+
+QWidget *BuildSystemOutputWindow::toolBar()
+{
+    if (!m_toolBar) {
+        m_toolBar = new StyledBar(this);
+        auto clearButton = new CommandButton(Core::Constants::OUTPUTPANE_CLEAR);
+        clearButton->setDefaultAction(&m_clear);
+        clearButton->setToolTipBase(m_clear.text());
+
+        m_filterOutputLineEdit = new FancyLineEdit;
+        m_filterOutputLineEdit->setButtonVisible(FancyLineEdit::Left, true);
+        m_filterOutputLineEdit->setButtonIcon(FancyLineEdit::Left, Utils::Icons::MAGNIFIER.icon());
+        m_filterOutputLineEdit->setFiltering(true);
+        m_filterOutputLineEdit->setHistoryCompleter("ProjectsMode.BuildSystemOutput.Filter");
+        connect(m_filterOutputLineEdit,
+                &FancyLineEdit::textChanged,
+                this,
+                &BuildSystemOutputWindow::updateFilter);
+        connect(m_filterOutputLineEdit,
+                &FancyLineEdit::returnPressed,
+                this,
+                &BuildSystemOutputWindow::updateFilter);
+        connect(m_filterOutputLineEdit, &FancyLineEdit::leftButtonClicked, this, [this] {
+            auto popup = new Core::OptionsPopup(m_filterOutputLineEdit,
+                                                {kRegExpActionId,
+                                                 kCaseSensitiveActionId,
+                                                 kInvertActionId});
+            popup->show();
+        });
+
+        auto zoomInButton = new CommandButton(Core::Constants::ZOOM_IN);
+        zoomInButton->setDefaultAction(&m_zoomIn);
+        auto zoomOutButton = new CommandButton(Core::Constants::ZOOM_OUT);
+        zoomOutButton->setDefaultAction(&m_zoomOut);
+
+        auto layout = new QHBoxLayout;
+        layout->setContentsMargins(0, 0, 0, 0);
+        layout->setSpacing(0);
+        m_toolBar->setLayout(layout);
+        layout->addWidget(clearButton);
+        layout->addWidget(m_filterOutputLineEdit);
+        layout->addWidget(zoomInButton);
+        layout->addWidget(zoomOutButton);
+        layout->addStretch();
+    }
+    return m_toolBar;
+}
+
+void BuildSystemOutputWindow::updateFilter()
+{
+    if (!m_filterOutputLineEdit)
+        return;
+    updateFilterProperties(m_filterOutputLineEdit->text(),
+                           m_filterActionCaseSensitive.isChecked() ? Qt::CaseSensitive
+                                                                    : Qt::CaseInsensitive,
+                           m_filterActionRegexp.isChecked(),
+                           m_invertFilterAction.isChecked());
+}
+
 // Standard third level for the generic case: i.e. all except for the Build/Run page
 class MiscSettingsPanelItem : public TreeItem // TypedTreeItem<TreeItem, MiscSettingsGroupItem>
 {
@@ -103,10 +256,8 @@ QVariant MiscSettingsPanelItem::data(int column, int role) const
 
     if (role == PanelWidgetRole) {
         if (!m_widget) {
-            QWidget *widget = m_factory->createWidget(m_project);
-            m_widget = new PanelsWidget(m_factory->displayName(),
-                                        QIcon(m_factory->icon()),
-                                        widget);
+            ProjectSettingsWidget *widget = m_factory->createWidget(m_project);
+            m_widget = new PanelsWidget(m_factory->displayName(), widget);
             m_widget->setFocusProxy(widget);
         }
 
@@ -148,7 +299,8 @@ public:
         : m_project(project)
     {
         QTC_ASSERT(m_project, return);
-        foreach (ProjectPanelFactory *factory, ProjectPanelFactory::factories())
+        const QList<ProjectPanelFactory *> factories = ProjectPanelFactory::factories();
+        for (ProjectPanelFactory *factory : factories)
             appendChild(new MiscSettingsPanelItem(factory, project));
     }
 
@@ -283,7 +435,7 @@ public:
         return activeItem ? activeItem->index() : QModelIndex();
     }
 
-    TreeItem *itemForProjectPanel(Core::Id panelId)
+    TreeItem *itemForProjectPanel(Utils::Id panelId)
     {
         return m_miscItem->findChildAtLevel(1, [panelId](const TreeItem *item){
             return static_cast<const MiscSettingsPanelItem *>(item)->factory()->id() == panelId;
@@ -337,11 +489,31 @@ public:
         setContextMenuPolicy(Qt::CustomContextMenu);
     }
 
+private:
     // remove branch indicators
     void drawBranches(QPainter *, const QRect &, const QModelIndex &) const final
     {
         return;
     }
+
+    bool userWantsContextMenu(const QMouseEvent *e) const
+    {
+        // On Windows, we get additional mouse events for the item view when right-clicking,
+        // causing unwanted kit activation (QTCREATORBUG-24156). Let's suppress these.
+        return HostOsInfo::isWindowsHost() && e->button() == Qt::RightButton;
+    }
+
+    void mousePressEvent(QMouseEvent *e) final
+    {
+        if (!userWantsContextMenu(e))
+            BaseTreeView::mousePressEvent(e);
+    }
+
+    void mouseReleaseEvent(QMouseEvent *e) final
+    {
+        if (!userWantsContextMenu(e))
+            BaseTreeView::mouseReleaseEvent(e);
+    }
 };
 
 class ComboBoxItem : public TreeItem
@@ -386,6 +558,14 @@ public:
         connect(m_projectSelection, QOverload<int>::of(&QComboBox::activated),
                 this, &ProjectWindowPrivate::projectSelected, Qt::QueuedConnection);
 
+        const auto switchProjectAction = new QAction(this);
+        ActionManager::registerAction(switchProjectAction, Core::Constants::GOTOPREVINHISTORY,
+                                      Context(Constants::C_PROJECTEXPLORER));
+        connect(switchProjectAction, &QAction::triggered, this, [this] {
+            if (m_projectSelection->count() > 1)
+                m_projectSelection->showPopup();
+        });
+
         SessionManager *sessionManager = SessionManager::instance();
         connect(sessionManager, &SessionManager::projectAdded,
                 this, &ProjectWindowPrivate::registerProject);
@@ -424,7 +604,8 @@ public:
 
         auto innerLayout = new QVBoxLayout;
         innerLayout->setSpacing(10);
-        innerLayout->setContentsMargins(14, innerLayout->spacing(), 14, 0);
+        innerLayout->setContentsMargins(PanelsWidget::PanelVMargin, innerLayout->spacing(),
+                                        PanelsWidget::PanelVMargin, 0);
         innerLayout->addWidget(m_manageKits);
         innerLayout->addSpacerItem(new QSpacerItem(10, 30, QSizePolicy::Maximum, QSizePolicy::Maximum));
         innerLayout->addWidget(activeLabel);
@@ -439,6 +620,23 @@ public:
 
         auto selectorDock = q->addDockForWidget(selectorView, true);
         q->addDockWidget(Qt::LeftDockWidgetArea, selectorDock);
+
+        m_buildSystemOutput = new BuildSystemOutputWindow;
+        auto output = new QWidget;
+        // ProjectWindow sets background role to Base which is wrong for the output window,
+        // especially the find tool bar (resulting in wrong label color)
+        output->setBackgroundRole(QPalette::Window);
+        output->setObjectName("BuildSystemOutput");
+        output->setWindowTitle(ProjectWindow::tr("Build System Output"));
+        auto outputLayout = new QVBoxLayout;
+        output->setLayout(outputLayout);
+        outputLayout->setContentsMargins(0, 0, 0, 0);
+        outputLayout->setSpacing(0);
+        outputLayout->addWidget(m_buildSystemOutput->toolBar());
+        outputLayout->addWidget(m_buildSystemOutput);
+        outputLayout->addWidget(new FindToolBarPlaceHolder(m_buildSystemOutput));
+        auto outputDock = q->addDockForWidget(output, true);
+        q->addDockWidget(Qt::RightDockWidgetArea, outputDock);
     }
 
     void updatePanel()
@@ -506,7 +704,7 @@ public:
             item->setData(0, QVariant(), ItemActivatedDirectlyRole);
     }
 
-    void activateProjectPanel(Core::Id panelId)
+    void activateProjectPanel(Utils::Id panelId)
     {
         if (ProjectItem *projectItem = m_projectsModel.rootItem()->childAt(0)) {
             if (TreeItem *item = projectItem->itemForProjectPanel(panelId))
@@ -547,7 +745,7 @@ public:
             if (auto kitPage = KitOptionsPage::instance())
                 kitPage->showKit(KitManager::kit(Id::fromSetting(projectItem->data(0, KitIdRole))));
         }
-        ICore::showOptionsDialog(Constants::KITS_SETTINGS_PAGE_ID, ICore::mainWindow());
+        ICore::showOptionsDialog(Constants::KITS_SETTINGS_PAGE_ID);
     }
 
     void handleImportBuild()
@@ -557,21 +755,19 @@ public:
         ProjectImporter *projectImporter = project ? project->projectImporter() : nullptr;
         QTC_ASSERT(projectImporter, return);
 
-        QString dir = project->projectDirectory().toString();
-        QString importDir = QFileDialog::getExistingDirectory(ICore::mainWindow(),
-                                                              ProjectWindow::tr("Import Directory"),
-                                                              dir);
-        FilePath path = FilePath::fromString(importDir);
+        FilePath importDir =
+                FileUtils::getExistingDirectory(nullptr, ProjectWindow::tr("Import Directory"),
+                                                project->projectDirectory());
 
         Target *lastTarget = nullptr;
         BuildConfiguration *lastBc = nullptr;
-        for (const BuildInfo &info : projectImporter->import(path, false)) {
+        for (const BuildInfo &info : projectImporter->import(importDir, false)) {
             Target *target = project->target(info.kitId);
             if (!target)
                 target = project->addTargetForKit(KitManager::kit(info.kitId));
             if (target) {
                 projectImporter->makePersistent(target->kit());
-                BuildConfiguration *bc = info.factory()->create(target, info);
+                BuildConfiguration *bc = info.factory->create(target, info);
                 QTC_ASSERT(bc, continue);
                 target->addBuildConfiguration(bc);
 
@@ -587,6 +783,7 @@ public:
 
     void setPanel(QWidget *panel)
     {
+        q->savePersistentSettings();
         if (QWidget *widget = q->centralWidget()) {
             q->takeCentralWidget();
             widget->hide(); // Don't delete.
@@ -597,6 +794,7 @@ public:
             if (q->hasFocus()) // we get assigned focus from setFocusToCurrentMode, pass that on
                 panel->setFocus();
         }
+        q->loadPersistentSettings();
     }
 
     ProjectWindow *q;
@@ -607,6 +805,7 @@ public:
     SelectorTree *m_selectorTree;
     QPushButton *m_importBuild;
     QPushButton *m_manageKits;
+    BuildSystemOutputWindow *m_buildSystemOutput;
 };
 
 //
@@ -623,13 +822,52 @@ ProjectWindow::ProjectWindow()
     setContextMenuPolicy(Qt::CustomContextMenu);
 }
 
-void ProjectWindow::activateProjectPanel(Core::Id panelId)
+void ProjectWindow::activateProjectPanel(Utils::Id panelId)
 {
     d->activateProjectPanel(panelId);
 }
 
+OutputWindow *ProjectWindow::buildSystemOutput() const
+{
+    return d->m_buildSystemOutput;
+}
+
+void ProjectWindow::hideEvent(QHideEvent *event)
+{
+    savePersistentSettings();
+    FancyMainWindow::hideEvent(event);
+}
+
+void ProjectWindow::showEvent(QShowEvent *event)
+{
+    FancyMainWindow::showEvent(event);
+    loadPersistentSettings();
+}
+
 ProjectWindow::~ProjectWindow() = default;
 
+const char PROJECT_WINDOW_KEY[] = "ProjectExplorer.ProjectWindow";
+
+void ProjectWindow::savePersistentSettings() const
+{
+    if (!centralWidget())
+        return;
+    QSettings * const settings = ICore::settings();
+    settings->beginGroup(PROJECT_WINDOW_KEY);
+    saveSettings(settings);
+    settings->endGroup();
+}
+
+void ProjectWindow::loadPersistentSettings()
+{
+    if (!centralWidget())
+        return;
+    QSettings * const settings = ICore::settings();
+    settings->beginGroup(PROJECT_WINDOW_KEY);
+    restoreSettings(settings);
+    settings->endGroup();
+}
+
 QSize SelectorDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
     QSize s = QStyledItemDelegate::sizeHint(option, index);
