@@ -26,14 +26,15 @@
 #include "targetsetuppage.h"
 #include "buildconfiguration.h"
 #include "buildinfo.h"
+#include "importwidget.h"
 #include "kit.h"
 #include "kitmanager.h"
-#include "importwidget.h"
 #include "project.h"
 #include "projectexplorerconstants.h"
 #include "session.h"
 #include "target.h"
 #include "targetsetupwidget.h"
+#include "task.h"
 
 #include <coreplugin/icore.h>
 
@@ -45,12 +46,13 @@
 #include <utils/algorithm.h>
 #include <utils/fancylineedit.h>
 
+#include <QApplication>
+#include <QCheckBox>
 #include <QFileInfo>
 #include <QLabel>
 #include <QMessageBox>
 #include <QScrollArea>
 #include <QVBoxLayout>
-#include <QCheckBox>
 
 using namespace Utils;
 
@@ -100,9 +102,11 @@ public:
 
         noValidKitLabel = new QLabel(setupTargetPage);
         noValidKitLabel->setWordWrap(true);
-        noValidKitLabel->setText(TargetSetupPage::tr("<span style=\" font-weight:600;\">No suitable kits found.</span><br/>"
-                                 "Add a kit in the <a href=\"buildandrun\">options</a> "
-                                 "or via the maintenance tool of the SDK."));
+        noValidKitLabel->setText("<span style=\" font-weight:600;\">"
+                                 + TargetSetupPage::tr("No suitable kits found.") + "</span><br/>"
+                                 + TargetSetupPage::tr("Add a kit in the <a href=\"buildandrun\">"
+                                                       "options</a> or via the maintenance tool of"
+                                                       " the SDK."));
         noValidKitLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
         noValidKitLabel->setVisible(false);
 
@@ -134,11 +138,15 @@ public:
         verticalLayout->setContentsMargins(0, 0, 0, 0);
         verticalLayout->addWidget(scrollArea);
 
+        auto horizontalLayout = new QHBoxLayout;
+        horizontalLayout->addWidget(allKitsCheckBox);
+        horizontalLayout->addSpacing(10);
+        horizontalLayout->addWidget(kitFilterLineEdit);
+
         auto verticalLayout_2 = new QVBoxLayout(setupTargetPage);
         verticalLayout_2->addWidget(headerLabel);
-        verticalLayout_2->addWidget(kitFilterLineEdit);
+        verticalLayout_2->addLayout(horizontalLayout);
         verticalLayout_2->addWidget(noValidKitLabel);
-        verticalLayout_2->addWidget(allKitsCheckBox);
         verticalLayout_2->addWidget(centralWidget);
         verticalLayout_2->addWidget(scrollAreaWidget);
 
@@ -159,13 +167,27 @@ public:
 
 } // namespace Internal
 
+static TasksGenerator defaultTasksGenerator(const TasksGenerator &childGenerator)
+{
+    return [childGenerator](const Kit *k) -> Tasks {
+        if (!k->isValid())
+            return {
+                CompileTask(Task::Error,
+                            QCoreApplication::translate("ProjectExplorer", "Kit is not valid."))};
+        if (childGenerator)
+            return childGenerator(k);
+        return {};
+    };
+}
+
 using namespace Internal;
 
-TargetSetupPage::TargetSetupPage(QWidget *parent) :
-    WizardPage(parent),
-    m_ui(new TargetSetupPageUi),
-    m_importWidget(new ImportWidget(this)),
-    m_spacer(new QSpacerItem(0,0, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding))
+TargetSetupPage::TargetSetupPage(QWidget *parent)
+    : WizardPage(parent)
+    , m_tasksGenerator(defaultTasksGenerator({}))
+    , m_ui(new TargetSetupPageUi)
+    , m_importWidget(new ImportWidget(this))
+    , m_spacer(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding))
 {
     m_importWidget->setVisible(false);
 
@@ -187,7 +209,7 @@ TargetSetupPage::TargetSetupPage(QWidget *parent) :
 
     setTitle(tr("Kit Selection"));
 
-    for (IPotentialKit *pk : g_potentialKits)
+    for (IPotentialKit *pk : qAsConst(g_potentialKits))
         if (pk->isEnabled())
             m_potentialWidgets.append(pk->createWidget(this));
 
@@ -217,14 +239,14 @@ void TargetSetupPage::initializePage()
     }
 }
 
-void TargetSetupPage::setRequiredKitPredicate(const Kit::Predicate &predicate)
+void TargetSetupPage::setTasksGenerator(const TasksGenerator &tasksGenerator)
 {
-    m_requiredPredicate = predicate;
+    m_tasksGenerator = defaultTasksGenerator(tasksGenerator);
 }
 
-QList<Core::Id> TargetSetupPage::selectedKits() const
+QList<Utils::Id> TargetSetupPage::selectedKits() const
 {
-    QList<Core::Id> result;
+    QList<Utils::Id> result;
     for (TargetSetupWidget *w : m_widgets) {
         if (w->isKitSelected())
             result.append(w->kit()->id());
@@ -232,11 +254,6 @@ QList<Core::Id> TargetSetupPage::selectedKits() const
     return result;
 }
 
-void TargetSetupPage::setPreferredKitPredicate(const Kit::Predicate &predicate)
-{
-    m_preferredPredicate = predicate;
-}
-
 TargetSetupPage::~TargetSetupPage()
 {
     disconnect();
@@ -259,7 +276,6 @@ void TargetSetupPage::setupWidgets(const QString &filterText)
         if (!filterText.isEmpty() && !k->displayName().contains(filterText, Qt::CaseInsensitive))
             continue;
         const auto widget = new TargetSetupWidget(k, m_projectPath);
-        setInitialCheckState(widget);
         connect(widget, &TargetSetupWidget::selectedToggled,
                 this, &TargetSetupPage::kitSelectionChanged);
         connect(widget, &TargetSetupWidget::selectedToggled, this, &QWizardPage::completeChanged);
@@ -272,6 +288,7 @@ void TargetSetupPage::setupWidgets(const QString &filterText)
     // Setup import widget:
     m_importWidget->setCurrentDirectory(Internal::importDirectory(m_projectPath));
 
+    kitSelectionChanged();
     updateVisibility();
 }
 
@@ -291,13 +308,7 @@ void TargetSetupPage::reset()
     m_ui->allKitsCheckBox->setChecked(false);
 }
 
-void TargetSetupPage::setInitialCheckState(TargetSetupWidget *widget)
-{
-    widget->setKitSelected(widget->isEnabled() && m_preferredPredicate
-                           && m_preferredPredicate(widget->kit()));
-}
-
-TargetSetupWidget *TargetSetupPage::widget(const Core::Id kitId,
+TargetSetupWidget *TargetSetupPage::widget(const Utils::Id kitId,
                                            TargetSetupWidget *fallback) const
 {
     return findOr(m_widgets, fallback, [kitId](const TargetSetupWidget *w) {
@@ -357,6 +368,7 @@ void TargetSetupPage::handleKitAddition(Kit *k)
 
     Q_ASSERT(!widget(k));
     addWidget(k);
+    kitSelectionChanged();
     updateVisibility();
 }
 
@@ -391,31 +403,48 @@ void TargetSetupPage::handleKitUpdate(Kit *k)
     updateVisibility();
 }
 
-void TargetSetupPage::selectAtLeastOneKit()
+void TargetSetupPage::selectAtLeastOneEnabledKit()
 {
-    bool atLeastOneKitSelected = anyOf(m_widgets, [](TargetSetupWidget *w) {
-            return w->isKitSelected();
+    if (anyOf(m_widgets, [](const TargetSetupWidget *w) { return w->isKitSelected(); })) {
+        // Something is already selected, we are done.
+        return;
+    }
+
+    TargetSetupWidget *toCheckWidget = nullptr;
+
+    const Kit *defaultKit = KitManager::defaultKit();
+
+    auto isPreferred = [this](const TargetSetupWidget *w) {
+        const Tasks tasks = m_tasksGenerator(w->kit());
+        return w->isEnabled() && tasks.isEmpty();
+    };
+
+    // Use default kit if that is preferred:
+    toCheckWidget = findOrDefault(m_widgets, [defaultKit, isPreferred](const TargetSetupWidget *w) {
+        return isPreferred(w) && w->kit() == defaultKit;
     });
 
-    if (!atLeastOneKitSelected) {
-        Kit * const defaultKit = KitManager::defaultKit();
-        if (defaultKit && isUsable(defaultKit)) {
-            if (TargetSetupWidget * const w = widget(defaultKit)) {
-                w->setKitSelected(true);
-                atLeastOneKitSelected = true;
-            }
-        }
+    if (!toCheckWidget) {
+        // Use the first preferred widget:
+        toCheckWidget = findOrDefault(m_widgets, isPreferred);
     }
-    if (!atLeastOneKitSelected) {
-        for (TargetSetupWidget * const w : qAsConst(m_widgets)) {
-            if (isUsable(w->kit())) {
-                w->setKitSelected(true);
-                atLeastOneKitSelected = true;
-            }
-        }
+
+    if (!toCheckWidget) {
+        // Use default kit if it is enabled:
+        toCheckWidget = findOrDefault(m_widgets, [defaultKit](const TargetSetupWidget *w) {
+            return w->isEnabled() && w->kit() == defaultKit;
+        });
     }
-    if (atLeastOneKitSelected) {
-        kitSelectionChanged();
+
+    if (!toCheckWidget) {
+        // Use the first enabled widget:
+        toCheckWidget = findOrDefault(m_widgets,
+                                      [](const TargetSetupWidget *w) { return w->isEnabled(); });
+    }
+
+    if (toCheckWidget) {
+        toCheckWidget->setKitSelected(true);
+
         emit completeChanged(); // Is this necessary?
     }
 }
@@ -488,10 +517,27 @@ void TargetSetupPage::kitSelectionChanged()
 
 void TargetSetupPage::kitFilterChanged(const QString &filterText)
 {
+    QPointer<QWidget> focusWidget = QApplication::focusWidget();
+    // Remember selected kits:
+    const std::vector<TargetSetupWidget *> selectedWidgets
+        = filtered(m_widgets, &TargetSetupWidget::isKitSelected);
+    const QVector<Utils::Id> selectedKitIds = transform<QVector>(selectedWidgets,
+                                                                [](const TargetSetupWidget *w) {
+                                                                    return w->kit()->id();
+                                                                });
+
     // Reset currently shown kits
     reset();
     setupWidgets(filterText);
-    selectAtLeastOneKit();
+
+    // Re-select kits:
+    for (TargetSetupWidget *w : qAsConst(m_widgets))
+        w->setKitSelected(selectedKitIds.contains(w->kit()->id()));
+
+    emit completeChanged();
+
+    if (focusWidget)
+        focusWidget->setFocus();
 }
 
 void TargetSetupPage::doInitializePage()
@@ -499,10 +545,18 @@ void TargetSetupPage::doInitializePage()
     reset();
     setupWidgets();
     setupImports();
-    selectAtLeastOneKit();
+
+    selectAtLeastOneEnabledKit();
+
     updateVisibility();
 }
 
+void TargetSetupPage::showEvent(QShowEvent *event)
+{
+    WizardPage::showEvent(event);
+    m_ui->kitFilterLineEdit->setFocus(); // Ensure "Configure Project" gets triggered on <Return>
+}
+
 void TargetSetupPage::changeAllKitsSelections()
 {
     if (m_ui->allKitsCheckBox->checkState() == Qt::PartiallyChecked)
@@ -554,7 +608,6 @@ void TargetSetupPage::removeWidget(TargetSetupWidget *w)
 TargetSetupWidget *TargetSetupPage::addWidget(Kit *k)
 {
     const auto widget = new TargetSetupWidget(k, m_projectPath);
-    setInitialCheckState(widget);
     updateWidget(widget);
     connect(widget, &TargetSetupWidget::selectedToggled,
             this, &TargetSetupPage::kitSelectionChanged);
@@ -575,6 +628,7 @@ TargetSetupWidget *TargetSetupPage::addWidget(Kit *k)
     } else {
         reLayout();
     }
+
     return widget;
 }
 
@@ -597,12 +651,12 @@ void TargetSetupPage::removeAdditionalWidgets(QLayout *layout)
 void TargetSetupPage::updateWidget(TargetSetupWidget *widget)
 {
     QTC_ASSERT(widget, return );
-    widget->update(m_requiredPredicate);
+    widget->update(m_tasksGenerator);
 }
 
 bool TargetSetupPage::isUsable(const Kit *kit) const
 {
-    return kit->isValid() && (!m_requiredPredicate || m_requiredPredicate(kit));
+    return !containsType(m_tasksGenerator(kit), Task::Error);
 }
 
 bool TargetSetupPage::setupProject(Project *project)
