@@ -50,6 +50,7 @@
 
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/icore.h>
+#include <qmlprojectmanager/qmlproject.h>
 #include <utils/algorithm.h>
 #include <utils/qtcassert.h>
 #include <utils/stylehelper.h>
@@ -512,7 +513,7 @@ public:
                                                                  ->currentModel();
 
                                        if (currentModel->rewriterView()
-                                           && currentModel->rewriterView()->inErrorState()) {
+                                           && !currentModel->rewriterView()->errors().isEmpty()) {
                                            throw DocumentError{};
                                        }
 
@@ -1471,16 +1472,19 @@ void DesignerActionManager::createDefaultDesignerActions()
                           &singleSelection,
                           &singleSelection));
 
-    addDesignerAction(new ModelNodeContextMenuAction(
-                          goToImplementationCommandId,
-                          goToImplementationDisplayName,
-                          {},
-                          rootCategory,
-                          QKeySequence(),
-                          42,
-                          &goImplementation,
-                          &singleSelectedAndUiFile,
-                          &singleSelectedAndUiFile));
+    const bool standaloneMode = QmlProjectManager::QmlProject::isQtDesignStudio();
+
+    if (!standaloneMode) {
+        addDesignerAction(new ModelNodeContextMenuAction(goToImplementationCommandId,
+                                                         goToImplementationDisplayName,
+                                                         {},
+                                                         rootCategory,
+                                                         QKeySequence(),
+                                                         42,
+                                                         &goImplementation,
+                                                         &singleSelectedAndUiFile,
+                                                         &singleSelectedAndUiFile));
+    }
 
     addDesignerAction(new ModelNodeContextMenuAction(
                           addSignalHandlerCommandId,
@@ -1502,6 +1506,17 @@ void DesignerActionManager::createDefaultDesignerActions()
                           &singleSelection,
                           &singleSelection));
 
+    addDesignerAction(new ModelNodeContextMenuAction(
+                          editMaterialCommandId,
+                          editMaterialDisplayName,
+                          {},
+                          rootCategory,
+                          QKeySequence(),
+                          44,
+                          &editMaterial,
+                          &modelHasMaterial,
+                          &isModel));
+
     addDesignerAction(new ModelNodeContextMenuAction(mergeTemplateCommandId,
                                                      mergeTemplateDisplayName,
                                                      {},
@@ -1550,9 +1565,12 @@ void DesignerActionManager::createDefaultAddResourceHandler()
             registerAddResourceHandler(AddResourceHandler(category, ext, op));
     };
 
+    // Having a single image type category creates too large of a filter, so we split images into
+    // categories according to their mime types
     auto transformer = [](const QByteArray& format) -> QString { return QString("*.") + format; };
     auto imageFormats = Utils::transform(QImageReader::supportedImageFormats(), transformer);
     imageFormats.push_back("*.hdr");
+    imageFormats.push_back("*.ktx");
 
     // The filters will be displayed in reverse order to these lists in file dialog,
     // so declare most common types last
