@@ -41,19 +41,18 @@ ModelNodeContextMenu::ModelNodeContextMenu(AbstractView *view) :
 {
 }
 
-static QSet<ActionInterface* > findMembers(QSet<ActionInterface* > actionInterface,
-                                                          const QByteArray &category)
+static QSet<ActionInterface *> findMembers(const QSet<ActionInterface *> actionInterface,
+                                           const QByteArray &category)
 {
-    QSet<ActionInterface* > ret;
+    QSet<ActionInterface *> ret;
 
-     foreach (ActionInterface* factory, actionInterface) {
-         if (factory->category() == category)
-             ret.insert(factory);
-     }
-     return ret;
+    for (ActionInterface *factory : actionInterface) {
+        if (factory->category() == category)
+            ret.insert(factory);
+    }
+    return ret;
 }
 
-
 void populateMenu(QSet<ActionInterface* > &actionInterfaces,
                   const QByteArray &category,
                   QMenu* menu,
@@ -63,12 +62,12 @@ void populateMenu(QSet<ActionInterface* > &actionInterfaces,
 
     actionInterfaces.subtract(matchingFactories);
 
-    QList<ActionInterface* > matchingFactoriesList = matchingFactories.toList();
+    QList<ActionInterface* > matchingFactoriesList = Utils::toList(matchingFactories);
     Utils::sort(matchingFactoriesList, [](ActionInterface *l, ActionInterface *r) {
         return l->priority() > r->priority();
     });
 
-    foreach (ActionInterface* actionInterface, matchingFactoriesList) {
+    for (ActionInterface* actionInterface : qAsConst(matchingFactoriesList)) {
         if (actionInterface->type() == ActionInterface::ContextMenu) {
             actionInterface->currentContextChanged(selectionContext);
             QMenu *newMenu = actionInterface->action()->menu();
@@ -91,7 +90,7 @@ void populateMenu(QSet<ActionInterface* > &actionInterfaces,
 
 void ModelNodeContextMenu::execute(const QPoint &position, bool selectionMenuBool)
 {
-    QMenu* mainMenu = new QMenu();
+    auto mainMenu = new QMenu();
 
     m_selectionContext.setShowSelectionTools(selectionMenuBool);
     m_selectionContext.setScenePosition(m_scenePos);
@@ -100,8 +99,7 @@ void ModelNodeContextMenu::execute(const QPoint &position, bool selectionMenuBoo
 
     manager.setupContext();
 
-     QSet<ActionInterface* > factories =
-             QSet<ActionInterface* >::fromList(manager.designerActions());
+    QSet<ActionInterface* > factories = Utils::toSet(manager.designerActions());
 
      populateMenu(factories, QByteArray(), mainMenu, m_selectionContext);
 
