@@ -86,7 +86,8 @@ void EnterTabDesignerAction::updateContext()
                     && selectedModelNode.metaInfo().isSubclassOf("QtQuick.Controls.TabView")) {
 
                 const NodeAbstractProperty defaultProperty = selectedModelNode.defaultNodeAbstractProperty();
-                foreach (const QmlDesigner::ModelNode &childModelNode, defaultProperty.directSubNodes()) {
+                const QList<QmlDesigner::ModelNode> childModelNodes = defaultProperty.directSubNodes();
+                for (const QmlDesigner::ModelNode &childModelNode : childModelNodes) {
                     createActionForTab(childModelNode);
                 }
             }
@@ -124,7 +125,7 @@ void EnterTabDesignerAction::createActionForTab(const ModelNode &modelNode)
         if (itemNode.isValid()) {
             QString what = tr("Step into: %1").
                     arg(itemNode.instanceValue("title").toString());
-            EnterTabAction *selectionAction = new EnterTabAction(what);
+            auto selectionAction = new EnterTabAction(what);
 
             SelectionContext nodeSelectionContext = selectionContext();
             nodeSelectionContext.setTargetNode(modelNode);
