@@ -405,7 +405,7 @@ void ModelPrivate::notifyNodeInstanceViewLast(Callable call)
         resetModel = true;
     }
 
-    for (QPointer<AbstractView> view : enabledViews()) {
+    for (const QPointer<AbstractView> &view : enabledViews()) {
         if (!view->isBlockingNotifications())
             call(view.data());
     }
@@ -434,7 +434,7 @@ void ModelPrivate::notifyNormalViewsLast(Callable call)
     if (nodeInstanceView() && !nodeInstanceView()->isBlockingNotifications())
         call(nodeInstanceView());
 
-    for (QPointer<AbstractView> view : enabledViews()) {
+    for (const QPointer<AbstractView> &view : enabledViews()) {
         if (!view->isBlockingNotifications())
             call(view.data());
     }
@@ -446,7 +446,7 @@ void ModelPrivate::notifyNormalViewsLast(Callable call)
 template<typename Callable>
 void ModelPrivate::notifyInstanceChanges(Callable call)
 {
-    for (QPointer<AbstractView> view : enabledViews()) {
+    for (const QPointer<AbstractView> &view : enabledViews()) {
         if (!view->isBlockingNotifications())
             call(view.data());
     }
@@ -843,6 +843,22 @@ void ModelPrivate::notifySignalHandlerPropertiesChanged(
     });
 }
 
+void ModelPrivate::notifySignalDeclarationPropertiesChanged(
+    const QVector<InternalSignalDeclarationPropertyPointer> &internalPropertyList,
+    AbstractView::PropertyChangeFlags propertyChange)
+{
+    notifyNodeInstanceViewLast([&](AbstractView *view) {
+        QVector<SignalDeclarationProperty> propertyList;
+        for (const InternalSignalDeclarationPropertyPointer &signalHandlerProperty : internalPropertyList) {
+            propertyList.append(SignalDeclarationProperty(signalHandlerProperty->name(),
+                                                      signalHandlerProperty->propertyOwner(),
+                                                      m_model,
+                                                      view));
+        }
+        view->signalDeclarationPropertiesChanged(propertyList, propertyChange);
+    });
+}
+
 void ModelPrivate::notifyScriptFunctionsChanged(const InternalNodePointer &node,
                                                 const QStringList &scriptFunctionList)
 {
@@ -1100,6 +1116,19 @@ void ModelPrivate::setSignalHandlerProperty(const InternalNodePointer &node, con
     notifySignalHandlerPropertiesChanged({signalHandlerProperty}, propertyChange);
 }
 
+void ModelPrivate::setSignalDeclarationProperty(const InternalNodePointer &node, const PropertyName &name, const QString &signature)
+{
+    AbstractView::PropertyChangeFlags propertyChange = AbstractView::NoAdditionalChanges;
+    if (!node->hasProperty(name)) {
+        node->addSignalDeclarationProperty(name);
+        propertyChange = AbstractView::PropertiesAdded;
+    }
+
+    InternalSignalDeclarationPropertyPointer signalDeclarationProperty = node->signalDeclarationProperty(name);
+    signalDeclarationProperty->setSignature(signature);
+    notifySignalDeclarationPropertiesChanged({signalDeclarationProperty}, propertyChange);
+}
+
 void ModelPrivate::setVariantProperty(const InternalNodePointer &node, const PropertyName &name, const QVariant &value)
 {
     AbstractView::PropertyChangeFlags propertyChange = AbstractView::NoAdditionalChanges;
