@@ -67,7 +67,8 @@ static inline QHash<PropertyName, QVariant> getProperties(const ModelNode &node)
 {
     QHash<PropertyName, QVariant> propertyHash;
     if (QmlObjectNode::isValidQmlObjectNode(node)) {
-        foreach (const AbstractProperty &abstractProperty, node.properties()) {
+        const QList<AbstractProperty> abstractProperties = node.properties();
+        for (const AbstractProperty &abstractProperty : abstractProperties) {
             if (abstractProperty.isVariantProperty()
                     || (abstractProperty.isBindingProperty()
                         && !abstractProperty.name().contains("anchors.")))
@@ -91,9 +92,10 @@ static inline QHash<PropertyName, QVariant> getProperties(const ModelNode &node)
 
 static inline void applyProperties(ModelNode &node, const QHash<PropertyName, QVariant> &propertyHash)
 {
-    QHash<PropertyName, QVariant> auxiliaryData  = node.auxiliaryData();
+    const QHash<PropertyName, QVariant> auxiliaryData  = node.auxiliaryData();
 
-    foreach (const PropertyName &propertyName, auxiliaryData.keys()) {
+    const QList<PropertyName> propertyNames = auxiliaryData.keys();
+    for (const PropertyName &propertyName : propertyNames) {
         if (node.hasAuxiliaryData(propertyName))
             node.setAuxiliaryData(propertyName, QVariant());
     }
@@ -108,11 +110,17 @@ static inline void applyProperties(ModelNode &node, const QHash<PropertyName, QV
     }
 }
 
-static void openFileComponent(const ModelNode &modelNode)
+static void openFileComponentForFile(const QString &fileName)
 {
     QmlDesignerPlugin::instance()->viewManager().nextFileIsCalledInternally();
-    Core::EditorManager::openEditor(modelNode.metaInfo().componentFileName(),
-        Utils::Id(), Core::EditorManager::DoNotMakeVisible);
+    Core::EditorManager::openEditor(FilePath::fromString(fileName),
+                                    Utils::Id(),
+                                    Core::EditorManager::DoNotMakeVisible);
+}
+
+static void openFileComponent(const ModelNode &modelNode)
+{
+    openFileComponentForFile(modelNode.metaInfo().componentFileName());
 }
 
 static void openFileComponentForDelegate(const ModelNode &modelNode)
@@ -138,7 +146,10 @@ static void openComponentSourcePropertyOfLoader(const ModelNode &modelNode)
         componentModelNode = modelNode.nodeListProperty("component").toModelNodeList().constFirst();
     }
 
-    Core::EditorManager::openEditor(componentModelNode.metaInfo().componentFileName(), Utils::Id(), Core::EditorManager::DoNotMakeVisible);
+    Core::EditorManager::openEditor(FilePath::fromString(
+                                        componentModelNode.metaInfo().componentFileName()),
+                                    Utils::Id(),
+                                    Core::EditorManager::DoNotMakeVisible);
 }
 
 static void openSourcePropertyOfLoader(const ModelNode &modelNode)
@@ -148,7 +159,10 @@ static void openSourcePropertyOfLoader(const ModelNode &modelNode)
     QString componentFileName = modelNode.variantProperty("source").value().toString();
 
     QFileInfo fileInfo(modelNode.model()->fileUrl().toLocalFile());
-    Core::EditorManager::openEditor(fileInfo.absolutePath() + "/" + componentFileName, Utils::Id(), Core::EditorManager::DoNotMakeVisible);
+    Core::EditorManager::openEditor(FilePath::fromString(fileInfo.absolutePath())
+                                        / componentFileName,
+                                    Utils::Id(),
+                                    Core::EditorManager::DoNotMakeVisible);
 }
 
 
@@ -269,7 +283,7 @@ bool DocumentManager::hasCurrentDesignDocument() const
 
 void DocumentManager::removeEditors(const QList<Core::IEditor *> &editors)
 {
-    foreach (Core::IEditor *editor, editors)
+    for (Core::IEditor *editor : editors)
         delete m_designDocumentHash.take(editor).data();
 }
 
@@ -298,6 +312,11 @@ bool DocumentManager::goIntoComponent(const ModelNode &modelNode)
     return false;
 }
 
+void DocumentManager::goIntoComponent(const QString &fileName)
+{
+    openFileComponentForFile(fileName);
+}
+
 bool DocumentManager::createFile(const QString &filePath, const QString &contents)
 {
     Utils::TextFileFormat textFileFormat;
@@ -341,11 +360,18 @@ Utils::FilePath DocumentManager::currentProjectDirPath()
         return {};
 
     Utils::FilePath qmlFileName = QmlDesignerPlugin::instance()->currentDesignDocument()->fileName();
-    ProjectExplorer::Project *project = ProjectExplorer::SessionManager::projectForFile(qmlFileName);
-    if (!project)
-        return {};
 
-    return project->projectDirectory();
+    ProjectExplorer::Project *project = ProjectExplorer::SessionManager::projectForFile(qmlFileName);
+    if (project)
+        return project->projectDirectory();
+
+    const QList projects = ProjectExplorer::SessionManager::projects();
+    for (auto p : projects) {
+        if (qmlFileName.startsWith(p->projectDirectory().toString()))
+            return p->projectDirectory();
+    }
+
+    return {};
 }
 
 QStringList DocumentManager::isoIconsQmakeVariableValue(const QString &proPath)
@@ -504,8 +530,14 @@ bool DocumentManager::belongsToQmakeProject()
 Utils::FilePath DocumentManager::currentResourcePath()
 {
     Utils::FilePath resourcePath = currentProjectDirPath();
+
     if (resourcePath.isEmpty())
         return currentFilePath().absolutePath();
+
+    FilePath contentFilePath = resourcePath.pathAppended("content");
+    if (contentFilePath.exists())
+        return contentFilePath;
+
     return resourcePath;
 }
 
