@@ -39,7 +39,6 @@
 #include <coreplugin/dialogs/restartdialog.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/progressmanager/progressmanager.h>
-#include <coreplugin/variablechooser.h>
 
 #include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/projectexplorericons.h>
@@ -55,10 +54,10 @@
 #include <utils/runextensions.h>
 #include <utils/treemodel.h>
 #include <utils/utilsicons.h>
+#include <utils/variablechooser.h>
 
 #include <QDesktopServices>
 #include <QDir>
-#include <QFileDialog>
 #include <QMessageBox>
 #include <QSortFilterProxyModel>
 #include <QTextBrowser>
@@ -78,7 +77,7 @@ class QtVersionItem : public TreeItem
     Q_DECLARE_TR_FUNCTIONS(QtSupport::QtVersion)
 
 public:
-    explicit QtVersionItem(BaseQtVersion *version)
+    explicit QtVersionItem(QtVersion *version)
         : m_version(version)
     {}
 
@@ -87,7 +86,7 @@ public:
         delete m_version;
     }
 
-    void setVersion(BaseQtVersion *version)
+    void setVersion(QtVersion *version)
     {
         m_version = version;
         update();
@@ -98,12 +97,12 @@ public:
         return m_version ? m_version->uniqueId() : -1;
     }
 
-    BaseQtVersion *version() const
+    QtVersion *version() const
     {
         return m_version;
     }
 
-    QVariant data(int column, int role) const
+    QVariant data(int column, int role) const final
     {
         if (!m_version)
             return TreeItem::data(column, role);
@@ -112,7 +111,7 @@ public:
             if (column == 0)
                 return m_version->displayName();
             if (column == 1)
-                return m_version->qmakeCommand().toUserOutput();
+                return m_version->qmakeFilePath().toUserOutput();
         }
 
         if (role == Qt::FontRole && m_changed) {
@@ -125,11 +124,12 @@ public:
             return m_icon;
 
         if (role == Qt::ToolTipRole) {
-            const QString row = "<tr><td>%1:</td><td>%2</td></tr>";
-            return QString("<table>"
+            const QString row = "<dt style=\"font-weight:bold\">%1:</dt>"
+                                "<dd>%2</dd>";
+            return QString("<dl style=\"white-space:pre\">"
                          + row.arg(tr("Qt Version"), m_version->qtVersionString())
-                         + row.arg(tr("Location of qmake"), m_version->qmakeCommand().toUserOutput())
-                         + "</table>");
+                         + row.arg(tr("Location of qmake"), m_version->qmakeFilePath().toUserOutput())
+                         + "</dl>");
         }
 
         return QVariant();
@@ -162,7 +162,7 @@ public:
     }
 
 private:
-    BaseQtVersion *m_version = nullptr;
+    QtVersion *m_version = nullptr;
     QIcon m_icon;
     QString m_buildLog;
     bool m_changed = false;
@@ -187,7 +187,7 @@ private:
     void userChangedCurrentVersion();
     void updateWidgets();
     void setupLinkWithQtButton();
-    BaseQtVersion *currentVersion() const;
+    QtVersion *currentVersion() const;
     QtVersionItem *currentItem() const;
     void showDebuggingBuildLog(const QtVersionItem *item);
 
@@ -222,11 +222,11 @@ private:
         QString toolTip;
         QIcon icon;
     };
-    ValidityInfo validInformation(const BaseQtVersion *version);
-    QList<ProjectExplorer::ToolChain*> toolChains(const BaseQtVersion *version);
-    QByteArray defaultToolChainId(const BaseQtVersion *version);
+    ValidityInfo validInformation(const QtVersion *version);
+    QList<ProjectExplorer::ToolChain*> toolChains(const QtVersion *version);
+    QByteArray defaultToolChainId(const QtVersion *version);
 
-    bool isNameUnique(const BaseQtVersion *version);
+    bool isNameUnique(const QtVersion *version);
     void updateVersionItem(QtVersionItem *item);
 
     TreeModel<TreeItem, TreeItem, QtVersionItem> *m_model;
@@ -261,11 +261,12 @@ QtOptionsPageWidget::QtOptionsPageWidget()
     m_ui.versionInfoWidget->setWidget(versionInfoWidget);
     m_ui.versionInfoWidget->setState(DetailsWidget::NoSummary);
 
-    m_autoItem = new StaticTreeItem(tr("Auto-detected"));
-    m_manualItem = new StaticTreeItem(tr("Manual"));
+    m_autoItem = new StaticTreeItem({ProjectExplorer::Constants::msgAutoDetected()},
+                                    {ProjectExplorer::Constants::msgAutoDetectedToolTip()});
+    m_manualItem = new StaticTreeItem(ProjectExplorer::Constants::msgManual());
 
     m_model = new TreeModel<TreeItem, TreeItem, QtVersionItem>();
-    m_model->setHeader({tr("Name"), tr("qmake Location")});
+    m_model->setHeader({tr("Name"), tr("qmake Path")});
     m_model->rootItem()->appendChild(m_autoItem);
     m_model->rootItem()->appendChild(m_manualItem);
 
@@ -295,7 +296,7 @@ QtOptionsPageWidget::QtOptionsPageWidget()
     if (selectedIndex >= 0)
         m_ui.documentationSetting->setCurrentIndex(selectedIndex);
 
-    QList<int> additions = transform(QtVersionManager::versions(), &BaseQtVersion::uniqueId);
+    QList<int> additions = transform(QtVersionManager::versions(), &QtVersion::uniqueId);
 
     updateQtVersions(additions, QList<int>(), QList<int>());
 
@@ -326,15 +327,15 @@ QtOptionsPageWidget::QtOptionsPageWidget()
     connect(ProjectExplorer::ToolChainManager::instance(), &ToolChainManager::toolChainsChanged,
             this, &QtOptionsPageWidget::toolChainsUpdated);
 
-    auto chooser = new Core::VariableChooser(this);
+    auto chooser = new VariableChooser(this);
     chooser->addSupportedWidget(m_versionUi.nameEdit, "Qt:Name");
     chooser->addMacroExpanderProvider([this] {
-        BaseQtVersion *version = currentVersion();
+        QtVersion *version = currentVersion();
         return version ? version->macroExpander() : nullptr;
     });
 }
 
-BaseQtVersion *QtOptionsPageWidget::currentVersion() const
+QtVersion *QtOptionsPageWidget::currentVersion() const
 {
     QtVersionItem *item = currentItem();
     if (!item)
@@ -414,7 +415,7 @@ static QString formatAbiHtmlList(const Abis &abis)
     return result;
 }
 
-QtOptionsPageWidget::ValidityInfo QtOptionsPageWidget::validInformation(const BaseQtVersion *version)
+QtOptionsPageWidget::ValidityInfo QtOptionsPageWidget::validInformation(const QtVersion *version)
 {
     ValidityInfo info;
     info.icon = m_validVersionIcon;
@@ -477,7 +478,7 @@ QtOptionsPageWidget::ValidityInfo QtOptionsPageWidget::validInformation(const Ba
     return info;
 }
 
-QList<ToolChain*> QtOptionsPageWidget::toolChains(const BaseQtVersion *version)
+QList<ToolChain*> QtOptionsPageWidget::toolChains(const QtVersion *version)
 {
     QList<ToolChain*> toolChains;
     if (!version)
@@ -496,7 +497,7 @@ QList<ToolChain*> QtOptionsPageWidget::toolChains(const BaseQtVersion *version)
     return toolChains;
 }
 
-QByteArray QtOptionsPageWidget::defaultToolChainId(const BaseQtVersion *version)
+QByteArray QtOptionsPageWidget::defaultToolChainId(const QtVersion *version)
 {
     QList<ToolChain*> possibleToolChains = toolChains(version);
     if (!possibleToolChains.isEmpty())
@@ -504,12 +505,12 @@ QByteArray QtOptionsPageWidget::defaultToolChainId(const BaseQtVersion *version)
     return QByteArray();
 }
 
-bool QtOptionsPageWidget::isNameUnique(const BaseQtVersion *version)
+bool QtOptionsPageWidget::isNameUnique(const QtVersion *version)
 {
     const QString name = version->displayName().trimmed();
 
     return !m_model->findItemAtLevel<2>([name, version](QtVersionItem *item) {
-        BaseQtVersion *v = item->version();
+        QtVersion *v = item->version();
         return v != version && v->displayName().trimmed() == name;
     });
 }
@@ -551,7 +552,7 @@ void BuildLogDialog::setText(const QString &text)
 
 void QtOptionsPageWidget::showDebuggingBuildLog(const QtVersionItem *item)
 {
-    BaseQtVersion *version = item->version();
+    QtVersion *version = item->version();
     if (!version)
         return;
     BuildLogDialog *dialog = new BuildLogDialog(this->window());
@@ -583,7 +584,7 @@ void QtOptionsPageWidget::updateQtVersions(const QList<int> &additions, const QL
 
     // Add changed/added items:
     foreach (int a, toAdd) {
-        BaseQtVersion *version = QtVersionManager::version(a)->clone();
+        QtVersion *version = QtVersionManager::version(a)->clone();
         auto *item = new QtVersionItem(version);
 
         // Insert in the right place:
@@ -601,25 +602,23 @@ QtOptionsPageWidget::~QtOptionsPageWidget()
 
 void QtOptionsPageWidget::addQtDir()
 {
-    FilePath qtVersion = FilePath::fromString(
-                QFileDialog::getOpenFileName(this,
-                                             tr("Select a qmake Executable"),
-                                             QString(),
-                                             BuildableHelperLibrary::filterForQmakeFileDialog(),
-                                             0,
-                                             QFileDialog::DontResolveSymlinks));
+    FilePath qtVersion = FileUtils::getOpenFilePath(this,
+                                                    tr("Select a qmake Executable"),
+                                                    {},
+                                                    BuildableHelperLibrary::filterForQmakeFileDialog(),
+                                                    0,
+                                                    QFileDialog::DontResolveSymlinks);
     if (qtVersion.isEmpty())
         return;
 
-    QFileInfo fi = qtVersion.toFileInfo();
     // should add all qt versions here ?
-    if (BuildableHelperLibrary::isQtChooser(fi))
-        qtVersion = FilePath::fromString(BuildableHelperLibrary::qtChooserToQmakePath(fi.symLinkTarget()));
+    if (BuildableHelperLibrary::isQtChooser(qtVersion))
+        qtVersion = BuildableHelperLibrary::qtChooserToQmakePath(qtVersion.symLinkTarget());
 
     auto checkAlreadyExists = [qtVersion](TreeItem *parent) {
         for (int i = 0; i < parent->childCount(); ++i) {
             auto item = static_cast<QtVersionItem *>(parent->childAt(i));
-            if (item->version()->qmakeCommand() == qtVersion) {
+            if (item->version()->qmakeFilePath() == qtVersion) {
                 return std::make_pair(true, item->version()->displayName());
             }
         }
@@ -641,7 +640,7 @@ void QtOptionsPageWidget::addQtDir()
     }
 
     QString error;
-    BaseQtVersion *version = QtVersionFactory::createQtVersionFromQMakePath(qtVersion, false, QString(), &error);
+    QtVersion *version = QtVersionFactory::createQtVersionFromQMakePath(qtVersion, false, QString(), &error);
     if (version) {
         auto item = new QtVersionItem(version);
         item->setIcon(version->isValid()? m_validVersionIcon : m_invalidVersionIcon);
@@ -671,18 +670,17 @@ void QtOptionsPageWidget::removeQtDir()
 
 void QtOptionsPageWidget::editPath()
 {
-    BaseQtVersion *current = currentVersion();
-    QString dir = currentVersion()->qmakeCommand().toFileInfo().absolutePath();
-    FilePath qtVersion = FilePath::fromString(
-        QFileDialog::getOpenFileName(this,
-                                     tr("Select a qmake Executable"),
-                                     dir,
-                                     BuildableHelperLibrary::filterForQmakeFileDialog(),
-                                     nullptr,
-                                     QFileDialog::DontResolveSymlinks));
+    QtVersion *current = currentVersion();
+    FilePath qtVersion =
+            FileUtils::getOpenFilePath(this,
+                                       tr("Select a qmake Executable"),
+                                       current->qmakeFilePath().absolutePath(),
+                                       BuildableHelperLibrary::filterForQmakeFileDialog(),
+                                       nullptr,
+                                       QFileDialog::DontResolveSymlinks);
     if (qtVersion.isEmpty())
         return;
-    BaseQtVersion *version = QtVersionFactory::createQtVersionFromQMakePath(qtVersion);
+    QtVersion *version = QtVersionFactory::createQtVersionFromQMakePath(qtVersion);
     if (!version)
         return;
     // Same type? then replace!
@@ -733,7 +731,7 @@ void QtOptionsPageWidget::userChangedCurrentVersion()
 void QtOptionsPageWidget::updateDescriptionLabel()
 {
     QtVersionItem *item = currentItem();
-    const BaseQtVersion *version = item ? item->version() : nullptr;
+    const QtVersion *version = item ? item->version() : nullptr;
     const ValidityInfo info = validInformation(version);
     if (info.message.isEmpty()) {
         m_versionUi.errorLabel->setVisible(false);
@@ -767,10 +765,10 @@ void QtOptionsPageWidget::updateWidgets()
 {
     delete m_configurationWidget;
     m_configurationWidget = nullptr;
-    BaseQtVersion *version = currentVersion();
+    QtVersion *version = currentVersion();
     if (version) {
         m_versionUi.nameEdit->setText(version->unexpandedDisplayName());
-        m_versionUi.qmakePath->setText(version->qmakeCommand().toUserOutput());
+        m_versionUi.qmakePath->setText(version->qmakeFilePath().toUserOutput());
         m_configurationWidget = version->createConfigurationWidget();
         if (m_configurationWidget) {
             m_versionUi.formLayout->addRow(m_configurationWidget);
@@ -802,9 +800,9 @@ static QString qtVersionsFile(const QString &baseDir)
            + Core::Constants::IDE_ID + '/' + "qtversion.xml";
 }
 
-static Utils::optional<QString> currentlyLinkedQtDir(bool *hasInstallSettings)
+static Utils::optional<FilePath> currentlyLinkedQtDir(bool *hasInstallSettings)
 {
-    const QString installSettingsFilePath = settingsFile(Core::ICore::resourcePath());
+    const QString installSettingsFilePath = settingsFile(Core::ICore::resourcePath().toString());
     const bool installSettingsExist = QFile::exists(installSettingsFilePath);
     if (hasInstallSettings)
         *hasInstallSettings = installSettingsExist;
@@ -812,7 +810,7 @@ static Utils::optional<QString> currentlyLinkedQtDir(bool *hasInstallSettings)
         const QVariant value = QSettings(installSettingsFilePath, QSettings::IniFormat)
                                    .value(kInstallSettingsKey);
         if (value.isValid())
-            return value.toString();
+            return FilePath::fromVariant(value);
     }
     return {};
 }
@@ -820,18 +818,20 @@ static Utils::optional<QString> currentlyLinkedQtDir(bool *hasInstallSettings)
 static QString linkingPurposeText()
 {
     return QtOptionsPageWidget::tr(
-        "Linking with a Qt installation automatically registers Qt versions and kits.");
+        "Linking with a Qt installation automatically registers Qt versions and kits, and other "
+        "tools that were installed with that Qt installer, in this Qt Creator installation. Other "
+        "Qt Creator installations are not affected.");
 }
 
 static bool canLinkWithQt(QString *toolTip)
 {
     bool canLink = true;
     bool installSettingsExist;
-    const Utils::optional<QString> installSettingsValue = currentlyLinkedQtDir(
+    const Utils::optional<FilePath> installSettingsValue = currentlyLinkedQtDir(
         &installSettingsExist);
     QStringList tip;
     tip << linkingPurposeText();
-    if (!FilePath::fromString(Core::ICore::resourcePath()).isWritablePath()) {
+    if (!Core::ICore::resourcePath().isWritableDir()) {
         canLink = false;
         tip << QtOptionsPageWidget::tr("%1's resource directory is not writable.")
                    .arg(Core::Constants::IDE_DISPLAY_NAME);
@@ -844,10 +844,10 @@ static bool canLinkWithQt(QString *toolTip)
         tip << QtOptionsPageWidget::tr("%1 is part of a Qt installation.")
                    .arg(Core::Constants::IDE_DISPLAY_NAME);
     }
-    const QString link = installSettingsValue ? *installSettingsValue : QString();
+    const FilePath link = installSettingsValue ? *installSettingsValue : FilePath();
     if (!link.isEmpty())
         tip << QtOptionsPageWidget::tr("%1 is currently linked to \"%2\".")
-                   .arg(Core::Constants::IDE_DISPLAY_NAME, link);
+                   .arg(QString(Core::Constants::IDE_DISPLAY_NAME), link.toUserOutput());
     if (toolTip)
         *toolTip = tip.join("\n\n");
     return canLink;
@@ -884,7 +884,7 @@ void QtOptionsPageWidget::apply()
     QtVersionManager::setDocumentationSetting(
         QtVersionManager::DocumentationSetting(m_ui.documentationSetting->currentData().toInt()));
 
-    QList<BaseQtVersion *> versions;
+    QtVersions versions;
     m_model->forItemsAtLevel<2>([&versions](QtVersionItem *item) {
         item->setChanged(false);
         versions.append(item->version()->clone());
@@ -944,11 +944,11 @@ static bool validateQtInstallDir(FancyLineEdit *input, QString *errorString)
     return true;
 }
 
-static QString defaultQtInstallationPath()
+static FilePath defaultQtInstallationPath()
 {
     if (HostOsInfo::isWindowsHost())
-        return {"C:/Qt"};
-    return QDir::homePath() + "/Qt";
+        return FilePath::fromString({"C:/Qt"});
+    return FileUtils::homePath() / "Qt";
 }
 
 void QtOptionsPageWidget::linkWithQt()
@@ -960,7 +960,9 @@ void QtOptionsPageWidget::linkWithQt()
     dialog.setWindowTitle(title);
     auto layout = new QVBoxLayout;
     dialog.setLayout(layout);
-    layout->addWidget(new QLabel(linkingPurposeText()));
+    auto tipLabel = new QLabel(linkingPurposeText());
+    tipLabel->setWordWrap(true);
+    layout->addWidget(tipLabel);
     auto pathLayout = new QHBoxLayout;
     layout->addLayout(pathLayout);
     auto pathLabel = new QLabel(tr("Qt installation path:"));
@@ -979,9 +981,10 @@ void QtOptionsPageWidget::linkWithQt()
             return false;
         return validateQtInstallDir(input, errorString);
     });
-    const Utils::optional<QString> currentLink = currentlyLinkedQtDir(nullptr);
-    pathInput->setPath(currentLink ? *currentLink : defaultQtInstallationPath());
+    const Utils::optional<FilePath> currentLink = currentlyLinkedQtDir(nullptr);
+    pathInput->setFilePath(currentLink ? *currentLink : defaultQtInstallationPath());
     auto buttons = new QDialogButtonBox;
+    layout->addStretch(10);
     layout->addWidget(buttons);
     auto linkButton = buttons->addButton(tr("Link with Qt"), QDialogButtonBox::AcceptRole);
     connect(linkButton, &QPushButton::clicked, &dialog, &QDialog::accept);
@@ -991,7 +994,7 @@ void QtOptionsPageWidget::linkWithQt()
     unlinkButton->setEnabled(currentLink.has_value());
     connect(unlinkButton, &QPushButton::clicked, &dialog, [&dialog, &askForRestart] {
         bool removeSettingsFile = false;
-        const QString filePath = settingsFile(Core::ICore::resourcePath());
+        const QString filePath = settingsFile(Core::ICore::resourcePath().toString());
         {
             QSettings installSettings(filePath, QSettings::IniFormat);
             installSettings.remove(kInstallSettingsKey);
@@ -1010,7 +1013,7 @@ void QtOptionsPageWidget::linkWithQt()
     if (dialog.result() == QDialog::Accepted) {
         const Utils::optional<QString> settingsDir = settingsDirForQtDir(pathInput->rawPath());
         if (QTC_GUARD(settingsDir)) {
-            QSettings(settingsFile(Core::ICore::resourcePath()), QSettings::IniFormat)
+            QSettings(settingsFile(Core::ICore::resourcePath().toString()), QSettings::IniFormat)
                 .setValue(kInstallSettingsKey, *settingsDir);
             askForRestart = true;
         }
