@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -38,7 +33,13 @@
 
 #include "settings.h"
 
-#include <iostream>
+#include <QLoggingCategory>
+
+#ifdef WITH_TESTS
+#include <QTest>
+#endif
+
+Q_LOGGING_CATEGORY(addDeviceLog, "qtc.sdktool.operations.adddevice", QtWarningMsg)
 
 const char DEVICEMANAGER_ID[] = "DeviceManager";
 const char DEFAULT_DEVICES_ID[] = "DefaultDevices";
@@ -48,9 +49,6 @@ const char DEVICE_ID_ID[] = "InternalId";
 
 static const char INTERNAL_DSEKTOP_DEVICE_ID[] = "Desktop Device";
 
-AddDeviceOperation::AddDeviceOperation()
-{ }
-
 QString AddDeviceOperation::name() const
 {
     return QLatin1String("addDev");
@@ -58,13 +56,13 @@ QString AddDeviceOperation::name() const
 
 QString AddDeviceOperation::helpText() const
 {
-    return QLatin1String("add a Device to Qt Creator");
+    return QLatin1String("add a Device");
 }
 
 QString AddDeviceOperation::argumentsHelpText() const
 {
-    return QLatin1String("    --id <ID>                                  id of the new kit (required).\n"
-                         "    --name <NAME>                              display name of the new kit (required).\n"
+    return QLatin1String("    --id <ID>                                  id of the new device (required).\n"
+                         "    --name <NAME>                              display name of the new device (required).\n"
                          "    --type <INT>                               type (required).\n"
                          "    --authentication <INT>                     authentication.\n"
                          "    --b2qHardware <STRING>                     Boot2Qt Platform Info Hardware.\n"
@@ -79,18 +77,14 @@ QString AddDeviceOperation::argumentsHelpText() const
                          "    --sshPort <INT>                            ssh port.\n"
                          "    --timeout <INT>                            timeout.\n"
                          "    --uname <STRING>                           uname.\n"
+                         "    --dockerRepo <STRING>                      Docker image repo.\n"
+                         "    --dockerTag <STRING>                       Docker image tag.\n"
+                         "    --dockerMappedPaths <STRING>               Docker mapped paths (semi-colon separated).\n"
                          "    <KEY> <TYPE:VALUE>                         extra key value pairs\n");
 }
 
 bool AddDeviceOperation::setArguments(const QStringList &args)
 {
-    m_authentication = -1;
-    m_origin = 1;
-    m_sshPort = 0;
-    m_timeout = 5;
-    m_type = -1;
-    m_version = 0;
-
     for (int i = 0; i < args.count(); ++i) {
         const QString current = args.at(i);
         const QString next = ((i + 1) < args.count()) ? args.at(i + 1) : QString();
@@ -239,6 +233,30 @@ bool AddDeviceOperation::setArguments(const QStringList &args)
             continue;
         }
 
+        if (current == QLatin1String("--dockerMappedPaths")) {
+            if (next.isNull())
+                return false;
+            ++i; // skip next;
+            m_dockerMappedPaths = next.split(';');
+            continue;
+        }
+
+        if (current == QLatin1String("--dockerRepo")) {
+            if (next.isNull())
+                return false;
+            ++i; // skip next;
+            m_dockerRepo = next;
+            continue;
+        }
+
+        if (current == QLatin1String("--dockerTag")) {
+            if (next.isNull())
+                return false;
+            ++i; // skip next;
+            m_dockerTag = next;
+            continue;
+        }
+
         if (next.isNull())
             return false;
         ++i; // skip next;
@@ -249,9 +267,9 @@ bool AddDeviceOperation::setArguments(const QStringList &args)
     }
 
     if (m_id.isEmpty())
-        std::cerr << "No id given for device." << std::endl << std::endl;
+        qCCritical(addDeviceLog) << "No id given for device.";
     if (m_displayName.isEmpty())
-        std::cerr << "No name given for device." << std::endl << std::endl;
+        qCCritical(addDeviceLog) << "No name given for device.";
 
     return !m_id.isEmpty() && !m_displayName.isEmpty() && m_type >= 0;
 }
@@ -262,10 +280,7 @@ int AddDeviceOperation::execute() const
     if (map.isEmpty())
         map = initializeDevices();
 
-    QVariantMap result = addDevice(map, m_id, m_displayName, m_type, m_authentication,
-                                   m_b2q_platformHardware, m_b2q_platformSoftware, m_debugServer,
-                                   m_freePortsSpec, m_host, m_keyFile, m_origin, m_osType,
-                                   m_password, m_sshPort, m_timeout, m_uname, m_version, m_extra);
+    QVariantMap result = addDevice(map);
 
     if (result.isEmpty() || map == result)
         return 2;
@@ -274,83 +289,97 @@ int AddDeviceOperation::execute() const
 }
 
 #ifdef WITH_TESTS
-bool AddDeviceOperation::test() const
+void AddDeviceOperation::unittest()
 {
     QVariantMap map = initializeDevices();
 
-    QVariantMap result = addDevice(map, QLatin1String("test id"), QLatin1String("test name"),
-                                   1, 2, QLatin1String("HW"), QLatin1String("SW"),
-                                   QLatin1String("debugServer"), QLatin1String("ports"),
-                                   QLatin1String("host"), QLatin1String("keyfile"), 3,
-                                   QLatin1String("ostype"), QLatin1String("passwd"), 4, 5,
-                                   QLatin1String("uname"), 6, KeyValuePairList());
+    AddDeviceData devData;
+    devData.m_id = "test id";
+    devData.m_displayName = "test name";
+    devData.m_type = 1;
+    devData.m_authentication = 2;
+    devData.m_b2q_platformHardware = "HW";
+    devData.m_b2q_platformSoftware = "SW";
+    devData.m_debugServer = "debugServer";
+    devData.m_freePortsSpec = "ports";
+    devData.m_host = "host";
+    devData.m_keyFile = "keyfile";
+    devData.m_origin = 3;
+    devData.m_osType = "ostype";
+    devData.m_password = "passwd";
+    devData.m_sshPort = 4;
+    devData.m_timeout = 5;
+    devData.m_uname = "uname";
+    devData.m_version = 6;
+    devData.m_dockerMappedPaths = QStringList{"/opt", "/data"};
+    devData.m_dockerRepo = "repo";
+    devData.m_dockerTag = "tag";
 
+
+    QVariantMap result = devData.addDevice(map);
     QVariantMap data = result.value(QLatin1String(DEVICEMANAGER_ID)).toMap();
     QVariantList devList = data.value(QLatin1String(DEVICE_LIST_ID)).toList();
-    if (devList.count() != 1)
-        return false;
+    QCOMPARE(devList.count(), 1);
     QVariantMap dev = devList.at(0).toMap();
-    if (dev.count() != 17)
-        return false;
-    if (dev.value(QLatin1String("Authentication")).toInt() != 2)
-        return false;
-    if (dev.value(QLatin1String("DebugServerKey")).toString() != QLatin1String("debugServer"))
-        return false;
-    if (dev.value(QLatin1String("FreePortsSpec")).toString() != QLatin1String("ports"))
-        return false;
-    if (dev.value(QLatin1String("Host")).toString() != QLatin1String("host"))
-        return false;
-    if (dev.value(QLatin1String("InternalId")).toString() != QLatin1String("test id"))
-        return false;
-    if (dev.value(QLatin1String("KeyFile")).toString() != QLatin1String("keyfile"))
-        return false;
-    if (dev.value(QLatin1String("Name")).toString() != QLatin1String("test name"))
-        return false;
-    if (dev.value(QLatin1String("Origin")).toInt() != 3)
-        return false;
-    if (dev.value(QLatin1String("OsType")).toString() != QLatin1String("ostype"))
-        return false;
-    if (dev.value(QLatin1String("Password")).toString() != QLatin1String("passwd"))
-        return false;
-    if (dev.value(QLatin1String("SshPort")).toInt() != 4)
-        return false;
-    if (dev.value(QLatin1String("Timeout")).toInt() != 5)
-        return false;
-    if (dev.value(QLatin1String("Type")).toInt() != 1)
-        return false;
-    if (dev.value(QLatin1String("Uname")).toString() != QLatin1String("uname"))
-        return false;
-    if (dev.value(QLatin1String("Version")).toInt() != 6)
-        return false;
+    QCOMPARE(dev.count(), 20);
+    QCOMPARE(dev.value(QLatin1String("Authentication")).toInt(), 2);
+    QCOMPARE(dev.value(QLatin1String("DebugServerKey")).toString(), QLatin1String("debugServer"));
+    QCOMPARE(dev.value(QLatin1String("FreePortsSpec")).toString(), QLatin1String("ports"));
+    QCOMPARE(dev.value(QLatin1String("Host")).toString(), QLatin1String("host"));
+    QCOMPARE(dev.value(QLatin1String("InternalId")).toString(), QLatin1String("test id"));
+    QCOMPARE(dev.value(QLatin1String("KeyFile")).toString(), QLatin1String("keyfile"));
+    QCOMPARE(dev.value(QLatin1String("Name")).toString(), QLatin1String("test name"));
+    QCOMPARE(dev.value(QLatin1String("Origin")).toInt(), 3);
+    QCOMPARE(dev.value(QLatin1String("OsType")).toString(), QLatin1String("ostype"));
+    QCOMPARE(dev.value(QLatin1String("Password")).toString(), QLatin1String("passwd"));
+    QCOMPARE(dev.value(QLatin1String("SshPort")).toInt(), 4);
+    QCOMPARE(dev.value(QLatin1String("Timeout")).toInt(), 5);
+    QCOMPARE(dev.value(QLatin1String("Type")).toInt(), 1);
+    QCOMPARE(dev.value(QLatin1String("Uname")).toString(), QLatin1String("uname"));
+    QCOMPARE(dev.value(QLatin1String("Version")).toInt(), 6);
+    QCOMPARE(dev.value(QLatin1String("DockerDeviceDataRepo")).toString(), "repo");
+    QCOMPARE(dev.value(QLatin1String("DockerDeviceDataTag")).toString(), "tag");
 
-    return true;
+    const QStringList paths = dev.value(QLatin1String("DockerDeviceMappedPaths")).toStringList();
+    QCOMPARE(paths, QStringList({"/opt", "/data"}));
 }
 #endif
 
-QVariantMap AddDeviceOperation::addDevice(const QVariantMap &map,
-                                          const QString &id, const QString &displayName, int type,
-                                          int auth, const QString &hwPlatform, const QString &swPlatform,
-                                          const QString &debugServer, const QString &freePorts,
-                                          const QString &host, const QString &keyFile,
-                                          int origin, const QString &osType, const QString &passwd,
-                                          int sshPort, int timeout, const QString &uname, int version,
-                                          const KeyValuePairList &extra)
+QVariantMap AddDeviceData::addDevice(const QVariantMap &map) const
 {
     QVariantMap result = map;
-    if (exists(map, id)) {
-        std::cerr << "Device " << qPrintable(id) << " already exists!" << std::endl;
+    if (exists(map, m_id)) {
+        qCCritical(addDeviceLog) << "Device " << qPrintable(m_id) << " already exists!";
         return result;
     }
 
     QVariantMap dmMap = map.value(QLatin1String(DEVICEMANAGER_ID)).toMap();
     QVariantList devList = dmMap.value(QLatin1String(DEVICE_LIST_ID)).toList();
 
-    QVariantMap devMap
-            = AddKeysOperation::addKeys(QVariantMap(),
-                                        createDevice(id, displayName, type, auth, hwPlatform,
-                                                     swPlatform, debugServer, freePorts, host,
-                                                     keyFile, origin, osType, passwd, sshPort,
-                                                     timeout, uname, version, extra));
+    KeyValuePairList dev;
+    dev.append(KeyValuePair(QLatin1String(DEVICE_ID_ID), QVariant(m_id)));
+    dev.append(KeyValuePair(QLatin1String("Name"), QVariant(m_displayName)));
+    dev.append(KeyValuePair(QLatin1String("Type"), QVariant(m_type)));
+    dev.append(KeyValuePair(QLatin1String("Authentication"), QVariant(m_authentication)));
+    dev.append(KeyValuePair(QLatin1String("Boot2Qt.PlatformInfoHardware"), QVariant(m_b2q_platformHardware)));
+    dev.append(KeyValuePair(QLatin1String("Boot2Qt.PlatformInfoSoftware"), QVariant(m_b2q_platformSoftware)));
+    dev.append(KeyValuePair(QLatin1String("DebugServerKey"), QVariant(m_debugServer)));
+    dev.append(KeyValuePair(QLatin1String("FreePortsSpec"), QVariant(m_freePortsSpec)));
+    dev.append(KeyValuePair(QLatin1String("Host"), QVariant(m_host)));
+    dev.append(KeyValuePair(QLatin1String("KeyFile"), QVariant(m_keyFile)));
+    dev.append(KeyValuePair(QLatin1String("Origin"), QVariant(m_origin)));
+    dev.append(KeyValuePair(QLatin1String("OsType"), QVariant(m_osType)));
+    dev.append(KeyValuePair(QLatin1String("Password"), QVariant(m_password)));
+    dev.append(KeyValuePair(QLatin1String("SshPort"), QVariant(m_sshPort)));
+    dev.append(KeyValuePair(QLatin1String("Timeout"), QVariant(m_timeout)));
+    dev.append(KeyValuePair(QLatin1String("Uname"), QVariant(m_uname)));
+    dev.append(KeyValuePair(QLatin1String("Version"), QVariant(m_version)));
+    dev.append(KeyValuePair(QLatin1String("DockerDeviceMappedPaths"), QVariant(m_dockerMappedPaths)));
+    dev.append(KeyValuePair(QLatin1String("DockerDeviceDataRepo"), QVariant(m_dockerRepo)));
+    dev.append(KeyValuePair(QLatin1String("DockerDeviceDataTag"), QVariant(m_dockerTag)));
+    dev.append(m_extra);
+
+    QVariantMap devMap = AddKeysData{dev}.addKeys(QVariantMap());
 
     devList.append(devMap);
 
@@ -361,7 +390,7 @@ QVariantMap AddDeviceOperation::addDevice(const QVariantMap &map,
     return result;
 }
 
-QVariantMap AddDeviceOperation::initializeDevices()
+QVariantMap AddDeviceData::initializeDevices()
 {
     QVariantMap dmData;
     dmData.insert(QLatin1String(DEFAULT_DEVICES_ID), QVariantMap());
@@ -372,19 +401,19 @@ QVariantMap AddDeviceOperation::initializeDevices()
     return data;
 }
 
-bool AddDeviceOperation::exists(const QString &id)
+bool AddDeviceData::exists(const QString &id)
 {
-    QVariantMap map = load(QLatin1String("Devices"));
+    QVariantMap map = Operation::load(QLatin1String("Devices"));
     return exists(map, id);
 }
 
-bool AddDeviceOperation::exists(const QVariantMap &map, const QString &id)
+bool AddDeviceData::exists(const QVariantMap &map, const QString &id)
 {
     if (id == QLatin1String(INTERNAL_DSEKTOP_DEVICE_ID))
         return true;
     QVariantMap dmMap = map.value(QLatin1String(DEVICEMANAGER_ID)).toMap();
-    QVariantList devList = dmMap.value(QLatin1String(DEVICE_LIST_ID)).toList();
-    foreach (const QVariant &dev, devList) {
+    const QVariantList devList = dmMap.value(QLatin1String(DEVICE_LIST_ID)).toList();
+    for (const QVariant &dev : devList) {
         QVariantMap devData = dev.toMap();
         QString current = devData.value(QLatin1String(DEVICE_ID_ID)).toString();
         if (current == id)
@@ -392,37 +421,3 @@ bool AddDeviceOperation::exists(const QVariantMap &map, const QString &id)
     }
     return false;
 }
-
-Operation::KeyValuePairList AddDeviceOperation::createDevice(const QString &id, const QString &displayName,
-                                                             int type, int auth, const QString &hwPlatform,
-                                                             const QString &swPlatform, const QString &debugServer,
-                                                             const QString &freePorts, const QString &host,
-                                                             const QString &keyFile, int origin,
-                                                             const QString &osType, const QString &passwd,
-                                                             int sshPort, int timeout, const QString &uname,
-                                                             int version, const Operation::KeyValuePairList &extra)
-{
-    Operation::KeyValuePairList dev;
-    dev.append(KeyValuePair(QLatin1String(DEVICE_ID_ID), QVariant(id)));
-    dev.append(KeyValuePair(QLatin1String("Name"), QVariant(displayName)));
-    dev.append(KeyValuePair(QLatin1String("Type"), QVariant(type)));
-
-    dev.append(KeyValuePair(QLatin1String("Authentication"), QVariant(auth)));
-    dev.append(KeyValuePair(QLatin1String("Boot2Qt.PlatformInfoHardware"), QVariant(hwPlatform)));
-    dev.append(KeyValuePair(QLatin1String("Boot2Qt.PlatformInfoSoftware"), QVariant(swPlatform)));
-    dev.append(KeyValuePair(QLatin1String("DebugServerKey"), QVariant(debugServer)));
-    dev.append(KeyValuePair(QLatin1String("FreePortsSpec"), QVariant(freePorts)));
-    dev.append(KeyValuePair(QLatin1String("Host"), QVariant(host)));
-    dev.append(KeyValuePair(QLatin1String("KeyFile"), QVariant(keyFile)));
-    dev.append(KeyValuePair(QLatin1String("Origin"), QVariant(origin)));
-    dev.append(KeyValuePair(QLatin1String("OsType"), QVariant(osType)));
-    dev.append(KeyValuePair(QLatin1String("Password"), QVariant(passwd)));
-    dev.append(KeyValuePair(QLatin1String("SshPort"), QVariant(sshPort)));
-    dev.append(KeyValuePair(QLatin1String("Timeout"), QVariant(timeout)));
-    dev.append(KeyValuePair(QLatin1String("Uname"), QVariant(uname)));
-    dev.append(KeyValuePair(QLatin1String("Version"), QVariant(version)));
-
-    dev.append(extra);
-
-    return dev;
-}
