@@ -210,7 +210,7 @@ void tst_Semantic::function_declaration_1()
     QCOMPARE(funTy->argumentCount(), 0);
     QCOMPARE(funTy->refQualifier(), Function::NoRefQualifier);
 
-    QVERIFY(decl->name()->isNameId());
+    QVERIFY(decl->name()->asNameId());
     const Identifier *funId = decl->name()->asNameId()->identifier();
     QVERIFY(funId);
 
@@ -261,7 +261,7 @@ void tst_Semantic::function_declaration_2()
     QCOMPARE(QByteArray(namedTypeId->chars(), namedTypeId->size()),
              QByteArray("QString"));
 
-    QVERIFY(decl->name()->isNameId());
+    QVERIFY(decl->name()->asNameId());
     const Identifier *funId = decl->name()->asNameId()->identifier();
     QVERIFY(funId);
 
@@ -378,7 +378,7 @@ void tst_Semantic::function_definition_1()
     QCOMPARE(funTy->argumentCount(), 0);
     QCOMPARE(funTy->refQualifier(), Function::NoRefQualifier);
 
-    QVERIFY(funTy->name()->isNameId());
+    QVERIFY(funTy->name()->asNameId());
     const Identifier *funId = funTy->name()->asNameId()->identifier();
     QVERIFY(funId);
 
@@ -612,7 +612,7 @@ void tst_Semantic::template_instance_1()
     Declaration *decl = templ->memberAt(1)->asClass()->memberAt(0)->asDeclaration();
     QVERIFY(decl);
 
-    FullySpecifiedType templArgs[] = {control->integerType(IntegerType::Int)};
+    TemplateArgument templArgs[] = {FullySpecifiedType(control->integerType(IntegerType::Int))};
     const Name *templId = control->templateNameId(control->identifier("QList"), false, templArgs, 1);
 
     FullySpecifiedType genTy = DeprecatedGenTemplateInstance::instantiate(templId, decl, control);
@@ -1041,5 +1041,5 @@ void tst_Semantic::enum_constantValueNegative()
     testEnumaratorDeclarator(e, 3, "1");
 }
 
-QTEST_MAIN(tst_Semantic)
+QTEST_GUILESS_MAIN(tst_Semantic)
 #include "tst_semantic.moc"
