@@ -1,6 +1,6 @@
 ############################################################################
 #
-# Copyright (C) 2016 The Qt Company Ltd.
+# Copyright (C) 2022 The Qt Company Ltd.
 # Contact: https://www.qt.io/licensing/
 #
 # This file is part of Qt Creator.
@@ -76,7 +76,7 @@ def ensureChecked(objectName, shouldBeChecked = True, timeout=20000):
 #                   or the object itself. If it is an object, it must exist already.
 # param expectedState is the expected enable state of the object
 def verifyEnabled(objectSpec, expectedState = True):
-    if isinstance(objectSpec, (str, unicode)):
+    if isString(objectSpec):
         waitFor("object.exists('" + str(objectSpec).replace("'", "\\'") + "')", 20000)
         foundObject = findObject(objectSpec)
     else:
@@ -154,7 +154,7 @@ def which(program):
 def cleanUpUserFiles(pathsToProFiles=None):
     if pathsToProFiles==None:
         return False
-    if isinstance(pathsToProFiles, (str, unicode)):
+    if isString(pathsToProFiles):
         filelist = glob.glob(pathsToProFiles+".user*")
     elif isinstance(pathsToProFiles, (list, tuple)):
         filelist = []
@@ -172,6 +172,7 @@ def cleanUpUserFiles(pathsToProFiles=None):
             doneWithoutErrors = False
     return doneWithoutErrors
 
+
 def invokeMenuItem(menu, item, *subItems):
     if platform.system() == "Darwin":
         try:
@@ -179,7 +180,7 @@ def invokeMenuItem(menu, item, *subItems):
         except:
             nativeMouseClick(waitForObject(":Qt Creator_Core::Internal::MainWindow", 1000), 20, 20, 0, Qt.LeftButton)
     # Use Locator for menu items which wouldn't work on macOS
-    if menu == "Tools" and item == "Options..." or menu == "File" and item == "Exit":
+    if menu == "Edit" and item == "Preferences..." or menu == "File" and item == "Exit":
         selectFromLocator("t %s" % item, item)
         return
     menuObject = waitForObjectItem(":Qt Creator.QtCreator.MenuBar_QMenuBar", menu)
@@ -189,31 +190,28 @@ def invokeMenuItem(menu, item, *subItems):
     itemObject = waitForObjectItem(objectMap.realName(menuObject), item)
     waitFor("itemObject.enabled", 2000)
     activateItem(itemObject)
-    numberedPrefix = "(&\\d \| )?"
+    numberedPrefix = "%d | "
     for subItem in subItems:
-        sub = itemObject.menu()
-        waitFor("sub.visible", 1000)
         # we might have numbered sub items (e.g. "Recent Files") - these have this special prefix
         if subItem.startswith(numberedPrefix):
-            actions = sub.actions()
             triggered = False
-            for i in range(actions.count()):
-                current = actions.at(i)
-                nonPrefix = subItem[len(numberedPrefix):]
-                matcher = re.match("%s(.*)" % numberedPrefix, str(current.text))
-                if matcher and matcher.group(2) == nonPrefix:
-                    itemObject = current
+            for i in range(1, 10):
+                try:
+                    itemObject = waitForObjectItem(itemObject, subItem % i, 1000)
                     activateItem(itemObject)
                     triggered = True
                     break
+                except:
+                    continue
             if not triggered:
                 test.fail("Could not trigger '%s' - item missing or code wrong?" % subItem,
                           "Function arguments: '%s', '%s', %s" % (menu, item, str(subItems)))
                 break # we failed to trigger - no need to process subItems further
         else:
-            itemObject = waitForObjectItem(sub, subItem)
+            itemObject = waitForObjectItem(itemObject, subItem)
             activateItem(itemObject)
 
+
 def logApplicationOutput():
     # make sure application output is shown
     ensureChecked(":Qt Creator_AppOutput_Core::Internal::OutputPaneToggleButton")
@@ -228,11 +226,11 @@ def logApplicationOutput():
 # get the output from a given cmdline call
 def getOutputFromCmdline(cmdline, environment=None, acceptedError=0):
     try:
-        return subprocess.check_output(cmdline, env=environment)
+        return stringify(subprocess.check_output(cmdline, env=environment))
     except subprocess.CalledProcessError as e:
         if e.returncode != acceptedError:
             test.warning("Command '%s' returned %d" % (e.cmd, e.returncode))
-        return e.output
+        return stringify(e.output)
 
 def selectFromFileDialog(fileName, waitForFile=False, ignoreFinalSnooze=False):
     def __closePopupIfNecessary__():
@@ -240,41 +238,29 @@ def selectFromFileDialog(fileName, waitForFile=False, ignoreFinalSnooze=False):
             test.log("Closing active popup widget")
             QApplication.activePopupWidget().close()
 
-    if platform.system() == "Darwin":
+    fName = os.path.basename(os.path.abspath(fileName))
+    pName = os.path.dirname(os.path.abspath(fileName)) + os.sep
+    try:
+        waitForObject("{name='QFileDialog' type='QFileDialog' visible='1'}", 5000)
+        pathLine = waitForObject("{name='fileNameEdit' type='QLineEdit' visible='1'}")
+        replaceEditorContent(pathLine, pName)
         snooze(1)
-        nativeType("<Command+Shift+g>")
+        clickButton(waitForObject("{text='Open' type='QPushButton'}"))
+        waitFor("str(pathLine.text)==''")
+        replaceEditorContent(pathLine, fName)
         snooze(1)
-        nativeType(fileName)
-        snooze(2)
-        nativeType("<Return>")
-        snooze(3)
+        __closePopupIfNecessary__()
+        clickButton(waitForObject("{text='Open' type='QPushButton'}"))
+    except:
+        nativeType("<Ctrl+a>")
+        nativeType("<Delete>")
+        nativeType(pName + fName)
+        seconds = len(pName + fName) / 20
+        test.log("Using snooze(%d) [problems with event processing of nativeType()]" % seconds)
+        snooze(seconds)
         nativeType("<Return>")
         if not ignoreFinalSnooze:
-            snooze(1)
-    else:
-        fName = os.path.basename(os.path.abspath(fileName))
-        pName = os.path.dirname(os.path.abspath(fileName)) + os.sep
-        try:
-            waitForObject("{name='QFileDialog' type='QFileDialog' visible='1'}", 5000)
-            pathLine = waitForObject("{name='fileNameEdit' type='QLineEdit' visible='1'}")
-            replaceEditorContent(pathLine, pName)
-            snooze(1)
-            clickButton(waitForObject("{text='Open' type='QPushButton'}"))
-            waitFor("str(pathLine.text)==''")
-            replaceEditorContent(pathLine, fName)
-            snooze(1)
-            __closePopupIfNecessary__()
-            clickButton(waitForObject("{text='Open' type='QPushButton'}"))
-        except:
-            nativeType("<Ctrl+a>")
-            nativeType("<Delete>")
-            nativeType(pName + fName)
-            seconds = len(pName + fName) / 20
-            test.log("Using snooze(%d) [problems with event processing of nativeType()]" % seconds)
-            snooze(seconds)
-            nativeType("<Return>")
-            if not ignoreFinalSnooze:
-                snooze(3)
+            snooze(3)
     if waitForFile:
         fileCombo = waitForObject(":Qt Creator_FilenameQComboBox")
         if not waitFor("str(fileCombo.currentText) in fileName", 5000):
@@ -283,7 +269,7 @@ def selectFromFileDialog(fileName, waitForFile=False, ignoreFinalSnooze=False):
 # add Qt documentations from given paths
 # param which a list/tuple of the paths to the qch files to be added
 def addHelpDocumentation(which):
-    invokeMenuItem("Tools", "Options...")
+    invokeMenuItem("Edit", "Preferences...")
     mouseClick(waitForObjectItem(":Options_QListView", "Help"))
     waitForObject("{container=':Options.qt_tabwidget_tabbar_QTabBar' type='TabItem' text='Documentation'}")
     clickOnTab(":Options.qt_tabwidget_tabbar_QTabBar", "Documentation")
@@ -309,7 +295,7 @@ def addCurrentCreatorDocumentation():
     if not os.path.exists(docPath):
         test.fatal("Missing current Qt Creator documentation (expected in %s)" % docPath)
         return
-    invokeMenuItem("Tools", "Options...")
+    invokeMenuItem("Edit", "Preferences...")
     mouseClick(waitForObjectItem(":Options_QListView", "Help"))
     waitForObject("{container=':Options.qt_tabwidget_tabbar_QTabBar' type='TabItem' text='Documentation'}")
     clickOnTab(":Options.qt_tabwidget_tabbar_QTabBar", "Documentation")
@@ -317,7 +303,7 @@ def addCurrentCreatorDocumentation():
     selectFromFileDialog(docPath)
     try:
         waitForObject("{type='QMessageBox' unnamed='1' visible='1' "
-                      "text?='Unable to register documentation.*'}", 3000)
+                      "text~='Unable to register documentation.*'}", 3000)
         test.passes("Qt Creator's documentation found already registered.")
         clickButton(waitForObject("{type='QPushButton' text='OK' unnamed='1' visible='1' "
                                   "container={name='groupBox' type='QGroupBox' visible='1'}}"))
@@ -361,29 +347,25 @@ def __checkParentAccess__(filePath):
 # options dialog and returns a dict holding the kits as keys
 # and a list of information of its configured Qt
 def getConfiguredKits():
-    def __retrieveQtVersionName__(target, version):
-        treeView = waitForObject(":qtdirList_QTreeView")
-        return str(treeView.currentIndex().data().toString())
-    # end of internal function for iterateQtVersions
     def __setQtVersionForKit__(kit, kitName, kitsQtVersionName):
         mouseClick(waitForObjectItem(":BuildAndRun_QTreeView", kit))
         qtVersionStr = str(waitForObjectExists(":Kits_QtVersion_QComboBox").currentText)
+        invalid = qtVersionStr.endswith(" (invalid)")
+        if invalid:
+            qtVersionStr = qtVersionStr[:-10]
         kitsQtVersionName[kitName] = qtVersionStr
     # end of internal function for iterate kits
 
     kitsWithQtVersionName = {}
-    result = {}
+    result = []
     # collect kits and their Qt versions
-    targetsQtVersions, qtVersionNames = iterateQtVersions(True, False, __retrieveQtVersionName__)
+    qtVersionNames = iterateQtVersions()
     # update collected Qt versions with their configured device and version
-    iterateKits(True, True, __setQtVersionForKit__, kitsWithQtVersionName)
+    iterateKits(False, True, __setQtVersionForKit__, kitsWithQtVersionName)
     # merge defined target names with their configured Qt versions and devices
     for kit, qtVersion in kitsWithQtVersionName.iteritems():
-        if kit in ('Fremantle', 'Harmattan', 'Qt Simulator'):
-            test.verify(qtVersion == 'None',
-                        "The outdated kit '%s' should not have a Qt version" % kit)
-        elif qtVersion in qtVersionNames:
-            result[kit] = targetsQtVersions[qtVersionNames.index(qtVersion)].items()[0]
+        if qtVersion in qtVersionNames:
+            result.append(kit)
         else:
             test.fail("Qt version '%s' for kit '%s' can't be found in qtVersionNames."
                       % (qtVersion, kit))
@@ -400,10 +382,10 @@ def enabledCheckBoxExists(text):
 
 # this function verifies if the text matches the given
 # regex inside expectedTexts
-# param text must be a single str/unicode
-# param expectedTexts can be str/unicode/list/tuple
+# param text must be a single str
+# param expectedTexts can be str/list/tuple
 def regexVerify(text, expectedTexts):
-    if isinstance(expectedTexts, (str,unicode)):
+    if isString(expectedTexts):
         expectedTexts = [expectedTexts]
     for curr in expectedTexts:
         pattern = re.compile(curr)
@@ -411,91 +393,48 @@ def regexVerify(text, expectedTexts):
             return True
     return False
 
+
 # function that opens Options Dialog and parses the configured Qt versions
-# param keepOptionsOpen set to True if the Options dialog should stay open when
-#       leaving this function
-# param alreadyOnOptionsDialog set to True if you already have opened the Options Dialog
-#       (if False this function will open it via the MenuBar -> Tools -> Options...)
-# param additionalFunction pass a function or name of a defined function to execute
-#       for each correctly configured item on the list of Qt versions
-#       (Qt versions having no assigned toolchain, failing qmake,... will be skipped)
-#       this function must take at least 2 parameters - the first is the target name
-#       and the second the version of the current selected Qt version item
-# param argsForAdditionalFunc you can specify as much parameters as you want to pass
-#       to additionalFunction from the outside
-# the function returns a list of dict holding target-version mappings if used without
-# additionalFunction
-# WATCH OUT! if you're using the additionalFunction parameter - this function will
-# return the list mentioned above as well as the returned value(s) from
-# additionalFunction. You MUST call this function like
-# result, additionalResult = _iterateQtVersions(...)
-# where additionalResult is the result of all executions of additionalFunction which
-# means it is a list of results.
-def iterateQtVersions(keepOptionsOpen=False, alreadyOnOptionsDialog=False,
-                      additionalFunction=None, *argsForAdditionalFunc):
-    result = []
-    additionalResult = []
-    if not alreadyOnOptionsDialog:
-        invokeMenuItem("Tools", "Options...")
+# the function returns a list of the found Qt versions
+def iterateQtVersions():
+    qtVersionNames = []
+    invokeMenuItem("Edit", "Preferences...")
     mouseClick(waitForObjectItem(":Options_QListView", "Kits"))
     clickOnTab(":Options.qt_tabwidget_tabbar_QTabBar", "Qt Versions")
-    pattern = re.compile("Qt version (?P<version>.*?) for (?P<target>.*)")
     treeView = waitForObject(":qtdirList_QTreeView")
     model = treeView.model()
     for rootIndex in dumpIndices(model):
         rootChildText = str(rootIndex.data()).replace(".", "\\.").replace("_", "\\_")
         for subIndex in dumpIndices(model, rootIndex):
             subChildText = str(subIndex.data()).replace(".", "\\.").replace("_", "\\_")
-            mouseClick(waitForObjectItem(treeView, ".".join([rootChildText,subChildText])))
-            currentText = str(waitForObject(":QtSupport__Internal__QtVersionManager.QLabel").text)
-            matches = pattern.match(currentText)
-            if matches:
-                target = matches.group("target").strip()
-                version = matches.group("version").strip()
-                result.append({target:version})
-                if additionalFunction:
-                    try:
-                        if isinstance(additionalFunction, (str, unicode)):
-                            currResult = globals()[additionalFunction](target, version, *argsForAdditionalFunc)
-                        else:
-                            currResult = additionalFunction(target, version, *argsForAdditionalFunc)
-                    except:
-                        t,v,_ = sys.exc_info()
-                        currResult = None
-                        test.fatal("Function to additionally execute on Options Dialog could not be found or "
-                                   "an exception occurred while executing it.", "%s(%s)" % (str(t), str(v)))
-                    additionalResult.append(currResult)
-    if not keepOptionsOpen:
-        clickButton(waitForObject(":Options.Cancel_QPushButton"))
-    if additionalFunction:
-        return result, additionalResult
-    else:
-        return result
+            treeView.scrollTo(subIndex)
+            mouseClick(waitForObjectItem(treeView, ".".join([rootChildText, subChildText])))
+            qtVersionNames.append(str(treeView.currentIndex().data().toString()))
+    return qtVersionNames
+
 
 # function that opens Options Dialog (if necessary) and parses the configured Kits
-# param keepOptionsOpen set to True if the Options dialog should stay open when
-#       leaving this function
+# param clickOkWhenDone set to True if the Options dialog should be closed by clicking the
+#       "OK" button. If False, the dialog will stay open
 # param alreadyOnOptionsDialog set to True if you already have opened the Options Dialog
-#       (if False this functions will open it via the MenuBar -> Tools -> Options...)
+#       (if False this function will open it via the MenuBar -> Edit -> Preferences...)
 # param additionalFunction pass a function or name of a defined function to execute
 #       for each configured item on the list of Kits
-#       this function must take at least 2 parameters - the first is the item (QModelIndex)
-#       of the current Kit (if you need to click on it) and the second the Kit name itself
-# param argsForAdditionalFunc you can specify as much parameters as you want to pass
+#       this function must take at least 2 parameters - the first is the full string
+#       of the kit which can be used in waitForObjectItem(), the second the Kit name itself
+# param argsForAdditionalFunc you can specify as many parameters as you want to pass
 #       to additionalFunction from the outside
-# the function returns a list of Kit names if used without an additional function
-# WATCH OUT! if you're using the additionalFunction parameter - this function will
-# return the list mentioned above as well as the returned value(s) from
+# this function will return a list of Kit names as well as the returned value(s) from
 # additionalFunction. You MUST call this function like
-# result, additionalResult = _iterateQtVersions(...)
+# result, additionalResult = iterateKits(...)
 # where additionalResult is the result of all executions of additionalFunction which
 # means it is a list of results.
-def iterateKits(keepOptionsOpen=False, alreadyOnOptionsDialog=False,
-                additionalFunction=None, *argsForAdditionalFunc):
+def iterateKits(clickOkWhenDone, alreadyOnOptionsDialog,
+                additionalFunction, *argsForAdditionalFunc):
     result = []
     additionalResult = []
     if not alreadyOnOptionsDialog:
-        invokeMenuItem("Tools", "Options...")
+        invokeMenuItem("Edit", "Preferences...")
     mouseClick(waitForObjectItem(":Options_QListView", "Kits"))
     clickOnTab(":Options.qt_tabwidget_tabbar_QTabBar", "Kits")
     treeView = waitForObject(":BuildAndRun_QTreeView")
@@ -508,31 +447,22 @@ def iterateKits(keepOptionsOpen=False, alreadyOnOptionsDialog=False,
     test.compare(manual.data().toString(), "Manual", "Verifying label for target section")
     for section in [autoDetected, manual]:
         for currentItem in dumpItems(model, section):
-            kitName = currentItem
-            if (kitName.endswith(" (default)")):
-                kitName = kitName.rsplit(" (default)", 1)[0]
+            kitName = currentItem.rsplit(" (default)", 1)[0]
             result.append(kitName)
-            item = ".".join([str(section.data().toString()),
-                             currentItem.replace(".", "\\.")])
-            if additionalFunction:
-                try:
-                    if isinstance(additionalFunction, (str, unicode)):
-                        currResult = globals()[additionalFunction](item, kitName, *argsForAdditionalFunc)
-                    else:
-                        currResult = additionalFunction(item, kitName, *argsForAdditionalFunc)
-                except:
-                    t,v,_ = sys.exc_info()
-                    currResult = None
-                    test.fatal("Function to additionally execute on Options Dialog could not be "
-                               "found or an exception occurred while executing it.", "%s(%s)" %
-                               (str(t), str(v)))
-                additionalResult.append(currResult)
-    if not keepOptionsOpen:
-        clickButton(waitForObject(":Options.Cancel_QPushButton"))
-    if additionalFunction:
-        return result, additionalResult
-    else:
-        return result
+            try:
+                item = ".".join([str(section.data().toString()),
+                                 currentItem.replace(".", "\\.")])
+                currResult = additionalFunction(item, kitName, *argsForAdditionalFunc)
+            except:
+                t, v, _ = sys.exc_info()
+                currResult = None
+                test.fatal("Function to additionally execute on Options Dialog could not be "
+                           "found or an exception occurred while executing it.", "%s(%s)" %
+                           (str(t), str(v)))
+            additionalResult.append(currResult)
+    if clickOkWhenDone:
+        clickButton(waitForObject(":Options.OK_QPushButton"))
+    return result, additionalResult
 
 # set a help viewer that will always be used, regardless of Creator's width
 
@@ -540,7 +470,7 @@ class HelpViewer:
     HELPMODE, SIDEBYSIDE, EXTERNALWINDOW = range(3)
 
 def setFixedHelpViewer(helpViewer):
-    invokeMenuItem("Tools", "Options...")
+    invokeMenuItem("Edit", "Preferences...")
     mouseClick(waitForObjectItem(":Options_QListView", "Help"))
     clickOnTab(":Options.qt_tabwidget_tabbar_QTabBar", "General")
     mode = "Always Show "
@@ -579,7 +509,7 @@ def dumpChildren(item):
 
 def writeTestResults(folder):
     if not os.path.exists(folder):
-        print "Skipping writing test results (folder '%s' does not exist)." % folder
+        print("Skipping writing test results (folder '%s' does not exist)." % folder)
         return
     resultFile = open("%s.srf" % os.path.join(folder, os.path.basename(squishinfo.testCase)), "w")
     resultFile.write("suite:%s\n" % os.path.basename(os.path.dirname(squishinfo.testCase)))
@@ -619,8 +549,8 @@ def clickOnTab(tabBarStr, tabText, timeout=5000):
     if not waitFor("object.exists(tabBarStr)", timeout):
         raise LookupError("Could not find QTabBar: %s" % objectMap.realName(tabBarStr))
     tabBar = findObject(tabBarStr)
-    if platform.system() == 'Darwin' and not tabBar.visible:
-        test.log("Using workaround for Mac.")
+    if not (platform.system() == 'Linux' or tabBar.visible):
+        test.log("Using workaround for Mac and Windows.")
         setWindowState(tabBar, WindowState.Normal)
         tabBar = waitForObject(tabBarStr, 2000)
     clickTab(tabBar, tabText)
@@ -652,11 +582,11 @@ def openVcsLog():
         waitForObject("{text='Version Control' type='QLabel' unnamed='1' visible='1' "
                       "window=':Qt Creator_Core::Internal::MainWindow'}", 2000)
     except:
-        invokeMenuItem("View", "Output Panes", "Version Control")
+        invokeMenuItem("View", "Output", "Version Control")
 
 def openGeneralMessages():
     if not object.exists(":Qt Creator_Core::OutputWindow"):
-        invokeMenuItem("View", "Output Panes", "General Messages")
+        invokeMenuItem("View", "Output", "General Messages")
 
 # function that retrieves a specific child object by its class
 # this is sometimes the best way to avoid using waitForObject() on objects that
@@ -676,3 +606,19 @@ def getHelpViewer():
 
 def getHelpTitle():
     return str(getHelpViewer().title())
+
+
+def isString(sth):
+    if sys.version_info.major > 2:
+        return isinstance(sth, str)
+    else:
+        return isinstance(sth, (str, unicode))
+
+# helper function to ensure we get str, converts bytes if necessary
+def stringify(obj):
+    stringTypes = (str, unicode) if sys.version_info.major == 2 else (str)
+    if isinstance(obj, stringTypes):
+        return obj
+    if isinstance(obj, bytes):
+        tmp = obj.decode('cp1252') if platform.system() in ('Microsoft','Windows') else obj.decode()
+        return tmp
