@@ -1,55 +1,59 @@
-#############################################################################
-##
-## Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-## Contact: http://www.qt-project.org/legal
-##
-## This file is part of Qt Creator.
-##
-## Commercial License Usage
-## Licensees holding valid commercial Qt licenses may use this file in
-## accordance with the commercial license agreement provided with the
-## Software or, alternatively, in accordance with the terms contained in
-## a written agreement between you and Digia.  For licensing terms and
-## conditions see http://www.qt.io/licensing.  For further information
-## use the contact form at http://www.qt.io/contact-us.
-##
-## GNU Lesser General Public License Usage
-## Alternatively, this file may be used under the terms of the GNU Lesser
-## General Public License version 2.1 or version 3 as published by the Free
-## Software Foundation and appearing in the file LICENSE.LGPLv21 and
-## LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-## following information to ensure the GNU Lesser General Public License
-## requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-# http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-##
-## In addition, as a special exception, Digia gives you certain additional
-## rights.  These rights are described in the Digia Qt LGPL Exception
-## version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
-##
-#############################################################################
+############################################################################
+#
+# Copyright (C) 2016 The Qt Company Ltd.
+# Contact: https://www.qt.io/licensing/
+#
+# This file is part of Qt Creator.
+#
+# Commercial License Usage
+# Licensees holding valid commercial Qt licenses may use this file in
+# accordance with the commercial license agreement provided with the
+# Software or, alternatively, in accordance with the terms contained in
+# a written agreement between you and The Qt Company. For licensing terms
+# and conditions see https://www.qt.io/terms-conditions. For further
+# information use the contact form at https://www.qt.io/contact-us.
+#
+# GNU General Public License Usage
+# Alternatively, this file may be used under the terms of the GNU
+# General Public License version 3 as published by the Free Software
+# Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+# included in the packaging of this file. Please review the following
+# information to ensure the GNU General Public License requirements will
+# be met: https://www.gnu.org/licenses/gpl-3.0.html.
+#
+############################################################################
 
 source("../../shared/qtcreator.py")
 
+def __highlighterDefinitionsDirectory__():
+    if platform.system() in ('Microsoft', 'Windows'):
+        basePath = os.path.expandvars("%LOCALAPPDATA%")
+    elif platform.system() == 'Linux':
+        basePath = os.path.expanduser("~/.local/share")
+    else:  # macOS
+        basePath = os.path.expanduser("~/Library/Application Support")
+    return os.path.join(basePath, "org.kde.syntax-highlighting", "syntax")
+
 def createFile(folder, filename):
-    __createProjectOrFileSelectType__("  General", "Text File", isProject = False)
+    __createProjectOrFileSelectType__("  General", "Empty File", isProject = False)
     replaceEditorContent(waitForObject("{name='nameLineEdit' visible='1' "
                                        "type='Utils::FileNameValidatingLineEdit'}"), filename)
     replaceEditorContent(waitForObject("{type='Utils::FancyLineEdit' unnamed='1' visible='1' "
-                                       "window=':New Text File_Core::BaseFileWizard'}"), folder)
+                                       "window={type='ProjectExplorer::JsonWizard' unnamed='1' "
+                                       "visible='1'}}"), folder)
     clickButton(waitForObject(":Next_QPushButton"))
     __createProjectHandleLastPage__()
 
 def clickTableGetPatternLineEdit(table, row):
-    clickItem(table, row, 5, 5, 0, Qt.LeftButton)
+    mouseClick(waitForObjectItem(table, row))
     return waitForObject("{name='patternsLineEdit' type='QLineEdit' visible='1'}")
 
 def getOrModifyFilePatternsFor(mimeType, filter='', toBePresent=None):
     toSuffixArray = lambda x : [pat.replace("*", "") for pat in x.split(";")]
 
     result = []
-    invokeMenuItem("Tools", "Options...")
-    waitForObjectItem(":Options_QListView", "Environment")
-    clickItem(":Options_QListView", "Environment", 14, 15, 0, Qt.LeftButton)
+    invokeMenuItem("Edit", "Preferences...")
+    mouseClick(waitForObjectItem(":Options_QListView", "Environment"))
     waitForObject("{container=':Options.qt_tabwidget_tabbar_QTabBar' type='TabItem' "
                   "text='MIME Types'}")
     clickOnTab(":Options.qt_tabwidget_tabbar_QTabBar", "MIME Types")
@@ -96,6 +100,8 @@ def getOrModifyFilePatternsFor(mimeType, filter='', toBePresent=None):
                     result = toSuffixArray(patterns)
                     test.passes("Added suffixes")
                 return result
+            else:
+                result = toSuffixArray(patterns)
         else:
             result = toSuffixArray(patterns)
     elif model.rowCount() > 1:
@@ -105,53 +111,33 @@ def getOrModifyFilePatternsFor(mimeType, filter='', toBePresent=None):
     clickButton(":Options.Cancel_QPushButton")
     return result
 
-def uncheckGenericHighlighterFallback():
-    invokeMenuItem("Tools", "Options...")
-    waitForObjectItem(":Options_QListView", "Text Editor")
-    clickItem(":Options_QListView", "Text Editor", 14, 15, 0, Qt.LeftButton)
+def addHighlighterDefinition(*languages):
+    syntaxDirectory = __highlighterDefinitionsDirectory__()
+    toBeChecked = (os.path.join(syntaxDirectory, x + ".xml") for x in languages)
+    test.log("Updating highlighter definitions...")
+    invokeMenuItem("Edit", "Preferences...")
+    mouseClick(waitForObjectItem(":Options_QListView", "Text Editor"))
     waitForObject("{container=':Options.qt_tabwidget_tabbar_QTabBar' type='TabItem' "
                   "text='Generic Highlighter'}")
     clickOnTab(":Options.qt_tabwidget_tabbar_QTabBar", "Generic Highlighter")
-    ensureChecked("{name='useFallbackLocation' text='Use fallback location' type='QCheckBox' "
-                  "visible='1'}", False)
-    clickButton(":Options.OK_QPushButton")
 
-def addHighlighterDefinition(language):
-    global tmpSettingsDir
-    test.log("Adding highlighter definitions for '%s'." % language)
-    invokeMenuItem("Tools", "Options...")
-    waitForObjectItem(":Options_QListView", "Text Editor")
-    clickItem(":Options_QListView", "Text Editor", 14, 15, 0, Qt.LeftButton)
-    waitForObject("{container=':Options.qt_tabwidget_tabbar_QTabBar' type='TabItem' "
-                  "text='Generic Highlighter'}")
-    clickOnTab(":Options.qt_tabwidget_tabbar_QTabBar", "Generic Highlighter")
-    clickButton("{text='Download Definitions...' type='QPushButton' unnamed='1' visible='1'}")
-    table = waitForObject("{name='definitionsTable' type='QTableWidget' visible='1'}")
-    model = table.model()
-    for row in range(model.rowCount()):
-        if str(model.data(model.index(row, 0)).toString()) == language:
-            clickItem(table, "%d/0" % row, 5, 5, 0, Qt.LeftButton)
-            clickButton("{name='downloadButton' text='Download Selected Definitions' "
-                        "type='QPushButton' visible='1'}")
-            # downloading happens asynchronously
-            languageFile = os.path.join(tmpSettingsDir, "QtProject", "qtcreator",
-                                        "generic-highlighter", "%s.xml" % language.lower())
-            test.verify(waitFor("os.path.exists(languageFile)", 10000),
-                        "Verifying whether file has been downloaded and placed to settings.")
-            clickButton("{text='Download Definitions...' type='QPushButton' unnamed='1' "
-                        "visible='1'}")
-            table = waitForObject("{name='definitionsTable' type='QTableWidget' visible='1'}")
-            model = table.model()
-            test.verify(str(model.data(model.index(row, 1))) != "",
-                        "Verifying a definition has been downloaded.")
-            clickButton("{text='Close' type='QPushButton' unnamed='1' visible='1'}")
-            clickButton(":Options.OK_QPushButton")
-            return True
-    test.fail("Could not find the specified language (%s) to download a highlighter definition"
-              % language)
-    clickButton("{text='Close' type='QPushButton' unnamed='1' visible='1'}")
-    clickButton(":Options.OK_QPushButton")
-    return False
+    clickButton("{text='Download Definitions' type='QPushButton' name='downloadDefinitions' visible='1'}")
+    updateStatus = "{name='updateStatus' type='QLabel' visible='1'}"
+    waitFor("object.exists(updateStatus)", 5000)
+    if waitFor('str(findObject(updateStatus).text) == "Download finished"', 5000):
+        test.verify(os.path.exists(syntaxDirectory),
+                    "Directory for syntax highlighter files exists.")
+        xmlFiles = glob.glob(os.path.join(syntaxDirectory, "*.xml"))
+        test.verify(len(xmlFiles) > 0, "Verified presence of syntax highlighter files. "
+                    "(Found %d)" % len(xmlFiles))
+        # should we check output (General Messages) as well?
+        test.passes("Updated definitions")
+        clickButton(":Options.OK_QPushButton")
+        return map(os.path.exists, toBeChecked)
+    else:
+        test.fail("Could not update highlighter definitions")
+        clickButton(":Options.Cancel_QPushButton")
+        return map(os.path.exists, toBeChecked)
 
 def hasSuffix(fileName, suffixPatterns):
     for suffix in suffixPatterns:
@@ -159,13 +145,25 @@ def hasSuffix(fileName, suffixPatterns):
             return True
     return False
 
+def displayHintForHighlighterDefinition(fileName, patterns, lPatterns, added, addedLiterate):
+    if hasSuffix(fileName, patterns):
+        return not added
+    if hasSuffix(fileName, lPatterns):
+        return not addedLiterate
+    test.warning("Got an unexpected suffix.", "Filename: %s, Patterns: %s"
+                 % (fileName, str(patterns + lPatterns)))
+    return False
+
 def main():
-    miss = "A highlight definition was not found for this file. Would you like to try to find one?"
-    startApplication("qtcreator" + SettingsPath)
+    miss = ("A highlight definition was not found for this file. Would you like to download "
+            "additional highlight definition files?")
+    startQC()
     if not startedWithoutPluginError():
         return
-    uncheckGenericHighlighterFallback()
-    patterns = getOrModifyFilePatternsFor("text/x-haskell", "haskell")
+
+    patterns = getOrModifyFilePatternsFor("text/x-haskell", "x-haskell")
+    lPatterns = getOrModifyFilePatternsFor("text/x-literate-haskell", "literate-haskell")
+
     folder = tempDir()
     filesToTest = ["Main.lhs", "Main.hs"]
     code = ['module Main where', '', 'main :: IO ()', '', 'main = putStrLn "Hello World!"']
@@ -173,7 +171,10 @@ def main():
     for current in filesToTest:
         createFile(folder, current)
         editor = getEditorForFileSuffix(current)
-        expectHint = hasSuffix(current, patterns)
+        if editor == None:
+            earlyExit("Something's really wrong! (did the UI change?)")
+            return
+        expectHint = hasSuffix(current, patterns) or hasSuffix(current, lPatterns)
         mssg = "Verifying whether hint for missing highlight definition is present. (expected: %s)"
         try:
             waitForObject("{text='%s' type='QLabel' unnamed='1' visible='1' "
@@ -189,24 +190,27 @@ def main():
 
     invokeMenuItem("File", "Save All")
     invokeMenuItem("File", "Close All")
-    addedHighlighterDefinition = addHighlighterDefinition("Haskell")
-    patterns = getOrModifyFilePatternsFor('text/x-haskell', 'haskell', ['.lhs', '.hs'])
+    addedHaskell, addedLiterateHaskell = addHighlighterDefinition("haskell", "literate-haskell")
+    patterns = getOrModifyFilePatternsFor('text/x-haskell', 'x-haskell', ['.hs'])
+    lPatterns = getOrModifyFilePatternsFor('text/x-literate-haskell', 'literate-haskell', ['.lhs'])
 
     home = os.path.expanduser("~")
     for current in filesToTest:
         recentFile = os.path.join(folder, current)
         if recentFile.startswith(home) and platform.system() in ('Linux', 'Darwin'):
             recentFile = recentFile.replace(home, "~", 1)
-        invokeMenuItem("File", "Recent Files", recentFile)
+        invokeMenuItem("File", "Recent Files", "%d | " + recentFile)
         editor = getEditorForFileSuffix(current)
+        display = displayHintForHighlighterDefinition(current, patterns, lPatterns,
+                                                      addedHaskell, addedLiterateHaskell)
         try:
             waitForObject("{text='%s' type='QLabel' unnamed='1' visible='1' "
                           "window=':Qt Creator_Core::Internal::MainWindow'}" % miss, 2000)
-            test.verify(not addedHighlighterDefinition and hasSuffix(current, patterns),
-                        "Hint for missing highlight definition was present.")
+            test.verify(display, "Hint for missing highlight definition was present "
+                        "- current file: %s" % current)
         except:
-            test.verify(addedHighlighterDefinition or not hasSuffix(current, patterns),
-                        "Hint for missing highlight definition is not shown.")
+            test.verify(not display, "Hint for missing highlight definition is not shown "
+                        "- current file: %s" % current)
         placeCursorToLine(editor, '.*%s' % code[-1], True)
         for _ in range(23):
             type(editor, "<Left>")
@@ -217,3 +221,10 @@ def main():
 
     invokeMenuItem("File", "Save All")
     invokeMenuItem("File", "Exit")
+
+def init():
+    syntaxDirectory = __highlighterDefinitionsDirectory__()
+    if not os.path.exists(syntaxDirectory):
+        return
+    test.log("Removing existing highlighter definitions folder")
+    deleteDirIfExists(syntaxDirectory)
